package seu.agent.BasicTask;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.PriorityQueue;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.SEUPoliceForce;
import seu.message.Type.ClearPathReason;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.PathBlockState;
import seu.object.Road.RoadUtilities;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

public class AssignAgentTask extends BasicTask<SEUPoliceForce> {//yuan

	private final Comparator<ClearPath> comparator;
	private final PriorityQueue<ClearPath> clearPathTaskQueue;
	private ClearPath currentClearPathTask;

	public AssignAgentTask(SEUPoliceForce agent, int maxDistance) {
		super(agent);
		comparator = new ClearPathTaskComparator();
		clearPathTaskQueue = new PriorityQueue<ClearPath>(10, comparator);
		addClearPathToFiredBuildingTask();
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		//System.out.println(agent.getTimeStep()+" time should AssignAgentTask " );
		if (agent.taskDone == true) {
			return false;
		}
		if (agent.taskDone == false && agent.targetBuilding != null) {
			if (agent.location().getID().getValue() == agent.targetBuilding
					.getValue()) {
				// System.out.println(agent.getID() + "finish1----------in"
				// + agent.getTimeStep());
				agent.targetBuilding = null;
				agent.taskDone = true;
				return false;
			}
		}
		// if (agent.targetBuilding == null) {
		// // System.out.println(agent.getID() + "finish2----------in"
		// // + agent.getTimeStep());
		//
		// return false;
		// }
		updateCurrentClearPathTask();
		if(currentClearPathTask!=null){
			state=State.ClearingPath;
		}else{
			state=State.Inactive;
		}
		if (state == State.ClearingPath) {
			return true;
		}else {
			state = State.Inactive;
			return false;
		}
	}

	@Override
	public void execute() {
//		if(agent.getID().getValue()==327290976)
//			System.out.println(agent.getID()+"   "+agent.getTimeStep()+"   assignagenttask   execute");
		if (state == State.ClearingPath) {
			//agent.sendClear(target.getID());		
			agent.sendClearPath(currentClearPathTask);//niu
			// System.out.println(agent.getTimeStep()+" time "+agent.getID()+" AssignAgentTask "+" Clear"+target.getID());
			 
		}
	}

	private void updateCurrentClearPathTask() {
		if (currentClearPathTask == null) {
			if (agent.targetBuilding != null) {
				addClearPathToFiredBuildingTask();
			}
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			}
		} else if (isOnGoalLocation() && currentClearPathTask.isCompleted()) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			} else {
				currentClearPathTask = null;
			}
		} 
	}

	
	private boolean isOnGoalLocation() {
		return agent.location().getID().getValue() == currentClearPathTask
				.getRoadIdFinish().getValue();
	}

	protected void addClearPathToFiredBuildingTask() {
		// Refuge responsibleRefuge = getResponsibleRefuge();
		// if (responsibleRefuge != null) {
		// ClearPath clearRefugePathTask = new ClearPath(responsibleRefuge
		// .getID(), agent.location().getID(),
		// ClearPathReason.RefugeIsBlocked);
		// addClearPathTask(clearRefugePathTask);
		// }
		if (agent.targetBuilding != null) {
			ClearPath clearRefugePathTask = new ClearPath(agent.location().getID(), agent.targetBuilding,
					ClearPathReason.RefugeIsBlocked);
			addClearPathTask(clearRefugePathTask);
		}
	}

	public void addClearPathTask(ClearPath clearPathTask) {
		if (!clearPathTaskQueue.contains(clearPathTask)) {
			clearPathTaskQueue.add(clearPathTask);
		}
	}
	
	protected Refuge getResponsibleRefuge() {
		List<Refuge> allrefuges = agent.getAllRefuges();
		for (Refuge refuge : allrefuges) {
			if (agent.isInMyPartition(refuge)
					|| agent.isInNeighborPartition(refuge)) {
				return refuge;
			}
		}
		return null;
	}
}

