package seu.agent.BasicTask;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.PriorityQueue;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.SEUPoliceForce;
import seu.message.Type.ClearPathReason;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.PathBlockState;
import seu.object.Road.RoadUtilities;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

public class ClearBlockedRoadToFireTask  extends BasicTask<SEUPoliceForce> {//yuan
	
	private final Comparator<ClearPath> comparator;
	private final PriorityQueue<ClearPath> clearPathTaskQueue;
	private ClearPath currentClearPathTask;
	boolean firstSelect;

	public ClearBlockedRoadToFireTask(SEUPoliceForce agent, int maxDistance) {
		super(agent);
		comparator = new ClearPathTaskComparator();
		clearPathTaskQueue = new PriorityQueue<ClearPath>(10, comparator);
		firstSelect = false;
		assignFire();
		addClearPathToFireTask();
		System.out.println("ClearBlockedRoadTofIRETask" +agent.getTimeStep());
	}

	public void assignFire() {//niu
		HashSet<StandardEntity> temp;
		temp = (HashSet<StandardEntity>) agent.getWorldModel()
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE);
		Iterator<StandardEntity> it = temp.iterator();
		agent.numOfPF = temp.size();
		while (it.hasNext()) {
			StandardEntity t = it.next();
			agent.PF.add(t);
			agent.PFSort.add(t.getID().getValue());
		}
		Collections.sort(agent.PFSort, new IntValueComprator());
		Collection<Building> e = agent.getWorldModel().getEntitiesOfType(Building.class,
				StandardEntityURN.BUILDING);
		//List<Building> result = new ArrayList<Building>();
		for (Building b : e) {
			if (b.isOnFire() && (agent.getWorldModel()
					.getTotalBurningArea(b) > 100 )) {
				temp.add(b);
			}
		}
		//temp = (HashSet<StandardEntity>) agent.getWorldModel()
				//.getEntitiesOfType(StandardEntityURN.GAS_STATION);
		agent.numOfFire = temp.size();
		it = temp.iterator();
		while (it.hasNext()) {
			StandardEntity t = it.next();
			agent.Fire.add(t);
			agent.FireSort.add(t.getID().getValue());
		}
		Collections.sort(agent.PFSort, new IntValueComprator());
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		if (firstSelect == false) {		
			EntityID temp ;
			if(agent.getNearestFire()==null){
				return false;
				}else{
					temp=agent.getNearestFire().getID();	
				}
			Path pathTemp = agent.getSearch().getPath(agent.getMeAsHuman(),
					(Area) agent.getModel().getEntity(temp), PathType.Shortest);
			if (pathTemp.size() < 20) {
				ClearPath clearFirePathTask = new ClearPath (agent.location().getID(),temp,
						ClearPathReason.ExtinguishFire);
				addClearPathTask(clearFirePathTask);
				agent.isSelected = true;
				agent.targetFire = (Building) (Area) agent.getModel()
						.getEntity(temp);
				 //System.err.println(agent.getID() + "add refuge"
				 //+ agent.targetRefuge);
			}
			firstSelect = true;
		}
		updateCurrentClearPathTask();
		if (agent.targetFire == null) {
			return false;
		}
		if (agent.existPO && !agent.isSelected) {
			ClearPath clearFirePathTask = new ClearPath(agent.location().getID(),agent.targetFire.getID(), 
					ClearPathReason.ExtinguishFire);
			agent.isSelected = true;
			// System.out.println(agent.getID().getValue() +
			// "is Responsible for "
			// + agent.targetRefuge.getID().getValue() + "time"
			// + agent.getTimeStep());
			addClearPathTask(clearFirePathTask);
		}
		if (agent.isSelected == false) {
			return false;
		}
		if (agent.isDone) {
			return false;
		}
		if (agent.getMeAsHuman().getPosition().getValue() == agent.targetFire
				.getID().getValue()) {
			agent.isDone = true;
			// System.out.println(agent.getID() + "isDone" +
			// agent.getTimeStep());
			return false;

		}
		
		if(currentClearPathTask!=null){
			state=State.ClearingPath;
			return true;
		}else{
			state=State.Inactive;
			return false;
		}
	}

	@Override
	public void execute() {
		 if (state == State.ClearingPath) {
			//agent.sendClear(target.getID());
			agent.sendClearPath(currentClearPathTask);//niu
			 //System.out.println(agent.getTimeStep()+" time "+agent.getID()+" ClearBlockedRoadToRefugeTask "+" Clear"+target.getID());

		}
	}

	private void updateCurrentClearPathTask() {
		if (currentClearPathTask == null) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			}
		} else if (isOnGoalLocation() && currentClearPathTask.isCompleted()) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			} else {
				currentClearPathTask = null;
			}
		} 
	}

	
	public void addClearPathTask(ClearPath clearPathTask) {
		if (!clearPathTaskQueue.contains(clearPathTask)) {
			clearPathTaskQueue.add(clearPathTask);

		}
	}
	
	private boolean isOnGoalLocation() {
		return agent.location().getID().getValue() == currentClearPathTask
				.getRoadIdFinish().getValue();
	}
	
	protected void addClearPathToFireTask() {
		if (agent.existPO) {
			return;
		}
		if (agent.isSelected == false) {
			
			Collection<Building> e = agent.getWorldModel().getEntitiesOfType(Building.class,
					StandardEntityURN.BUILDING);
			List<Building> Fires = new ArrayList<Building>();
			for (Building b : e) {
				if (b.isOnFire() && ( agent.getWorldModel()
						.getTotalBurningArea(b) > 100 || agent.getTimeStep()<50)) {
					Fires.add(b);
				}
			}
			if (Fires == null) {

				//System.err.println(agent.getMeAsHuman().getID() + "No GasStation"
				//		+ " time" + agent.getTimeStep());
				return;
			}
			for (Building next : Fires) {

				if (agent.FireSort.indexOf(next.getID().getValue()) == agent.PFSort
						.indexOf((agent.getID().getValue()))) {
					ClearPath clearFirePathTask = new ClearPath(agent.location().getID(),next.getID(),
							ClearPathReason.ExtinguishFire);
					agent.targetFire = next;
					agent.isSelected = true;
					// System.out
					// .println(agent.getID().getValue()
					// + "  is Responsible for "
					// + next.getID().getValue());
					addClearPathTask(clearFirePathTask);
					break;
				}
			}

		}
	}

	public class IntValueComprator implements Comparator<Object> {

		@Override
		public int compare(Object arg0, Object arg1) {

			int first = ((Integer) arg0).intValue();
			int second = ((Integer) arg0).intValue();

			if (first > second) {
				return 0;
			} else {
				return 1;
			}
		}
	}
	
}

