package seu.agent.BasicTask;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.PriorityQueue;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.GasStation;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.SEUPoliceForce;
import seu.message.Type.ClearPathReason;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.PathBlockState;
import seu.object.Road.RoadUtilities;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

public class ClearBlockedRoadToGasStationTask extends BasicTask<SEUPoliceForce> {//yuan
	
	private final Comparator<ClearPath> comparator;
	private final PriorityQueue<ClearPath> clearPathTaskQueue;
	private ClearPath currentClearPathTask;
	boolean firstSelect;

	public ClearBlockedRoadToGasStationTask(SEUPoliceForce agent, int maxDistance) {
		super(agent);
		comparator = new ClearPathTaskComparator();
		clearPathTaskQueue = new PriorityQueue<ClearPath>(10, comparator);
		//clearPathTaskQueue = new PriorityQueue<ClearPath>(10);
		firstSelect = false;
				//assignRefuge();
		 //System.out.println(agent.getTimeStep()+ "            "+agent.getMeAsHuman().getID()+"              4");

				addClearPathToGasStationTask();
		//System.out.println("ClearBlockedRoadToGasStationTask" +agent.getTimeStep());
	}

	public void assignGasStation() {//niu
		HashSet<StandardEntity> temp;
		temp = (HashSet<StandardEntity>) agent.getWorldModel()
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE);
		Iterator<StandardEntity> it = temp.iterator();
		agent.numOfPF = temp.size();
		while (it.hasNext()) {
			StandardEntity t = it.next();
			agent.PF.add(t);
			agent.PFSort.add(t.getID().getValue());
		}
		Collections.sort(agent.PFSort, new IntValueComprator());
		temp = (HashSet<StandardEntity>) agent.getWorldModel()
				.getEntitiesOfType(StandardEntityURN.GAS_STATION);
		agent.numOfGasStation = temp.size();
		it = temp.iterator();
		while (it.hasNext()) {
			StandardEntity t = it.next();
			agent.gasStation.add(t);
			agent.gasStationSort.add(t.getID().getValue());
		}
		Collections.sort(agent.gasStationSort, new IntValueComprator());   //yuan7.9
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		updateCurrentClearPathTask();
		if(currentClearPathTask!=null){
			state=State.ClearingPath;
			//System.out.println(agent.getTimeStep()+"  ,"+agent.getMeAsHuman().getID() +"  ,"+currentClearPathTask.getRoadIdFinish()+"   ClearBlockedRoadToGSTask");
			return true;
		}else{
			state=State.Inactive;
			return false;
		}		
				
	}

	@Override
	public void execute() {
		if ( state == State.ClearingPath) {
			//agent.sendClear(target.getID());
			if(agent.getID().getValue()==327290976)
			 System.out.println(agent.getTimeStep()+" time "+agent.getID()+" ClearBlockedRoadToGSTask ");
			agent.sendClearPath(currentClearPathTask);//niu
			

		}
	}

	private void updateCurrentClearPathTask() {
		if (currentClearPathTask == null) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			}
		} else if (isOnGoalLocation() && currentClearPathTask.isCompleted()) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			} else {
				currentClearPathTask = null;
			}
		}
	}

	
	public void addClearPathTask(ClearPath clearPathTask) {
		if (!clearPathTaskQueue.contains(clearPathTask)) {
			clearPathTaskQueue.add(clearPathTask);

		}
	}

	private boolean isOnGoalLocation() {
		return agent.location().getID().getValue() == currentClearPathTask
				.getRoadIdFinish().getValue();
	}

	protected void addClearPathToGasStationTask() {
//		if (!agent.existPO) {
//			return;
//		}

		 List<GasStation> gasstations = agent.getAllGasStations();
		 List<GasStation> gs=new ArrayList<GasStation>();
		 for (GasStation next : gasstations) {
			 if(agent.isInMyPartition(next)){
				 gs.add(next);
			 } 
		 }

		if (firstSelect == false) {      //wujei  7.9
			
			if (gasstations!= null&&!gasstations.isEmpty()){
				
				for (GasStation next : gasstations) {
					
					 if(agent.isInMyPartition(next)){
//							
						
						 ClearPath clearGasStationPathTask = new ClearPath(agent.location().getID(),next.getID(),
									ClearPathReason.GasStationIsBlocked);
						agent.targetGasStation=next;
						//agent.targetRefuge = next;
											
						addClearPathTask(clearGasStationPathTask);
						
						//break;
					}		 
				}						
			}			
		}
		firstSelect = true;	}

	public class IntValueComprator implements Comparator<Object> {

		@Override
		public int compare(Object arg0, Object arg1) {

			int first = ((Integer) arg0).intValue();
			int second = ((Integer) arg0).intValue();

			if (first > second) {
				return 0;
			} else {
				return 1;
			}
		}
	}
}

