package seu.agent.BasicTask;

import java.awt.Point;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.PriorityQueue;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.FunctionType;
import seu.agent.Police.SEUPoliceForce;
import seu.message.Type.ClearPathReason;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.PathBlockState;
import seu.object.Road.RoadUtilities;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

public class ClearMainRoadTask extends BasicTask<SEUPoliceForce> {//yuan
	private final Comparator<ClearPath> comparator;
	private final PriorityQueue<ClearPath> clearPathTaskQueue;
	private ClearPath currentClearPathTask;
	private final Path targetPath;

	public ClearMainRoadTask(SEUPoliceForce agent, int maxDistance, Path p) {
		super(agent);
		comparator = new ClearPathTaskComparator();
		clearPathTaskQueue = new PriorityQueue<ClearPath>(10, comparator);
		targetPath = p;
		// assignRefuge();
		addClearPathToRefugeTask();
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
		//System.out.println(agent.getTimeStep()+" time "+agent.getID()+" ClearMainRoadTask "+" Clear DOWN");
//		if (agent.getFunctionType() == FunctionType.connectQuartations) {
//			agent.setFunctionType(FunctionType.dynamicQuartations);
//		}
	}

	@Override
	public boolean shouldExecute() {
		updateCurrentClearPathTask();	
		if(currentClearPathTask!=null){
			state=State.ClearingPath;
		}else{
			state=State.Inactive;
		}
		if (state == State.ClearingPath) {
			return true;
		}else {
			state = State.Inactive;
			return false;
		}
	}

	@Override
	public void execute() {
		if(state == State.ClearingPath) {
			//agent.sendClear(target.getID());
			agent.sendClearPath(currentClearPathTask);//niu
//			if(agent.getID().getValue()==327290976)
//			System.out.println(agent.getTimeStep()+" time "+agent.getID()+" ClearMainRoadTask "+" Clear");
		}
	}

	private void updateCurrentClearPathTask() {
		if (currentClearPathTask == null) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			}
		} else if (isOnGoalLocation() && currentClearPathTask.isCompleted()) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			} else {
				currentClearPathTask = null;
			}
		}
	}

	

	public void addClearPathTask(ClearPath clearPathTask) {
		if (!clearPathTaskQueue.contains(clearPathTask)) {
			clearPathTaskQueue.add(clearPathTask);
		}
	}

	
	private boolean isOnGoalLocation() {
		return agent.location().getID().getValue() == currentClearPathTask
				.getRoadIdFinish().getValue();
	}

	protected void addClearPathToRefugeTask() {
		ClearPath clearRefugePathTask = new ClearPath(targetPath.getStart(),
				targetPath.getEntities().get(targetPath.size() - 1),
				ClearPathReason.RefugeIsBlocked);
		// System.out
		// .println(agent.getID().getValue()
		// + "  is Responsible for "
		// + next.getID().getValue());
		addClearPathTask(clearRefugePathTask);

	}

	public class IntValueComprator implements Comparator<Object> {

		@Override
		public int compare(Object arg0, Object arg1) {

			int first = ((Integer) arg0).intValue();
			int second = ((Integer) arg0).intValue();

			if (first > second) {
				return 0;
			} else {
				return 1;
			}
		}
	}
}

