package seu.agent.BasicTask;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.PriorityQueue;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.SEUPoliceForce;
import seu.message.MessagePriority;
import seu.message.Type.ClearPathReason;
import seu.message.Type.RoadIsClearedMessage;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.PathBlockState;
import seu.object.Road.RoadUtilities;
import seu.utilities.Path;
import seu.utilities.Geometry.DistanceUtilities;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.Cost.PathType;

public class ClearPathTask extends BasicTask<SEUPoliceForce> {//yuan

	private final Comparator<ClearPath> comparator;
	private final PriorityQueue<ClearPath> clearPathTaskQueue;
	private ClearPath currentClearPathTask;
    private boolean isAddRefuge;
	
	public ClearPathTask(SEUPoliceForce agent, int maxDistance,
			boolean isAddRefuge) {
		super(agent);
		comparator = new ClearPathTaskComparator();
		clearPathTaskQueue = new PriorityQueue<ClearPath>(10, comparator);
		this.isAddRefuge=isAddRefuge;
		if (isAddRefuge) {
			addClearPathToRefugeTask();
		}
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
 
		updateCurrentClearPathTask();
		if(currentClearPathTask==null){
			state=State.Inactive;
			return false;
			}
		EntityID start=currentClearPathTask.getRoadIdStart();
	    EntityID finish=currentClearPathTask.getRoadIdFinish();
	    StandardEntity s=agent.getModel().getEntity(start);
	    StandardEntity f=agent.getModel().getEntity(finish);
	    if(!agent.isInMyPartition(s)&&!agent.isInNeighborPartition(s)
	    		&&!agent.isInMyPartition(f)&&!agent.isInNeighborPartition(f)){
	    	return false;
	    }else{
			state = State.ClearingPath;
			return true;
		}
	}
	@Override
	public void execute() {//yuan		
//		if(agent.getID().getValue()==327290976)
//			System.out.println(agent.getID()+"   "+agent.getTimeStep()+"    clearpathtask execute");
		if ( state == State.ClearingPath) {
			agent.sendClearPath(currentClearPathTask);
			if(agent.getID().getValue()==327290976)
			System.out.println(agent.getTimeStep()+" time "+agent.getID()+" ClearPathTask "+" Clear");
		}
	}

	private void updateCurrentClearPathTask() {		
		if (currentClearPathTask == null) {
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			}
		} else if (isOnGoalLocation() && currentClearPathTask.isCompleted()) {//yuan
			RoadIsClearedMessage message = new RoadIsClearedMessage(
					currentClearPathTask.roadIdStart);
			agent.sendMessage(message, MessagePriority.Medium);
			// System.out.println(agent.getTimeStep() + "---" + agent.getID()
			// + "Send" + message);
			if (!clearPathTaskQueue.isEmpty()) {
				currentClearPathTask = clearPathTaskQueue.poll();
			} else {
				currentClearPathTask = null;
			}
		}
}


	
	public void addClearPathTask(ClearPath clearPathTask) {
		if (!clearPathTaskQueue.contains(clearPathTask)) {
			clearPathTaskQueue.add(clearPathTask);
		}
	}

	protected void addClearPathToRefugeTask() {  //refugeÎªstart£¬meÎªfinish
		Refuge responsibleRefuge = getResponsibleRefuge();
		if (responsibleRefuge != null) {
			ClearPath clearRefugePathTask = new ClearPath(
					 agent.location().getID(),responsibleRefuge.getID(),  //yuan
					ClearPathReason.RefugeIsBlocked);
			addClearPathTask(clearRefugePathTask);
		}
	}

	protected Refuge getResponsibleRefuge() {//±ŸÇø»òÏàÁÚÇø
		List<Refuge> allrefuges = agent.getAllRefuges();
		for (Refuge refuge : allrefuges) {
			if (agent.isInMyPartition(refuge)
					|| agent.isInNeighborPartition(refuge)) {
				return refuge;
			}
		}
		return null;
	}
	
	private boolean isOnGoalLocation() {
		return agent.location().getID().getValue() == currentClearPathTask
				.getRoadIdFinish().getValue();
	}

	public PriorityQueue<ClearPath> getClearPathQueue() {
		return clearPathTaskQueue;
	}
	
	@Override
	public String toString() {
		return super.toString() + "ClearPathTask [currentClearPathTask="
				+ currentClearPathTask + "]";
	}
	
	public void updateQueue(){      //yuan
		System.out.println(agent.getTimeStep()+"  "+agent.getID()+"  updateQueue1");
		int j=clearPathTaskQueue.size();
		if(j==0) return;		
		List<ClearPath> tempClearPath=new ArrayList<ClearPath>();
		ClearPath path1,targetClearPath=null;	
		for(int i=0;i<j;i++){
			path1=clearPathTaskQueue.poll();			
			if(path1.getRoadIdStart()==agent.getTarget().getID()){
				targetClearPath=path1;
			}else{
				tempClearPath.add(path1);
			}
		}
		if(targetClearPath==null){
			for(int i=j-1;i>=0;i--){
				clearPathTaskQueue.add(tempClearPath.get(i));
			}
		}else{
			for(int i=j-2;i>=0;i--){
				clearPathTaskQueue.add(tempClearPath.get(i));
			}
			System.out.println(agent.getTimeStep()+"  "+agent.getID()+"  updateQueue2");
			clearPathTaskQueue.add(targetClearPath);
		}		
	}

}
