package seu.agent.BasicTask;

import java.util.Collections;
import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import seu.agent.DecisionManager;
import seu.agent.SEUAgent;
import seu.agent.Police.SEUPoliceForce;
import seu.message.CommunicationCondition;
import seu.message.MessagePriority;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.object.Exploration;
import seu.utilities.DistanceComparator;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class ExplorationTask extends
		BasicTask<SEUAgent<? extends StandardEntity>> {

	private Exploration<Building> exploration;
	Path path;

	public ExplorationTask(SEUAgent<? extends StandardEntity> agent) {
		super(agent);
	}

	@Override
	public boolean shouldExecute() {
		agent.sendGetTaskAssigmentMessage();
		if (
			state == State.MovingToGasStation||state == State.MovingToImporBuild ){//niu
			return false;
		}
		if ((state == State.Inactive || state == State.Exploring)
				&& !isExplorationCompeted()) {
			exploration = agent.getExploration();
			// List<Building> buildings = exploration.getUnexploredEntities();
			// List<Building> safeBuildings = buildings;
			List<Building> buildings = null;
			// agent.nofoundbuilding;
			if (!(agent.getCommCondition() == CommunicationCondition.NoCommunication || agent
					.getCommCondition() == CommunicationCondition.Limited)) {
				if (agent.getMeAsHuman().getStandardURN() == StandardEntityURN.FIRE_BRIGADE
						|| agent.getMeAsHuman().getStandardURN() == StandardEntityURN.AMBULANCE_TEAM) {
					buildings = agent.maybebu ;// getAgentTasks();
					// buildings.addAll(agent.nofoundbuilding);
					if (buildings.size() == 0) {
						buildings.addAll(agent.nofoundbuilding);
					}

				} else {
					buildings = agent.nofoundbuilding;

				}
				Collections.sort(
						buildings,
						new DistanceComparator(agent.location(), agent
								.getModel()));
			} else {
				buildings = exploration.getUnexploredEntities();

			}

			// new
			// BuildingDistanceComparator(agent,
			// agent.getModel());
			// new DistanceComparator(agent.getLocation(agent.getTimeStep()),
			// agent.getModel()));

			List<Building> safeBuildings = DecisionManager
					.ignoreBurningBuildings(buildings);
			if (!(agent instanceof SEUPoliceForce)) {
				safeBuildings.removeAll(agent.getTravelMemory()
						.getUnreachableLocations());
			} else {
				safeBuildings = DecisionManager
						.ignoreBurnedBuildings(safeBuildings);
			}
			if (safeBuildings.isEmpty()) {
				return false;
			}
			PathType pt;
			if (agent instanceof SEUPoliceForce) {
				pt = PathType.Shortest;
			} else {
				pt = PathType.EmptyAndSafe;
			}
			Path path = agent.getSearch().getPath(agent.getMeAsHuman(),
					safeBuildings, pt);

			if (path == null) {

				agent.getTravelMemory().addUnreachableLocations(safeBuildings,
						agent.getTimeStep());
				return false;
			} else if (path.hasInfiniteCost()) {
				path = agent.getSearch().getPath(agent.getMeAsHuman(),
						safeBuildings, PathType.Shortest);
				if (path != null && path.isPassable()) {
					ClearPathIsNeededMessage message = new ClearPathIsNeededMessage(
							path.getStart(), path.getDestination(),
							ClearPathReason.SaveHuman);
					agent.sendMessage(message, MessagePriority.Low);
				}
				return false;
			} else if (path.isPassable()) {
				this.path = path;
				state = State.Exploring;
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	@Override
	public void execute() {
		agent.sendMove(path);
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
		path = null;
	}

	private boolean isExplorationCompeted() {
		// exploration = agent.getExploration();
		// return exploration.isExplorationCompeted();
		if (agent.nofoundbuilding.size() == 0)
			return true;
		return false;
	}
}

