package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Road;
import seu.agent.Firebrigade.SEUFireBrigade;
import seu.message.MessagePriority;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.object.Building.BurningBuildingComparator;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class ExtinguishDangerousBuildingTask extends BasicTask<SEUFireBrigade> {//刚发热的房屋

	Building building;
	Building lastHeatingBuilding;
	int lastTimeStep;
	Path path;

	public ExtinguishDangerousBuildingTask(SEUFireBrigade agent) {
		super(agent);
		lastHeatingBuilding = null;
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		SEUFireBrigade fb;
		if(state==State.MovingToGasStation||state==State.MovingToImporBuild){//niu
			return false;
		}
		if (lastHeatingBuilding != null) {
			if (agent.getTimeStep() - lastTimeStep > 1) {
				lastHeatingBuilding = null;
			} else {
				path = agent.getSearch().getPath(agent.getMeAsHuman(),
						lastHeatingBuilding, PathType.EmptyAndSafe);
				if (path == null || path.isEmpty()) {
					List<Building> collBuilding = new ArrayList<Building>();
					collBuilding.add(lastHeatingBuilding);
					agent.getTravelMemory().addUnreachableLocations(
							collBuilding, agent.getTimeStep());
					Road r = agent.getEntranceRoad(lastHeatingBuilding);
					if (r != null) {
						// ClearPathIsNeededMessage message = new
						// ClearPathIsNeededMessage(
						// r.getID(), r.getID(),
						// ClearPathReason.ExploreBuilding);
						// agent.sendMessage(message, MessagePriority.Medium);
						// System.out.println("ClearPathIsNeededMessage2");
					}
					lastHeatingBuilding = null;
					return false;
				} else if (path.hasInfiniteCost()) {
					path = agent.getSearch().getPath(agent.getMeAsHuman(),
							lastHeatingBuilding, PathType.Shortest);
					if (path != null && path.isPassable()) {
						ClearPathIsNeededMessage message = new ClearPathIsNeededMessage(
								path.getStart(), path.getDestination(),
								ClearPathReason.ExtinguishFire);
						agent.sendMessage(message, MessagePriority.Medium);
					}
					lastHeatingBuilding = null;
					return false;
				} else if (path.isPassable()) {
					this.path = path;
					state = State.Exploring;
					lastHeatingBuilding = null;
					return true;
				} else {
					lastHeatingBuilding = null;

					return false;
				}
			}
		}
		Building building;
		fb = getFireAgent();
		building = getNextBuilding();
		if (building != null && fb.canExtinguish(building)) {
			this.building = building;
			state = State.Extinguishing;
			return true;
		}
		return false;
	}

	@Override
	public void execute() {
		if (state == State.Extinguishing) {
			getFireAgent().sendExtinguish(building);
			//System.out.println("extinguish danger"+"id "+agent.getID());
		} else if (state == State.Exploring) {
			//System.out.println("moving 1        !!"+"id "+agent.getID()+"  pathimp    " +path.getDestination());
			agent.sendMove(path);
		}
	}

	public SEUFireBrigade getFireAgent() {
		return agent;
	}

	private Building getNextBuilding() {
		List<Building> heating;
		List<Building> buildings;
		BurningBuildingComparator comperator;

		heating = agent.getHeatingBuildings();
		if (heating.isEmpty()) {
			return null;
		} else {
			buildings = new ArrayList<Building>(heating);
			comperator = new BurningBuildingComparator(getModel(),
					agent.getTimeStep());
			Collections.sort(buildings, comperator);
			lastHeatingBuilding = buildings.get(0);
			lastTimeStep = agent.getTimeStep();
			return buildings.get(0);
		}
	}
}
