package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import seu.agent.DecisionManager;
import seu.agent.SEUAgent;
import seu.agent.Police.SEUPoliceForce;
import seu.object.Exploration;
import seu.object.Building.BuildingInfo;
import seu.utilities.DistanceComparator;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class PFPartitionExplorationTask extends
		BasicTask<SEUAgent<? extends StandardEntity>> {

	private final Exploration<Building> exploration;
	Path path;
	boolean isAddNeighbour;

	public PFPartitionExplorationTask(SEUPoliceForce agent) {
		super(agent);
		exploration = new Exploration<Building>(agent.getModel());
		exploration.addEntitiesToExplore(agent.getBuildingsInPartition());
		Collections.sort(exploration.getUnexploredEntities(),
				new DistanceComparator(agent.getMeAsHuman(), agent.getModel()));
		isAddNeighbour = false;
	}

	@Override
	public boolean shouldExecute() {

		if ((state == State.Inactive || state == State.Exploring)
				&& !isExplorationCompeted()) {
			// exploration = agent.getExploration();
			List<Building> buildings = exploration.getUnexploredEntities();
			if (buildings.isEmpty())
				return false;
			// System.out.println(agent + " before remove" + buildings.size());
			removeExploredEntity(buildings);
			if (buildings.isEmpty())
				return false;
			// System.out.println(agent + " after remove" + buildings.size());
			// List<Building> safeBuildings = buildings;
			List<Building> safeBuildings = DecisionManager
					.ignoreBurningBuildings(buildings);
			safeBuildings = DecisionManager
					.ignoreBurnedBuildings(safeBuildings);
			// safeBuildings.removeAll(agent.getTravelMemory()
			// .getUnreachableLocations());
			if (safeBuildings.isEmpty()) {
				return false;
			}

			Path path = agent.getSearch().getPath(agent.getMeAsHuman(),
					safeBuildings, PathType.Shortest);

			if (path == null) {

				agent.getTravelMemory().addUnreachableLocations(safeBuildings,
						agent.getTimeStep());
				return false;
			} else {
				this.path = path;
				state = State.Exploring;
				return true;
			}
		} else {
			return false;
		}
	}

	private void removeExploredEntity(List<Building> buildings) {
		// TODO Auto-generated method stub
		List<Building> temp = new ArrayList<Building>();
		temp.addAll(buildings);
		// System.out.println(agent + "unexploredBuilidngs"
		// + agent.getUnexploredBuildings().size());
		for (Building b : temp) {
			if (!agent.getExploration().getUnexploredEntities().contains(b)) {
				// System.out.println("remove" + b);
				buildings.remove(b);
				exploration.addToExplorationHistory(b);
			}
			BuildingInfo building = agent.getModel().getBuildingInfo(b.getID());
			if (building.getBuilding().isIgnitionDefined()
					&& building.getBuilding().isFierynessDefined()
					&& !(building.getBuilding().getFierynessEnum() == Fieryness.UNBURNT || (building
							.getBuilding().getFierynessEnum() == Fieryness.WATER_DAMAGE))) {
				buildings.remove(b);
				exploration.addToExplorationHistory(b);
			}
		}
	}

	@Override
	public void execute() {
		agent.sendMove(path);
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
		path = null;
	}

	private boolean isExplorationCompeted() {
		if (exploration.isExplorationCompeted() && isAddNeighbour == false) {
			isAddNeighbour = true;
			exploration.addEntitiesToExplore(agent
					.getBuildingsInNeighbourPartition());
			return false;
		}
		return exploration.isExplorationCompeted();
	}
}
