package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.SEUPoliceForce;
import seu.agent.command.seuCommand;
import seu.agent.command.seuMove;
import seu.agent.command.seuMoveXY;
import seu.agent.command.seuRest;
import seu.utilities.RandomSelector;

/**
 * 
 * @author Ivan
 * 
 */
public class PFStuckInBuildingTask extends AbstractStuckTask {

	private enum pfStuckStep {
		Safe, CleanAround, MovingToEdge, MovingOut, EdgeClean
	}

	private static final int CHECK_STEP = 2;
	private static final int LEAST_MOVE_LEN = 2500;

	private EntityID stuckhere;
	private pfStuckStep stuckState;
	private final int ignoreTimestamp;
	private final StandardWorldModel world;
	private final SEUPoliceForce agent;

	public PFStuckInBuildingTask(SEUPoliceForce agent) {
		super(agent);
		this.agent = agent;
		this.ignoreTimestamp = agent.getStartTime();
		this.world = agent.getModel();
	}

	@Override
	public void takeDone() {

	}

	public boolean hasInsightInRangeBlockade() {
		for (int i = 0; i < agent.getEntityInsight().size(); i++) {
			StandardEntity se = agent.getEntityInsight().get(i);
			if (se instanceof Blockade) {
				Blockade block = (Blockade) se;
				if (agent.isInClearRange(block)) {
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public boolean shouldExecute() {
		int time = agent.getTimeStep();

		if (time < agent.getStartTime() + CHECK_STEP)
			return false;
		boolean flag = false;
		// if (agent.getID().getValue() == 36531
		// || agent.getID().getValue() == 36541)
		// flag = true;
		stuckState = pfStuckStep.Safe;
		if (stuckState == pfStuckStep.Safe) {
			stuckhere = agent.getMeAsHuman().getPosition();
			// if (flag == true) {
			// System.out.println("building s1");
			// }
			if (!isAlwaysMoveOrRest())
				return false;
			// if (flag == true) {
			// // System.out.println("s2");
			// }
			if (!isAlwaysSamePlace())
				return false;
			// if (flag == true) {
			// System.out.println("s3");
			// }
			if (!isNotMoveTooLong())
				return false;
			// if (flag == true) {
			// System.out.println("s4");
			// }
			if (hasInsightInRangeBlockade()) {
				stuckState = pfStuckStep.CleanAround;
				return true;

			} else {
				// if (flag == true) {
				// System.out.println("s5");
				// }
				stuckState = pfStuckStep.MovingToEdge;
				return true;
			}
		} else if (stuckState == pfStuckStep.CleanAround) {
			if (hasInsightInRangeBlockade()) {
				return true;
			} else {
				stuckState = pfStuckStep.MovingToEdge;
				return true;
			}
		} else if (stuckState == pfStuckStep.MovingToEdge
				|| stuckState == pfStuckStep.EdgeClean) {
			if (hasInsightInRangeBlockade()) {
				stuckState = pfStuckStep.EdgeClean;
				return true;
			} else {
				stuckState = pfStuckStep.MovingOut;
				return true;
			}
		} else if (stuckState == pfStuckStep.MovingOut) {
			if (!agent.location().equals(stuckhere)) {
				return false;
			}
			return false;
		} else {
			return false;
		}
	}

	@Override
	public void execute() {
//		if(agent.getID().getValue()==327290976)
//		System.out.println(agent.getTimeStep()+" time "+agent.getID()+" PFStuckInBuildingTask "+" Clear");
		if (stuckState == pfStuckStep.CleanAround
				|| stuckState == pfStuckStep.EdgeClean) {
			Blockade blk = agent.getClosestBlockade();
			if (blk != null) {
				//agent.sendClear(blk.getID());
				agent.sendClearArea(blk.getID());//niu
				 //System.out.println(agent.getTimeStep()+" time "+agent.getID()+" PFStuckInBuildingTask "+" Clear"+blk.getID());

			} else {
				agent.sendMove(agent.getTimeStep(), agent.randomWalk());
			}
			state = State.CleanAllAround;
		} else if (stuckState == pfStuckStep.MovingToEdge) {
			state = State.MoveingToEdge;
			Area here = (Area) world.getEntity(agent.getMeAsHuman()
					.getPosition());
			List<EntityID> path = new ArrayList<EntityID>();
			path.add(here.getID());
			Set<Edge> cad = new HashSet<Edge>();
			for (Edge e : here.getEdges()) {
				if (e.isPassable())
					cad.add(e);
			}
			if (cad.isEmpty()) {
				agent.sendMove(agent.getTimeStep(), agent.randomWalk());
			} else {
				// FIXME don't random
				Random r = new Random();
				Edge pick = RandomSelector.randomSelect(cad, 1, r).iterator()
						.next();
				int x = (pick.getStartX() + pick.getEndX()) / 2;
				int y = (pick.getStartY() + pick.getEndY()) / 2;
				agent.sendMove(this.agent.getTimeStep(), path, x, y);
			}
		} else if (stuckState == pfStuckStep.MovingOut) {
			state = State.MoveOutOfThePlace;
			Blockade blk = agent.getClosestBlockade();
			if (blk != null) {
				//agent.sendClear(blk.getID());
				agent.sendClearArea(blk.getID());//niu
				// System.out.println(agent.getTimeStep()+" time "+agent.getID()+" PFStuckInBuildingTask "+" Clear"+blk.getID());

			} else {
				agent.sendMove(agent.getTimeStep(), agent.randomWalk());

			}
		}
		agent.sendMove(agent.getTimeStep(), agent.randomWalk());

	}

	@Override
	public String toString() {
		return PFStuckInBuildingTask.class.getName();
	}

	private boolean isAlwaysSamePlace() {
		int time = agent.getTimeStep();
		for (int i = 1; i < CHECK_STEP; i++) {
			EntityID info1 = agent.getLocation(time - i);
			EntityID info2 = agent.getLocation(time - i + 1);
			// System.out.print(info1 + " " + info2 + " ");
			if (info1 == null || info2 == null)
				return false;
			if (info1.getValue() != info2.getValue())
				return false;

		}
		return true;
	}

	private boolean isNotMoveTooLong() {
		int time = agent.getTimeStep();
		int totals = 0;
		for (int i = 1; i < CHECK_STEP; i++) {
			EntityID info1 = agent.getLocation(time - i);
			EntityID info2 = agent.getLocation(time - i + 1);
			totals += agent.getModel().manhattanDistance(info1, info2);
		}
		if (totals > LEAST_MOVE_LEN) {
			return false;
		} else {
			return true;
		}
	}

	private boolean isAlwaysMoveOrRest() {
		int time = agent.getTimeStep();
		for (int i = 1; i <= CHECK_STEP; i++) {
			seuCommand ac = this.agent.getCommands().get(time - i);
			if ((ac instanceof seuMove) || (ac instanceof seuMoveXY)
					|| (ac instanceof seuRest)) {
			} else {
				return false;
			}
		}
		return true;
	}

	@Override
	public void taskDone() {
		// TODO Auto-generated method stub
		this.state = State.Inactive;
		this.stuckhere = null;
		this.stuckState = pfStuckStep.Safe;
	}

}
