package seu.agent.BasicTask;

import java.util.List;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUAgent;
import seu.agent.command.seuCommand;
import seu.agent.command.seuMove;
import seu.agent.command.seuMoveXY;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;
import seu.utils.geo.GeoUtils;
import seu.utils.geo.Point;
import seu.utils.geo.Polygon;

public class PFStuckInRoadTask extends AbstractStuckTask {

	private static final int CHECK_STEP = 2;
	private static final int LEAST_MOVE_LEN = 2000;
	private static final int LEAST_MOVE_NEIGHBOUR = 3;

	private final StandardWorldModel world;

	private Road pickRoad = null;
	private EntityID pickBlockadeId = null;
	private EntityID stuckWhere;
	private int stuckX;
	private int stuckY;
	private List<EntityID> lastMoveCommandPath;
	private final SEUAgent agent;

	public PFStuckInRoadTask(SEUAgent<? extends StandardEntity> agent) {
		super(agent);
		this.agent = agent;
		this.world = agent.getModel();
	}

	@Override
	public void takeDone() {

	}

	private void reset() {
		pickRoad = null;
		pickBlockadeId = null;
		stuckWhere = null;
		lastMoveCommandPath = null;
	}

	@Override
	public boolean shouldExecute() {
		int time = agent.getTimeStep();
		state = State.Safe;
		if (time < agent.getStartTime() + CHECK_STEP)
			return false;

		if (agent.getMeAsHuman().isBuriednessDefined())
			if (agent.getMeAsHuman().getBuriedness() > 0)
				return false;

		if (agent.location() instanceof Building)
			return false;

		if (state == State.Safe) {

			if (!isAlwaysMove())
				return false;
			// if (!isAlwaysSamePlace())
			// return false;
			if (!isNotMoveTooLong()) {
				if (!isLingerAbout())
					return false;
				else {
					// System.err.println(agent.getMeAsHuman() + "in "
					// + agent.getTimeStep() + " is linger about");
				}
			}

			stuckX = agent.getMeAsHuman().getX();
			stuckY = agent.getMeAsHuman().getY();

			// stuckState = State.CleanNearest;
			stuckWhere = agent.getMeAsHuman().getPosition();
			lastMoveCommandPath = null;

			seuCommand c = (seuCommand) this.agent.getCommands().get(time - 1);
			if (c instanceof seuMove) {
				lastMoveCommandPath = ((seuMove) c).getPath();
			} else if (c instanceof seuMoveXY) {
				lastMoveCommandPath = ((seuMoveXY) c).getPath();
			}

			pickBlockade();

			if (pickBlockadeId != null) {
				state = State.CleanNearest;
				return true;

			} else {
				state = State.RandomMove;
				// log.warn("Out-of-Sight-Stuck");
				return true;
			}
		}

		if (state == State.CleanNearest) {
			if (pickRoad != null && pickBlockadeId != null) {
				if (pickRoad.getBlockades().contains(pickBlockadeId)) {
					Blockade b = (Blockade) world.getEntity(pickBlockadeId);

					// log.info("B:" + b + " " + b.getRepairCost());
					if (b != null) {
						if (b.isRepairCostDefined()) {
							if (b.getRepairCost() > 0) {
								// Still Need clean
								return true;
							}
						}
					}
				}

				// Done
				pickRoad = null;
				pickBlockadeId = null;
				state = State.TryMove;
			} else {
				// log.warn("Bad stuck state");
				pickRoad = null;
				pickBlockadeId = null;
				state = State.TryMove;
			}
		}

		if (state == State.TryMove) {
			return true;
		}

		if (state == State.MoveEst) {
			if (GeoUtils.dist(this.stuckX, this.stuckY, agent.getMeAsHuman()
					.getX(), agent.getMeAsHuman().getY()) < LEAST_MOVE_LEN) {
				stuckX = agent.getMeAsHuman().getX();
				stuckY = agent.getMeAsHuman().getY();

				state = State.CleanNearest;
				stuckWhere = agent.getMeAsHuman().getPosition();
				lastMoveCommandPath = null;

				seuCommand c = (seuCommand) this.agent.getCommands().get(
						time - 1);
				if (c instanceof seuMove) {
					lastMoveCommandPath = ((seuMove) c).getPath();
				} else if (c instanceof seuMoveXY) {
					lastMoveCommandPath = ((seuMoveXY) c).getPath();
				}

				pickBlockade();

				if (pickBlockadeId != null) {
					state = State.CleanNearest;
					return true;

				} else {
					// TODO should blind-clean
					// log.warn("Out-of-Sight-Stuck");
					return false;
				}
			} else {
				state = State.Safe;
				return false;
			}
		}
		return false;
	}

	private void pickBlockade() {

		this.pickRoad = null;
		this.pickBlockadeId = null;

		int hx = agent.getMeAsHuman().getX();
		int hy = agent.getMeAsHuman().getY();

		double mindist = Double.MAX_VALUE;
		for (int i = 0; i < agent.getEntityInsight().size(); i++) {
			StandardEntity se = (StandardEntity) agent.getEntityInsight()
					.get(i);
			if (se instanceof Road) {
				Road r = (Road) se;
				if (!r.isBlockadesDefined())
					continue;

				for (EntityID bid : r.getBlockades()) {
					if( world.getEntity(bid)==null){//niu13.10.14
						continue;}
					Blockade blk = (Blockade) world.getEntity(bid);
					if (!blk.isRepairCostDefined())
						continue;

					if (blk.getRepairCost() <= 0)
						continue;

					if (!agent.isInClearRange(blk))
						continue;

					Point p = Polygon.getNearestEdgeOfPolygon(hx, hy,
							blk.getApexes());
					double temp = GeoUtils.dist(hx, hy, p.getX(), p.getY());
					if (temp < mindist) {
						mindist = temp;
						pickBlockadeId = blk.getID();
						pickRoad = r;
					}
				}
			}
		}
	}

	@Override
	public void execute() {
//		if(agent.getID().getValue()==327290976)
//		System.out.println(agent.getTimeStep()+" time "+agent.getID()+" PFStuckInRoadTask ");
		// log.debug("@" + context.getTimestamp() + " PF:" +
		// context.getHuman().getID() + " Into StuckInBuildingState:" +
		// stuckState);
		if (state == State.Safe) {
			// log.error("Bad StuckInRoad state");
			agent.sendRest(agent.getTimeStep());
		}

		if (state == State.CleanNearest) {
			if (pickBlockadeId == null) {
				// log.error("Bad StuckInRoad state");
				agent.sendRest(agent.getTimeStep());
			} else {
				//agent.sendClear(pickBlockadeId);
				agent.sendClearArea(pickBlockadeId);//niu
				 //System.out.println(agent.getTimeStep()+" time "+agent.getID()+" PFStuckInRoadTask "+" Clear"+pickBlockadeId);

			}
		}

		if (state == State.TryMove) {
			if (lastMoveCommandPath != null) {
				state = State.MoveEst;
				agent.sendMove(agent.getTimeStep(), lastMoveCommandPath);
			} else {
				// log.error("Bad StuckInRoad state");
				agent.sendRest(agent.getTimeStep());
			}
		}

		if (state == State.RandomMove) {
			state = State.Safe;
			agent.sendMove(agent.getTimeStep(), agent.randomWalk());
		}

		agent.sendRest(agent.getTimeStep());
	}

	// @Override
	// public AntCommand execute() {
	// if (stuckState == StuckStep.CleanAround) {
	// Blockade blk = behavior.getRandomOneOfInsightInRangeBlockade();
	// if (blk != null) {
	// return new AntClear(this.context.getTimestamp(), blk.getID());
	// } else {
	// log.warn("Bad stuck state");
	// stuckState = StuckStep.MovingOut;
	// }
	// }
	//
	// if (stuckState == StuckStep.MovingOut) {
	// Blockade blk = behavior.getRandomOneOfInsightInRangeBlockade();
	// if (blk != null) {
	// return new AntClear(this.context.getTimestamp(), blk.getID());
	// } else {
	// return new AntMove(this.context.getTimestamp(),
	// behavior.getRandomWalk());
	// }
	// } else {
	// log.warn("Bad stuck state");
	// return new AntMove(this.context.getTimestamp(),
	// behavior.getRandomWalk());
	// }
	// return null;
	// }

	@Override
	public String toString() {
		return PFStuckInRoadTask.class.getName();
	}

	private boolean isLingerAbout() {
		int time = agent.getTimeStep();
		if (time < CHECK_STEP * 2 + agent.getStartTime())
			return false;
		for (int i = 1; i < CHECK_STEP * 2 - 3; i++) {
			EntityID info1 = agent.getLocation(time - i);
			EntityID info2 = agent.getLocation(time - i + 3);
			Path p = agent.getSearch().getPath((Area) world.getEntity(info1),
					(Area) world.getEntity(info2), PathType.Shortest);
			if (p.size() >= LEAST_MOVE_NEIGHBOUR)
				return false;
		}
		return true;
	}

	private boolean isAlwaysSamePlace() {
		int time = agent.getTimeStep();
		for (int i = 1; i < CHECK_STEP; i++) {
			EntityID info1 = agent.getLocation(time - i);
			EntityID info2 = agent.getLocation(time - i + 1);
			if (!info1.equals(info2))
				return false;

		}
		return true;
	}

	private boolean isNotMoveTooLong() {
		int time = agent.getTimeStep();
		int totals = 0;
		for (int i = 1; i < CHECK_STEP; i++) {
			EntityID info1 = agent.getLocation(time - i);
			EntityID info2 = agent.getLocation(time - i + 1);
			totals += agent.getModel().manhattanDistance(info1, info2);
		}
		if (totals > LEAST_MOVE_LEN) {
			return false;
		} else {
			return true;
		}
	}

	private boolean isAlwaysMove() {
		int time = agent.getTimeStep();
		for (int i = 1; i <= CHECK_STEP; i++) {
			seuCommand ac = (seuCommand) this.agent.getCommands().get(time - i);
			if ((ac instanceof seuMove) || (ac instanceof seuMoveXY)) {
			} else {
				return false;
			}
		}
		return true;
	}

	@Override
	public void taskDone() {
		// TODO Auto-generated method stub
		this.state = State.Inactive;
		this.state = State.Safe;

		reset();
	}

}
