package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Ambulance.SEUAmbulanceTeam;
import seu.message.MessagePriority;
import seu.message.Type.CivilianIsSavedOrDeadMessage;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.object.BuriedHumanComparator;
import seu.utilities.CivDistanceComparator;
import seu.utilities.EntityComparator;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class SaveAgentTask extends AmbulanceTeamTask {

	private final List<SaveHuman> saveHumanTasks;

	// 被忽略的任务和已完成救援的人
	private final List<EntityID> savedAndRemovedHumans;

	// 已经救援的人
	private final List<EntityID> savedHumansInSendUnload;
	// private final List<EntityID> savedhumansInSendRescue;

	// 上一周期的Buriedness
	int lastBuriedness = 0;

	// 本周期的Briedness
	int currentBuriedness = 0;

	public List<EntityID> getSavedHumans() {
		return savedAndRemovedHumans;
	}

	// 当前周期的目标
	Human currentHuman;
	// 上周期的目标
	Human lastHuman;

	public Human getCurrentHuman() {
		return currentHuman;
	}
    int starttime;
	Path path;
	boolean isAgents;
	boolean isPathRequested;

	public SaveAgentTask(SEUAmbulanceTeam agent, boolean isAgents) {
		super(agent);
		saveHumanTasks = new ArrayList<SaveHuman>();

		savedHumansInSendUnload = new ArrayList<EntityID>();
		savedAndRemovedHumans = new ArrayList<EntityID>();
		// savedhumansInSendRescue = new ArrayList<EntityID>();
		this.isAgents = isAgents;
	}

	@Override
	public void taskDone() {
		if (state != State.MovingToRefuge) {
			state = State.Inactive;
		}
	}

	// 判断该Human是否需要被救援
	public boolean isHumanNeeded2BRescued(Human h) {
		if (agent.getDeadHumans().contains(h.getID())
				|| savedHumansInSendUnload.contains(h.getID())
				// || (savedhumansInSendRescue.contains(h.getID()))
				|| savedAndRemovedHumans.contains(h.getID()))
			return true;
		else
			return false;
	}

	/**
	 * @param h是任务目标进行判断的Human
	 * @return 判断结果，该消息产生的任务Human是否应该救
	 */
	public boolean isMessageTaskNeedToDo(Human h) {
		if (h.isPositionDefined())
			if (agent.getModel().getEntity(h.getPosition()) instanceof Refuge)
				return false;
		if (h.isBuriednessDefined())
			if (h.getBuriedness() == 0)
				return false;
		return true;
	}

	// 从任务列表中给任务
	public void giveTaskFromTaskLists() {
		Collections.sort(saveHumanTasks,
				new CivDistanceComparator(agent.location(), agent.getModel()));
		currentHuman = null;
		for (SaveHuman sh : saveHumanTasks) {
			if (!isHumanNeeded2BRescued(sh.human)
					&& isMessageTaskNeedToDo(sh.human)
			// && sh.human.isPositionDefined()
			) {

				currentHuman = sh.human;
				break;
			}
		}
	}

	@Override
	public boolean shouldExecute() {
		State expectedMove;
		Path path = null;
		// 移除无法看到的任务，任务目标不在指定地点，可能被其他AT救回Refuge
		removeInvalidTasks();
		expectedMove = getExpectedMove();

		for (EntityID id : savedHumansInSendUnload) {
			removeSaveTask(id);
		}

		for (EntityID id : agent.getDeadHumans()) {
			removeSaveTask(id);
		}

		removeSaveTask(agent.getID());

		needToRescueHumanInTasks();

		if (currentHuman != null) {
			if (expectedMove == State.MovingToHuman
					&& isHumanNeeded2BRescued(currentHuman)) {
				// No.1
				// System.err.println(agent.getID() + "No.1");
				giveTaskFromTaskLists();
				if (currentHuman == null) {
					state = State.Inactive;
					return false;
				}

			}
		} else {
			// No.2
			// System.err.println(agent.getID() + "No.2");
			giveTaskFromTaskLists();
			if (currentHuman == null) {
				state = State.Inactive;
				return false;
			}
		}

		// 估算死亡时间
		int deltaBuriedness;
		// 最大救援人数
		int maxATs = Integer.MIN_VALUE;
		// 最短死亡时间
		int minDT = Integer.MAX_VALUE;
		// 救援所需周期数
		int rescueCycles;

		if (currentHuman != null && lastHuman == currentHuman
				&& currentHuman.isBuriednessDefined()) {

			lastBuriedness = currentBuriedness;
			currentBuriedness = currentHuman.getBuriedness();
			deltaBuriedness = lastBuriedness - currentBuriedness;

			if (deltaBuriedness > 0) {

				int deadTime = Integer.MAX_VALUE;

				// 最大救援人数
				if (maxATs < deltaBuriedness)
					maxATs = deltaBuriedness;

				rescueCycles = currentHuman.getBuriedness() / maxATs;

				if (currentHuman.isHPDefined()
						&& currentHuman.isDamageDefined()) {
					if (currentHuman.getDamage() != 0)
						deadTime = currentHuman.getHP()
								/ currentHuman.getDamage();
				}

				// 最短死亡时间
				if (deadTime < minDT) {
					minDT = deadTime;
				}

				// 如果所需要救援周期数大于最短死亡时间，则该智能体必然死亡
				if (rescueCycles > deadTime) {
					// 放弃无法救援的人
					// System.err.println("Remove" + currentHuman
					// + " estimated 2 B Dead");

					savedAndRemovedHumans.add(currentHuman.getID());

					CivilianIsSavedOrDeadMessage mesage = new CivilianIsSavedOrDeadMessage(
							(currentHuman.getID()));
					agent.sendMessage(mesage, MessagePriority.High);
					// No.3
					// System.err.println(agent.getID() + "No.3");
					giveTaskFromTaskLists();
					if (currentHuman == null) {
						state = State.Inactive;
						return false;
					}
				}

			}
		}
		lastHuman = currentHuman;
		if (expectedMove == State.MovingToHuman) {

			// List<Building> buildingsWithHuman;

			// buildingsWithHuman = getBuildingsWithHuman();
			if (currentHuman == null) {
				return false;
			}
			try {
				path = getSearch().getPath(agent.getMeAsHuman(), currentHuman,
						PathType.EmptyAndSafe);
			} catch (Exception e) {

			}
			if (path != null) {
				if (path.isPassable()) {

					this.path = path;

					state = expectedMove;
					return true;
				} else if (path.hasInfiniteCost()) {
					path = getSearch().getPath(agent.getMeAsHuman(),
							currentHuman, PathType.Shortest);

					if (!path.isEmpty()) {
						requestPathTo(path.getDestination(),
								ClearPathReason.SaveHuman);
						state = State.Inactive;
						return false;
					} else {
						state = State.Inactive;
						return false;
					}
				} else {
					state = State.Inactive;
					return false;
				}
			} else {
				state = State.Inactive;
				return false;
			}
		} else if (expectedMove == State.RescuingHuman) {
			state = expectedMove;
			return true;
		} else if (expectedMove == State.LoadingHuman) {
			state = expectedMove;

			return true;
		} else if (expectedMove == State.MovingToRefuge) {
			if(agent.hasnoRefuge()){//niu13.10.16 没reguge
				path=agent.getPathToSafePlace();
			}else{
			path = agent.getPathToRefuge();
			}
			if (path != null) {
				if (path.isPassable()) {

					this.path = path;

					state = expectedMove;
					return true;
				} else if (path.hasInfiniteCost()) {

					path = agent.getPathToRefuge(PathType.Shortest);
					if (path != null && path.isPassable()) {
						requestPathTo(path.getDestination(),
								ClearPathReason.MoveHumanToRefuge);
						state = State.Inactive;
						return false;
					} else {
						state = State.Inactive;
						return false;
					}
				} else {
					state = State.Inactive;
					return false;
				}
			} else {
				state = State.Inactive;
				return false;
			}
		} else if (expectedMove == State.UnloadingHuman) {
			state = expectedMove;
			return true;
		} else {
			state = State.Inactive;
			return false;
		}

	}

	@Override
	public void execute() {
		if (state == State.Inactive) {
		}
		if (state == State.MovingToHuman) {
			agent.sendMove(path);
		}
		if (state == State.RescuingHuman) {
			agent.sendRescue(currentHuman);
			// if (!savedhumansInSendRescue.contains(currentHuman.getID())) {
			// savedhumansInSendRescue.add(currentHuman.getID());
			// }
		}
		if (state == State.LoadingHuman) {
			CivilianIsSavedOrDeadMessage mesage = new CivilianIsSavedOrDeadMessage(
					currentHuman.getID());
			agent.sendMessage(mesage, MessagePriority.Medium);

			agent.sendLoad(currentHuman);
		}
		if (state == State.MovingToRefuge) {
			agent.sendMove(path);
		}
		if (state == State.UnloadingHuman) {

			agent.sendUnload();

			if (!savedAndRemovedHumans.contains(currentHuman.getID()))
				savedAndRemovedHumans.add(currentHuman.getID());

			if (!savedHumansInSendUnload.contains(currentHuman.getID())) {
				savedHumansInSendUnload.add(currentHuman.getID());

			}

		}
	}

	protected State getExpectedMove() {
		Civilian civilian;
		Human human;

		if (!isAgents) {
			civilian = getLoadedCivilian();
			if (civilian != null) {
				currentHuman = civilian;
				if(agent.hasnoRefuge()&& (agent.isAtSafePlace()
						)){//niu13.10.16
					return State.UnloadingHuman;
				}
				if (agent.isInRefuge()) {
					return State.UnloadingHuman;
				} else {
					return State.MovingToRefuge;
				}
			}
		}

		human = getHumanToLoad();
		if (human != null) {
			if (human instanceof Civilian) {
				civilian = (Civilian) human;
				if (shouldLoadRescuedCivilian(civilian)) {
					currentHuman = civilian;
					starttime=agent.getTimeStep();
					return State.LoadingHuman;
				} else {
				}
			} else {
				removeSaveTask(human.getID());
			}
		}
		// System.err.println(agent.getID() + "No.4");
		human = getHumanToRescue();
		if (human != null) {
			currentHuman = human;
			return State.RescuingHuman;
		}

		return State.MovingToHuman;
	}

	public boolean hasTaskForHuman(Human human) {
		for (SaveHuman task : saveHumanTasks) {
			if (task.getHuman(getModel()).equals(human)) {
				return true;
			}
		}
		return false;
	}

	public void addSaveTask(SaveHuman saveHumanTask) {
		Human human;
		StandardEntity location;

		human = saveHumanTask.getHuman(getModel());
		location = saveHumanTask.getLocationOfHuman();

		// 解决抛出异常问题
		if (null != location) {
			human.setPosition(location.getID());
		}

		if (!hasTaskForHuman(human)
				&& !savedAndRemovedHumans.contains(human.getID())
				&& !(location instanceof Refuge)
				&& (location instanceof Building)) {

			// if (!saveHumanTasks.contains(saveHumanTask))
			// saveHumanTasks.add(saveHumanTask);
			// printSaveHumanTask(saveHumanTask);
			for (SaveHuman s : saveHumanTasks) {
				if (s.human.getID().getValue() == saveHumanTask.human.getID()
						.getValue()) {
					return;
				}
			}
			saveHumanTasks.add(saveHumanTask);
		}
	}

	public void removeSaveTask(EntityID humanId) {
		// System.out.println("Cvilian " + humanId.getValue());
		// System.out.println(saveHumanTasks_.toString());
		SaveHuman saveHumanTask = null;

		for (SaveHuman next : saveHumanTasks) {
			if (next.getHuman(getModel()).getID().getValue() == humanId
					.getValue()) {
				saveHumanTask = next;
			}
		}
		if (saveHumanTask != null) {
			removeSaveTask(saveHumanTask);

		}
		// System.out.println(saveHumanTasks.toString());
	}

	public void removeSaveTask(SaveHuman task) {
		if (task != null) {
			// System.out.println("time  " + agent.getTimeStep() + "  id  "
			// + agent + "  task" + task + "  " + task.getHumanId());
			saveHumanTasks.remove(task);
			savedAndRemovedHumans.add(task.getHumanId());
		}
		// System.out.println(saveHumanTasks.toString());
	}

	public boolean shouldLoadRescuedCivilian(Civilian civilian) {
		SEUAmbulanceTeam rat;
		List<AmbulanceTeam> nearAgents, saverAgents;

		EntityComparator comparator;
		AmbulanceTeam loader;
		boolean shouldLoad = false;

		comparator = new EntityComparator();
		rat = getAmbulanceAgent();
		saverAgents = new ArrayList<AmbulanceTeam>();
		nearAgents = rat.getNearbyTeamAgents();

		// 处理周围没有智能体的异常
		try {
			for (AmbulanceTeam at : nearAgents) {
				if (at.getPosition().equals(civilian.getPosition())) {
					saverAgents.add(at);
				}
				Collections.sort(saverAgents, comparator);
				loader = saverAgents.get(0);
				shouldLoad = rat.getMeAsHuman().equals(loader);
			}
		} catch (Exception e) {
			System.err.println("There Is No Agents Around");
		}

		return shouldLoad;
	}

	private void requestPathTo(EntityID target, ClearPathReason reason) {
		StandardEntity location;
		ClearPathIsNeededMessage message;

		location = agent.location();
		message = new ClearPathIsNeededMessage(location.getID(), target, reason);
		agent.sendMessage(message, MessagePriority.High);
	}

	private Civilian getLoadedCivilian() {
		List<Civilian> civilians;

		civilians = agent.getCiviliansAround();
		for (Civilian civilian : civilians) {
			if (civilian.isPositionDefined()) {
				EntityID position;

				position = civilian.getPosition();
				if (position.equals(agent.getID())) {
					return civilian;
				}
			}
		}
		if (state == State.LoadingHuman) {
			if (currentHuman != null) {
				if (currentHuman instanceof Civilian) {
					Civilian civilian;

					civilian = (Civilian) currentHuman;
					if (agent.canSee(civilian.getID())) {
						return civilian;
					}
				}
			}
		}
		return null;
	}

	private Human getHumanToLoad() {
		List<? extends Human> humans;

		if (isAgents) {
			humans = agent.getFoundAgents();
		} else {
			humans = agent.getCiviliansAround();
		}
		for (Human human : humans) {
			if (human.isPositionDefined()) {
				EntityID positionId;
				StandardEntity position;

				positionId = human.getPosition();
				position = getModel().getEntity(positionId);
				if (!(position instanceof Refuge)) {
					if (positionId.equals(agent.getMeAsHuman().getPosition())) {
						if (position instanceof Building) {
							if (human.isBuriednessDefined()) {
								if (human.getBuriedness() == 0) {
									if (human.isHPDefined()) {
										if (human.getHP() > 0) {
											return human;
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return null;
	}

	private Human getHumanToRescue() {
		List<? extends Human> humansAround;
		List<Human> humans;

		humans = new ArrayList<Human>();

		if (isAgents) {
			humansAround = agent.getFoundAgents();
		} else {
			humansAround = agent.getCiviliansAround();
		}
		for (Human human : humansAround) {
			if (human.isPositionDefined()) {
				EntityID positionId;
				StandardEntity position;

				positionId = human.getPosition();
				position = getModel().getEntity(positionId);
				if (position instanceof Building) {
					if (positionId.equals(agent.getMeAsHuman().getPosition())) {
						if (!(position instanceof Refuge)) {
							if (human.isBuriednessDefined()) {
								if (human.getBuriedness() > 0) {
									if (human.isHPDefined()) {
										if (human.getHP() > 0) {
											if (agent.canSee(human.getID())) {
												humans.add(human);
											} else {
												removeSaveTask(human.getID());
											}
										} else {

										}
									} else {
									}
								} else {
								}
							} else {
							}
						} else {
						}
					} else {
					}
				} else {
				}
			}
		}
		if (humans.isEmpty()) {
			return null;
		} else {
			BuriedHumanComparator comparator;
			comparator = new BuriedHumanComparator();

			Collections.sort(humans, comparator);
			return humans.get(0);
		}
	}

	public List<Building> getBuildingsWithHuman() {
		List<Building> buildings;

		buildings = new ArrayList<Building>();
		for (SaveHuman task : saveHumanTasks) {
			StandardEntity position;
			if (!savedAndRemovedHumans.contains(task.getHumanId())) {
				position = task.getLocationOfHuman();
				if (position != null && position instanceof Building) {
					Building building;

					building = (Building) position;
					if (!building.isOnFire()
							&& building.getFierynessEnum() != Fieryness.BURNT_OUT) {
						buildings.add(building);
					} else {
					}
				} else {
				}
			} else {
			}
		}
		return buildings;
	}

	// 判断任务列表中任务是否有效
	private void needToRescueHumanInTasks() {
		Human human = null;

		List<SaveHuman> copysavehuman;
		copysavehuman = new ArrayList<SaveHuman>();
		for (SaveHuman p1 : saveHumanTasks)
			copysavehuman.add(p1);
		for (SaveHuman pp : copysavehuman) {
			human = pp.human;
			if (human.isPositionDefined()) {
				EntityID positionId;
				StandardEntity position;

				positionId = human.getPosition();
				position = getModel().getEntity(positionId);
				if (!(position instanceof Building) && isAgents)   //在路上也可能受伤niu 14.2.13
					if (human.isHPDefined())
						if (human.getHP() == 0)
							removeSaveTask(human.getID());
				if (position instanceof Refuge)
					removeSaveTask(human.getID());
				if (human.isBuriednessDefined())
					if (human.getBuriedness() == 0)
						removeSaveTask(human.getID());
				if (human.isHPDefined())
					if (human.getHP() == 0)
						removeSaveTask(human.getID());
			}
		}

	}

	public void removeInvalidTasks() {
		Area area;
		List<SaveHuman> temp;

		area = (Area) agent.getMeAsHuman().getPosition(getModel());
		temp = new ArrayList<SaveHuman>(saveHumanTasks);
		if (area instanceof Building) {
			for (SaveHuman task : temp) {
				if (area.equals(task.getLocationOfHuman())) {
					if (!agent.canSee(task.getHuman(getModel()).getID())) {
						// System.out.println("time  "+agent.getTimeStep()+"   id   "+agent+"  area  "+area
						// +"  target   "+task.getHumanId()
						// +"  targetlocation "+task.getLocationOfHuman());
						CivilianIsSavedOrDeadMessage mesage = new CivilianIsSavedOrDeadMessage(
								(task.human.getID()));
						// system.out.println("  time  ")
						agent.sendMessage(mesage, MessagePriority.Medium);
						removeSaveTask(task);
					}
				}
			}
		}
	}
}