package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import rescuecore2.misc.Pair;
import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUAgent;
import seu.agent.command.seuCommand;
import seu.agent.command.seuMove;
import seu.agent.command.seuMoveXY;
import seu.message.MessagePriority;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.utilities.Search.Cost.PathType;
import seu.utils.geo.Converter;
import seu.utils.geo.GeoUtils;
import seu.utils.geo.Line;
import seu.utils.geo.Point;
import seu.utils.geo.Segment;

/**
 * 
 * @author Oliver
 * 
 */
public class StuckInRoadTask extends AbstractStuckTask {

	private static final int CHECK_STEP = 2;
	private static final int DEFAULT_DISTANCE_TO_POS = 1000;
	private static final int MIN_DISTANCE = 100;
	private static final int REACH_POS_MIN_DISTANCE = 800;
	private static final int DEFAULT_DISTANCE_TO_MOVE_OUT = 20000;
	private static final int WAIT_TIME_TO_SEND_STUCK_MESSAGE = 2;
	private static final int MAX_MOVE_TO_POS_TRY_TIME = 2;

	private Road stuckRoad;
	private List<EntityID> stuckMovePath;
	private EntityID nextAreaID;
	private Area nextArea;
	private Edge edgeToNextArea;
	private boolean isLineInStuckRoad;
	private Line line;
	private Point ref;
	private Point pos;
	private List<EntityID> pathToPos;

	private StuckStep stuckState;

	private boolean needSendStuckMessageLater;
	private int currentWaitTimeToSendStuckMessage;
	private int moveToPosCount;
	SEUAgent<? extends StandardEntity> agent;

	private enum StuckStep {
		NotStuck, MovingToPos, MovingOut
	}

	public StuckInRoadTask(SEUAgent<? extends StandardEntity> agent) {
		super(agent);
		this.agent = agent;
		this.stuckState = StuckStep.NotStuck;

		this.moveToPosCount = 0;
	}

	@Override
	public void takeDone() {

	}

	public void overleap() {           //此函数从未调用
		if (this.needSendStuckMessageLater) {
			this.currentWaitTimeToSendStuckMessage++;
			if (currentWaitTimeToSendStuckMessage >= WAIT_TIME_TO_SEND_STUCK_MESSAGE) {
				sendStuckMessage();
				currentWaitTimeToSendStuckMessage = 0;
				needSendStuckMessageLater = false;
				this.stuckRoad = null;
			}
			// stuckRoad for message
		} else {
			this.stuckRoad = null;
		}
		this.stuckState = StuckStep.NotStuck;
		this.stuckMovePath = null;
		this.nextAreaID = null;
		this.nextArea = null;
		this.edgeToNextArea = null;
		this.isLineInStuckRoad = false;
		this.line = null;
		this.ref = null;
		this.pos = null;
		this.pathToPos = null;
		this.moveToPosCount = 0;
	}

	@Override
	public void execute() {
		//System.out.println(agent.getID()+"  StuckInRoadTask     Execute");
		if (stuckState == StuckStep.MovingToPos) {
			state = State.MoveEst;
			//System.out.println("stuckinroadtask"+"           "+"   movingtopos");
			agent.sendMove(agent.getTimeStep(), pathToPos, pos.getX(),
					pos.getY());
		} else if (stuckState == StuckStep.MovingOut) {
			state = State.MovingOutOfStuck;
			//System.out.println("stuckinroadtask"+"           "+"3   ");

			if (pos != null) {
				agent.sendMove(agent.getTimeStep(), pathToPos, pos.getX(),
						pos.getY());
			} else {
				agent.sendMove(agent.getTimeStep(), pathToPos);
			}
		} else {
			agent.sendMove(agent.getTimeStep(), agent.randomWalk());
		}
	}

	@Override
	public boolean shouldExecute() {
		
		//System.out.println(agent.getID()+"  StuckInRoadTask    shouldExecute");
		// no need to check stuck at start
		if (agent.getTimeStep() < ignoreTimestamp + CHECK_STEP)
			return false;
		//stuckState = StuckStep.NotStuck;
		StandardEntity here = agent.location();
		if (stuckState == StuckStep.NotStuck) {
			if (!(here instanceof Road) || !isStuck(CHECK_STEP)) {
				return false;
			}
			stuckRoad = (Road) here;
			if (isAgentInBlockade(stuckRoad, agent.getMeAsHuman())) {
				sendStuckMessage();
				return false;
			}

			this.stuckMovePath = getStuckMovePath();
			if (stuckMovePath == null) {
				return false;
			}

			for (EntityID next : stuckMovePath) {
				if (next.equals(stuckRoad.getID())) {
					continue;
				}
				if (stuckRoad.getNeighbours().contains(next)) {
					nextAreaID = next;
					break;
				}
			}
			if (nextAreaID == null) {
				if (stuckMovePath.size() > 1) {
					// log.error("nextAreaID not found! " + stuckMovePath);
				} else {
					// log.error("FIXME: short path problem!");
					sendStuckMessage();
				}
				return false;
			}
			nextArea = (Area) world.getEntity(nextAreaID);
			edgeToNextArea = stuckRoad.getEdgeTo(nextAreaID);

			if (!setPosToMove()) {
				this.needSendStuckMessageLater = true;
				return false;
			}
			this.state = State.MovingOutOfStuck;
			this.stuckState = StuckStep.MovingToPos;
			return true;
		} else if (stuckState == StuckStep.MovingToPos) {
			if (here instanceof Building) {
				// log.warn("Get out of stuck when MovingToPos:" + pos);
				return false;
			} else {
				if (!hasReachPos()) {
					if (moveToPosCount == 0) {
						// log.info("hasReachPos first fail!");
					}
					moveToPosCount++;
					if (moveToPosCount >= MAX_MOVE_TO_POS_TRY_TIME) {
						this.needSendStuckMessageLater = true;
						return false;
					} else {
						StandardEntity e = agent.getModel().getEntity(
								pathToPos.get(pathToPos.size() - 1));
						if (e instanceof Area) {
							Area a = (Area) e;
							pathToPos = agent
									.getSearch()
									.getPath(agent.getMeAsHuman(), a,
											PathType.Shortest).getEntities();
							return true;
						}
						return false;
					}
				}
				Pair<Point, Area> pair = getPositionToMoveOut();
				// log.info("getPositionToMoveOut:" + pair);
				if (pair != null) {
					pos = pair.first();
					StandardEntity e = agent.getModel().getEntity(
							pair.second().getID());
					if (e instanceof Area) {
						Area a = (Area) e;
						pathToPos = agent
								.getSearch()
								.getPath(agent.getMeAsHuman(), a,
										PathType.Shortest).getEntities();
					}
				} else {
					pos = null;
					pathToPos = agent.randomWalk();
				}
				stuckState = StuckStep.MovingOut;
				return true;
			}
		} else if (stuckState == StuckStep.MovingOut) {
			if (!here.equals(stuckRoad)) {
				state = State.Inactive;
				stuckState = StuckStep.NotStuck;
						// success
				return false;
			} else {
				sendStuckMessage();
				// failed!
				// log.info("Try failed in moving out stuck in road!");
				return false;
			}
		} else {
			// log.error("Unexpected stuckstate:" + stuckState);
			return false;
		}
	}

	private boolean isAgentInBlockade(Road here, Human me) {
		if (!here.isBlockadesDefined()) {
			return false;
		}
		for (EntityID id : here.getBlockades()) {
			Blockade b = (Blockade) world.getEntity(id);
			if (GeoUtils.inPolygon(b.getApexes(), me.getX(), me.getY())) {
				return true;
			}
		}
		return false;
	}

	private boolean setPosToMove() {
		this.line = null;
		this.ref = null;
		Segment segDiv = getDivSegment(stuckRoad, nextAreaID, me.getX(),
				me.getY());
		Set<Blockade> facedBlocakesOnStuckRoad = getFacedBlocakesOnStuckRoad(
				segDiv, stuckRoad, nextAreaID, world);
		if (!facedBlocakesOnStuckRoad.isEmpty()) {
			line = getPassableLine(new Line2D(new Point2D(segDiv.getX1(),
					segDiv.getY1()),
					new Point2D(segDiv.getX2(), segDiv.getY2())),
					facedBlocakesOnStuckRoad);
			if (line == null) {
				// log.error("Line is null");
				return false;
			}
			ref = new Point(
					(edgeToNextArea.getStartX() + edgeToNextArea.getEndX()) / 2,
					(edgeToNextArea.getStartY() + edgeToNextArea.getEndY()) / 2);
			isLineInStuckRoad = true;
			// log.info("line:" + line + " ref" + ref + " isLineInStuckRoad:" +
			// isLineInStuckRoad);
		} else {
			if (nextArea instanceof Building) {
				// log.error("Never should agent stuck like this!");
				return false;
			}
			Road nextRoad = (Road) nextArea;
			List<Blockade> blockades = new ArrayList<Blockade>();
			if (nextRoad.isBlockadesDefined()) {
				for (EntityID id : nextRoad.getBlockades()) {
					blockades.add((Blockade) world.getEntity(id));
				}
			}
			line = getPassableLine(
					new Line2D(new Point2D(edgeToNextArea.getStartX(),
							edgeToNextArea.getStartY()), new Point2D(
							edgeToNextArea.getEndX(), edgeToNextArea.getEndY())),
					blockades);
			if (line == null) {
				// log.error("Line is null");
				return false;
			}
			ref = new Point(me.getX(), me.getY());
			isLineInStuckRoad = false;
		}
		int distance = DEFAULT_DISTANCE_TO_POS;
		while (true) {
			pos = GeoUtils.getPerpendicular(line, distance, ref,
					!isLineInStuckRoad);
			if (pos == null) {
				// log.error("getPerpendicular failed!");
				break;
			}
			if (GeoUtils.inPolygon(stuckRoad.getApexList(), pos.getX(),
					pos.getY())) {
				break;
			} else {
				distance /= 2;
				if (distance <= MIN_DISTANCE) {
					pos = null;
					break;
				}
			}
		}
		if (pos == null) {
			// log.info("pos not found!");
			return false;
		}
		pathToPos = new ArrayList<EntityID>();
		pathToPos.add(stuckRoad.getID());
		return true;
	}

	private Pair<Point, Area> getPositionToMoveOut() {
		List<Area> areas = getRelatedArea();
		int distance = DEFAULT_DISTANCE_TO_MOVE_OUT;
		while (true) {
			pos = GeoUtils.getPerpendicular(line, distance, ref,
					isLineInStuckRoad);
			if (pos == null) {
				// log.error("getPerpendicular failed!");
				break;
			}
			for (Area area : areas) {
				if (GeoUtils.inPolygon(area.getApexList(), pos.getX(),
						pos.getY())) {
					return new Pair<Point, Area>(pos, area);
				}
			}
			distance /= 2;
			if (distance <= MIN_DISTANCE) {
				pos = null;
				break;
			}
		}
		// log.error("no pos to move!");
		return null;
	}

	private List<Area> getRelatedArea() {
		List<Area> rv = new ArrayList<Area>();
		if (isLineInStuckRoad) {
			rv.add(stuckRoad);
		}
		rv.add(nextArea);
		for (EntityID id : nextArea.getNeighbours()) {
			Area area = (Area) world.getEntity(id);
			rv.add(area);
		}
		return rv;
	}

	private List<EntityID> getStuckMovePath() {
		int time = agent.getTimeStep();
		for (int i = 1; i <= CHECK_STEP; i++) {
			seuCommand ac = this.agent.getCommands().get(time - i);
			if (ac instanceof seuMove) {
				return ((seuMove) ac).getPath();
			} else if (ac instanceof seuMoveXY) {
				return ((seuMoveXY) ac).getPath();
			}
		}
		return null;
	}

	private boolean hasReachPos() {
		if (Math.abs(me.getX() - pos.getX()) + Math.abs(me.getY() - pos.getY()) <= REACH_POS_MIN_DISTANCE) {
			return true;
		} else {
			return false;
		}
	}

	private void sendStuckMessage() {
		if (stuckRoad != null) {
			// sendMessage(new AgentIsStuckMessage(stuckRoad, context.getHuman()
			// .getX(), context.getHuman().getY()));
			// addVoiceMessage(new MVoiceAgentStuckInRoad(context.getAgentID(),
			// stuckRoad, context.getHuman().getX(), context.getHuman()
			// .getY()));
			ClearPathIsNeededMessage message = new ClearPathIsNeededMessage(
					stuckRoad.getID(), stuckRoad.getID(),
					ClearPathReason.AgentIsStuck);
			agent.sendMessage(message, MessagePriority.High);
//			System.out.println(agent.toString() + "in " + agent.getTimeStep()
//					+ "send message" + message);

		} else {
			// log.error("No stuckRoad, why send stuck message?");
		}
	}

	private Segment getDivSegment(Road stuckRoad, EntityID neighbour, int ax,
			int ay) {
		Edge edge = stuckRoad.getEdgeTo(neighbour);
		int lenX = edge.getEndX() - edge.getStartX();
		int lenY = edge.getEndY() - edge.getStartY();
		Segment seg1 = new Segment(ax - 10 * lenX, ay - 10 * lenY, ax, ay);
		Segment seg2 = new Segment(ax, ay, ax + 10 * lenX, ay + 10 * lenY);
		Point mid = new Point(ax, ay);

		List<Segment> segs = Converter
				.generateSegments(stuckRoad.getApexList());

		for (Segment next : segs) {
			Point intp = GeoUtils.getSegmentIntersectionPoint(seg1, next);
			if (intp != null) {
				seg1 = new Segment(intp, mid);
			}
			intp = GeoUtils.getSegmentIntersectionPoint(seg2, next);
			if (intp != null) {
				seg2 = new Segment(mid, intp);
			}
		}

		return new Segment(seg1.getOrigin(), seg2.getEnd());
	}

	private Set<Blockade> getFacedBlocakesOnStuckRoad(Segment divSeg,
			Road stuckRoad, EntityID neighbour, StandardWorldModel world) {
		Edge edge = stuckRoad.getEdgeTo(neighbour);
		Point flag = new Point((edge.getStartX() + edge.getEndX()) / 2,
				(edge.getStartY() + edge.getEndY()) / 2);

		Set<Blockade> ret = new HashSet<Blockade>();

		if (!stuckRoad.isBlockadesDefined()) {
			return ret;
		}
		for (EntityID bid : stuckRoad.getBlockades()) {
			Blockade blockade = (Blockade) world.getEntity(bid);
			if (GeoUtils.onSameSide(divSeg.getOrigin(), divSeg.getEnd(), flag,
					new Point(blockade.getX(), blockade.getY()))) {
				ret.add(blockade);
			}
		}
		return ret;
	}

	@Override
	public void taskDone() {
		// TODO Auto-generated method stub
		//this.stuckState = StuckStep.NotStuck;
//		this.stuckRoad = null;
//		this.stuckMovePath = null;
//		this.nextAreaID = null;
//		this.nextArea = null;
//		this.edgeToNextArea = null;
//		this.isLineInStuckRoad = false;
//		this.line = null;
//		this.ref = null;
//		this.pos = null;
//		this.pathToPos = null;
//		this.needSendStuckMessageLater = false;
//		this.moveToPosCount = 0;
		this.state = State.Inactive;
	}

}
