


package seu.agent.BasicTask;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.GasStation;
import rescuecore2.standard.entities.Hydrant;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Firebrigade.SEUFireBrigade;
import seu.message.MessagePriority;
import seu.message.Type.CivilianIsSavedOrDeadMessage;
import seu.message.Type.ClearPathIsNeededMessage;
import seu.message.Type.ClearPathReason;
import seu.message.Type.GasStationIsBurningMessage;
import seu.message.Type.Message;
import seu.object.SEUWorldModel;
import seu.utilities.Path;
import seu.utilities.Search.Cost.PathType;

public class WaterRefillTask extends BasicTask<SEUFireBrigade> {

	Path path;
	int maxWater;
	int minWater;
	Hydrant h;
	
	
	protected SEUWorldModel world;
	public WaterRefillTask(SEUFireBrigade agent, int maxWater, int minWater) {
		super(agent);
		this.maxWater = maxWater;
		this.minWater = minWater;
		if (agent.getNo() % 3 == 0) {
			minWater = maxWater / 2;
		}
	}

	@Override
	public void taskDone() {
		state = State.Inactive;
	}

	@Override
	public boolean shouldExecute() {
		State expectedState;
		expectedState = getExpectedState();
		if (expectedState == State.FillingWaterInRefuge) {
			state = expectedState;
			return true;
		} else if(expectedState == State.FillingWaterInHydrant){
			//System.out.println("FillingWaterInHydrant"+h.getID());
			state = expectedState;
			return true;
		}else if (expectedState == State.MovingToRefuge) {
			path = agent.getPathToRefuge();
			if (path != null && !path.isEmpty()) {
				if (path.isPassable()) {
					state = expectedState;
					return true;
				} else if (path.hasInfiniteCost()) {
					Path path1;
					path1 = agent.getPathToRefuge1111(PathType.Shortest);
					if (path1 != null && path1.isPassable()) {
						ClearPathIsNeededMessage message = new ClearPathIsNeededMessage(
								path1.getStart(), path1.getDestination(),
								ClearPathReason.SaveMySelf);
						agent.sendMessage(message, MessagePriority.High);
					}
					if( !(agent.getReachedHydrants().isEmpty())){//仿真初期refuges不可达,而部分hydrant可达
						for(Hydrant h:agent.getReachedHydrants()){
					path=agent.getPathToHydrant(h);
					state = expectedState;
					return true;
						}
					}
				}
			}
		}
		else if (expectedState == State.MovingToHydrant) {
			path = agent.getPathToHydrant(h);
			//System.out.println("FillingWaterInHydrant"+h.getID());
			if (path != null && !path.isEmpty()) {
				if (path.isPassable()) {
					state = expectedState;
					return true;
				} else if (path.hasInfiniteCost()) {
					Path path;
					path = agent.getPathToHydrant(PathType.Shortest,h);
					if (path != null && path.isPassable()) {
						ClearPathIsNeededMessage message = new ClearPathIsNeededMessage(
								path.getStart(), path.getDestination(),
								ClearPathReason.WaterRefill);
						agent.sendMessage(message, MessagePriority.High);
					}
				}
			}
		}
		
		return false;
	}

	

	public State getExpectedState() {
		FireBrigade fb;
		int water;
		fb = (FireBrigade) agent.getMeAsHuman();
		water = fb.getWater();
		if (water < maxWater) {
			if (agent.isInRefuge()) {
				if (shouldStayInRefuge()) {
					return State.FillingWaterInRefuge;
					
				} else {
					return State.Inactive;
				}
			}
			else if (isAtHydrant()) {//niu
//				if(agent.getFailInHydrantTime()!=0 &&
//						agent.getTimeStep()-agent.getFailInHydrantTime()<5){//加水失败，直接去refuge
//					System.out.print(agent.getTimeStep()+"MovingTorefuge"+agent.getID());
//					return State.MovingToRefuge;
//				}
				
				if (shouldStayAtHydrant()) {				
					return State.FillingWaterInHydrant;
				} else {
					return State.Inactive;//torefuge?
				}
			}
			
			
			
			else if(water <= maxWater/3 ){
				
				
				
					Refuge r=agent.getNearestRefuge();
					 h=agent.getNearestHydrant();
							
				
					if(r==null && h!=null){
						//System.out.print("MovingToHydrant");
						return State.MovingToHydrant;  
					}
					
					if(r!=null && h!=null){
						boolean refill=agent.refillchoose(r,h);
					
					//path = agent.getPathToHydrant();
				
					if(refill  && agent.getMeAsHuman().getDamage()==0){	//niu
						
						return State.MovingToHydrant;         //niu
					}else{
					      return State.MovingToRefuge;       
					     }
					}
					
						return State.MovingToRefuge;
					
					
			} 
			else {
					return State.Inactive;
			}
		} else {
			return null;
		}
	}

	@Override
	public void execute() {
		if (state == State.MovingToRefuge) {
			agent.sendMove(path);
			//System.out.println("MovingToRefuge");
		} else if (state == State.MovingToHydrant) {
			agent.sendMove(path);
			//System.out.println("MovingToHydrant");
		}else if (state == State.FillingWaterInRefuge) {
			agent.sendRest();
			//System.out.println("FillingWaterInRefuge");
		}else if(state == State.FillingWaterInHydrant){
			
			agent.sendRest();
			//System.out.println("FillingWaterInHydrant");
			
		}
	}

	
	public boolean shouldStayInRefuge() {
		List<FireBrigade> nearAgents;
		List<EntityID> team;
		int teamSize, nearAgentSize;
		double p, r;
		FireBrigade fb;
		int water;

		team = getModel().getTeam();
		teamSize = team.size();
		if (teamSize > 0 && maxWater > 0) {
			fb = (FireBrigade) agent.getMeAsHuman();
			water = fb.getWater();
			nearAgents = getFireAgent().getNearbyTeamAgents();
			nearAgentSize = nearAgents.size() - 1;
			p = ((double) nearAgentSize / teamSize)
					* ((double) water / maxWater);
			p = p * p;
			r = agent.getRandomSource().nextDouble();
			if (r < p) {
				//System.out.println("leave refuge"+((FireBrigade) agent.getMeAsHuman()).getWater());
				return false;
			}
		}
		
		return true;
	}

	public boolean shouldStayAtHydrant() {
		FireBrigade fb;
		int water;
		fb = (FireBrigade) agent.getMeAsHuman();
		water = fb.getWater();
		//System.out.println("should stay in hydrant?");
	if(water<maxWater/2 ){
		List<StandardEntity> entities=agent.getEntityInsight();
		List<FireBrigade> fbs=new ArrayList<FireBrigade>();
		List<Building> ebuildings=new ArrayList<Building>();
		for (StandardEntity next : entities){
			if(next instanceof FireBrigade){
				FireBrigade f=(FireBrigade) next;
				if(!fbs.contains(f)){
					fbs.add(f);
				}
			}
			if(next instanceof Building){
				Building B=(Building) next;
				if(((B.isFierynessDefined() && B.getFieryness()<2) ||
						 B.isFierynessDefined())
						&& !ebuildings.contains(B)){
					ebuildings.add(B);
				}
			}
		}
		
		
		if(!ebuildings.isEmpty()) {//niu prevent refill in the fire center
		if(fbs.isEmpty()){
		return true;
		}else{
			//System.out.println("fbs is not empty");
			for(FireBrigade f1:fbs){
				if( f1.getID().getValue()<agent.getID().getValue()
						){
					agent.setFailInHydrantTime(agent.getTimeStep());
					agent.setlastHydrant(h);
					//System.out.println(agent.getID()+"fail in hydrant"+agent.getlastHydrant());
					return false;
					
				}
			}
			return true;
		}
		}else{
			agent.setFailInHydrantTime(agent.getTimeStep());
			agent.setlastHydrant(h);
			return false;
		}
	}else{
		System.out.println("water"+fb.getWater()+"     ");
		return false;
	}
	}
	
	public boolean isAtHydrant() {//niu
		//boolean AtHydrant;
		//StandardEntity location;
		//location = agent.location();
	
		if( agent.location()instanceof Hydrant ){//wj//getLocation(agent.getTimeStep())== h.getID()){
			//for (FireBrigade next : agent.getAllFirebrigades()){
			//	if(next.getPosition(world).getID().equals(agent.location()) && next.getHP()>0
			//				&&next.getID().getValue()<agent.getID().getValue());//wj 
			//			return false;
			//		}
			return true;
		}
	
		//HydrantInUseMessage hydrantInUseMessage = new HydrantInUseMessage(
		//		location.getID(),timeStep);
		//sendMessage(hydrantInUseMessage,MessagePriority.High);
		//System.out.println(" sendHydrantInUsemessage ");
		
		return false;
	}
	
	public SEUFireBrigade getFireAgent() {
		return agent;
	}
	
}
