 package seu.agent;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import rescuecore2.worldmodel.EntityID;
import seu.object.SEUWorldModel;

public class DecisionManager {

	public int maxWater;
	public static final int MAX_BURRIEDNESS = 61;
	public static final int MAX_HP_WILL_DIE = 5000;
	public static final int MIN_HP_HELPLESS = 2000;
	public static final int MIN_BURIEDNESS_HELPLESS = 20;

	public static HealthState getHealthState(Human human, SEUWorldModel model,
			int timeStep) {
		StandardEntity position = human.getPosition(model);
		boolean onFire = (position instanceof Building)
				&& ((Building) position).isOnFire();
		if (human.isHPDefined() && human.getHP() == 0) {
			return HealthState.Dead;
		} else if (onFire) {
			return HealthState.Burning;
		} else if (!canBeSaved(human, timeStep)) {
			return HealthState.Helpless;
		} else if (human.isDamageDefined() && human.getDamage() >= 0
				&& human.isHPDefined() && human.getHP() <= MAX_HP_WILL_DIE) {
			return HealthState.WillDieSoon;
		} else if (human.isHPDefined() && human.getHP() <= MAX_HP_WILL_DIE) {
			return HealthState.WillDie;
		} else {
			return HealthState.WillLive;
		}
	}

	public static int getDeathTime(Human human, int timeStep) {
		int deathTime;
		if (human.isHPDefined() && human.isDamageDefined()) {
			if (human.getDamage() != 0) {
				deathTime =   (int) (((Math.sqrt(human.getHP()*human.getHP()+0.498*human.getHP())
						 -human.getDamage())-human.getDamage())/0.249);
						//human.getHP() / human.getDamage();
				 
			} else {
				deathTime = -1;
			}
		} else {
			deathTime = -1;
		}
		return deathTime;
		
		
		
		
		
	}

	public static boolean canBeSaved(Human human, int timeStep) {
		int deathTime = getDeathTime(human, timeStep);
		if (human.isBuriednessDefined()) {
			if (human.getBuriedness() > deathTime) {
				return false;
			}
		}
		return true;
	}

	public static List<Building> ignoreBurnedBuildings(List<Building> buildings) {
		List<Building> result = new ArrayList<Building>();
		for (Building next : buildings) {
			if (!next.isOnFire()) {
				if (next.isFierynessDefined()) {
					if (Fieryness.UNBURNT == next.getFierynessEnum()
							|| Fieryness.WATER_DAMAGE == next
									.getFierynessEnum()) {
						result.add(next);
					}
				} else {
					result.add(next);
				}
			}
		}
		return result;
	}

	public static List<Building> ignoreBlacakedBuildings(List<Building> buildings, SEUAgent agent) {
		
		List<Building> result = new ArrayList<Building>();

		for(Building next:buildings){
			
			Road entrance=agent.getEntranceRoad(next);
			 
			
			if(entrance!=null&&entrance.isBlockadesDefined()&&entrance.getBlockades().size()>0)
			result.remove(next);//niu 移除
		}
		
		return result;
		
	}
	// 房屋
	public static List<Building> ignoreBurningBuildings(List<Building> buildings) {
		List<Building> result = new ArrayList<Building>();
		for (Building next : buildings) {
			if (!next.isOnFire()) {
				if (next.isFierynessDefined()) {
					if (Fieryness.UNBURNT == next.getFierynessEnum()
							|| Fieryness.WATER_DAMAGE == next
									.getFierynessEnum()
							|| Fieryness.MINOR_DAMAGE == next
									.getFierynessEnum()
							|| Fieryness.MODERATE_DAMAGE == next
									.getFierynessEnum()
							|| Fieryness.SEVERE_DAMAGE == next
									.getFierynessEnum()) {
						result.add(next);
					}
				} else {
					result.add(next);
				}
			}
		}
		return result;
	}
}
