package seu.agent.Firebrigade;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.object.SEUWorldModel;
import seu.object.Partition.FirePartition;

public class FireTaskManager {

	private final List<Building> burningBuildings;
	private final List<FirePartition> underFirePartitions;
	private final List<EntityID> fireBrigades;

	public FireTaskManager(SEUWorldModel world) {
		burningBuildings = new ArrayList<Building>();
		underFirePartitions = new ArrayList<FirePartition>();
		fireBrigades = new ArrayList<EntityID>();
		Collection<StandardEntity> fBrigades = world
				.getEntitiesOfType(StandardEntityURN.FIRE_BRIGADE);
		for (StandardEntity next : fBrigades) {
			EntityID f = next.getID();
			fireBrigades.add(f);
		}
	}

	public void updateState(SEUWorldModel world, int timeStep) {
		updateBurningBuildings(timeStep, world);
		updateUnderFirePartitions(world);
		assignFireBrigadesToPartitions();

	}

	private void assignFireBrigadesToPartitions() {
		// System.out.println("Fire brigades " + fireBrigades.size());
		double numberOfFireBrigades = fireBrigades.size();
		double totalBurningArea = 0;
		for (FirePartition next : underFirePartitions) {
			totalBurningArea += next.getSize();
		}
		int agentIndex = 0;
		for (FirePartition next : underFirePartitions) {
			double numberOfAgentsShouldGo = Math.round(next.getSize()
					/ (totalBurningArea / numberOfFireBrigades));
			for (int i = 0; i < numberOfAgentsShouldGo; i++) {
				if (agentIndex < fireBrigades.size()) {
					next.assignAgent(fireBrigades.get(agentIndex));
					agentIndex++;
				}
			}
		}
	}

	private void updateBurningBuildings(int currentTimeStep, SEUWorldModel world) {
		burningBuildings.clear();
		Collection<StandardEntity> e = world
				.getEntitiesOfType(StandardEntityURN.BUILDING);
		for (StandardEntity next : e) {
			if (next instanceof Building) {
				Building b = (Building) next;
				if (b.isOnFire()) {
					burningBuildings.add(b);
				}
			}
		}
	}

	public void printFireRegions() {
		// int count = 1;
		// System.out.println("--------------------------");
		// System.out.println("Number of burning buildings: " +
		// burningBuildings.size());
		// System.out.println(" Number of under fire regions " +
		// underFireRegions.size());
		// for (FireRegion region : underFireRegions) {
		// List<Building> temp = region.getBuildings();
		// System.out.println("Fire region : " + count);
		// System.out.println("Size : " + region.getSize());
		// System.out.println("Agents : " + region.getAssignedAgents().size());
		// for (Building next : temp) {
		// System.out.println("Building " +next);
		// System.out.println("temp      "+next.getTemperature());
		// System.out.println("fieryness "+next.getFieryness());
		// System.out.println("floors "+next.getFloors());
		// System.out.println("area "+next.getGroundArea());
		// }
		// count++;
		// }
	}

	public void printBurningBuildings() {
		// int count = 1;
		// System.out.println("--------------------------");
		// System.out.println("Number of burning buildings: " +
		// burningBuildings.size());
		// System.out.println(" Number of under fire regions " +
		// underFireRegions.size());
		// for (Building : burningBuildings) {
		// List<Building> temp = region.getBuildings();
		// System.out.println("Fire region : " + count);
		// System.out.println("Size : " + region.getSize());
		// System.out.println("Agents : " + region.getAssignedAgents().size());
		// for (Building next : temp) {
		// // System.out.println("Building " +next);
		// // System.out.println("temp      "+next.getTemperature());
		// // System.out.println("fieryness "+next.getFieryness());
		// // System.out.println("floors "+next.getFloors());
		// // System.out.println("area "+next.getGroundArea());
		// }
		// count++;
		// }
	}

	private void updateUnderFirePartitions(SEUWorldModel world) {

		underFirePartitions.clear();
		ArrayList<Building> buildings = new ArrayList<Building>();
		boolean[] check = new boolean[burningBuildings.size()];

		int k = 0;
		if (!burningBuildings.isEmpty()) {
			for (Building building : burningBuildings) {
				buildings.add(building);
				check[k] = true;
				k++;
			}
			for (int i = 0; i < buildings.size(); i++) {
				if (check[i]) {
					Building buildingCenter = buildings.get(i);
					check[i] = false;
					List<Building> fireSiteBuildings = new ArrayList<Building>();
					fireSiteBuildings.add(buildingCenter);
					for (int j = 0; j < buildings.size(); j++) {
						Building buildingOther = buildings.get(j);
						if (isNeighbourBuilding(buildingCenter, buildingOther,
								world) && check[j]) {
							fireSiteBuildings.add(buildingOther);
							check[j] = false;
						}
					}
					FirePartition fireRegion = new FirePartition(fireSiteBuildings,
							buildingCenter);
					underFirePartitions.add(fireRegion);

				}
			}

		}

	}

	private boolean isNeighbourBuilding(Building center, Building building,
			SEUWorldModel world) {
		return false;
	}

	public boolean contains(Building building) {

		for (Building nextBurningBuildings : burningBuildings) {
			if (nextBurningBuildings.getID().getValue() == building.getID()
					.getValue()) {
				return true;
			}
		}
		return false;
	}

	public List<FirePartition> getUnderFirePartitions() {

		return underFirePartitions;
	}

}
