package seu.agent.Firebrigade;

import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumSet;
import java.util.List;

import rescuecore2.messages.Command;
import rescuecore2.misc.Pair;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUCentre;
import seu.assignment.HungarianAssignment;
import seu.message.MessagePriority;
import seu.message.Type.AgentPositionMessage;
import seu.message.Type.AssignFiredBuildingMessage;
import seu.message.Type.AssignImportantBuildingsMessage;
import seu.message.Type.BuildingIsBurningMessage;
import seu.message.Type.GasStationIsBurningMessage;//niu
import seu.message.Type.BuildingCloseToGasStationIsBurningMessage;//niu
import seu.message.Type.BuildingIsExploredMessage;
import seu.message.Type.BuildingIsExtinguishedMessage;
import seu.message.Type.ExtinguishFireTaskMessage;
import seu.message.Type.ImportantBuildingsMessage;
import seu.message.Type.Message;
import seu.message.Type.MessageType;
import seu.object.SEUWorldModel;
import seu.object.Partition.FirePartition;

/**
 * SEU centre agent.
 */
public class SEUFireStation extends SEUCentre {

	// private static final Log LOG = LogFactory.getLog(SEUFireStation.class);

	private static final String MAX_WATER_KEY = "fire.tank.maximum";
	private static final String MAX_DISTANCE_KEY = "fire.extinguish.max-distance";
	private static final String MAX_POWER_KEY = "fire.extinguish.max-sum";
	//protected Collection<Pair<EntityID, Fieryness>> buildings=new ArrayList<Pair<EntityID, Fieryness>>();//niu
	//protected Collection<Pair<EntityID,EntityID>> agents=new ArrayList<Pair<EntityID,EntityID>>();//niu
	protected List<EntityID> buildings=new ArrayList<EntityID>();
//	protected List<EntityID> agents=new ArrayList<EntityID>();
	protected List<FireBrigade> agents=new ArrayList<FireBrigade>();
	protected List<EntityID> locations=new ArrayList<EntityID>();
	protected List<Fieryness> fierynesses=new ArrayList<Fieryness>();
	double[][] costMatrix;
	FireTaskManager fireTaskManager;

	public SEUFireStation() {
		super();
	}

	@Override
	public String toString() {
		return "SEU Fire Station";
	}

	@Override
	protected void postConnect() {
		super.postConnect();

		this.world = (SEUWorldModel) super.model;

		world.indexClass(StandardEntityURN.BUILDING, StandardEntityURN.REFUGE);

		config.getIntValue(MAX_WATER_KEY);
		config.getIntValue(MAX_DISTANCE_KEY);
		config.getIntValue(MAX_POWER_KEY);

		fireTaskManager = new FireTaskManager(world);

	}

	@Override
	protected void thinkAndAct() {

		// sendExtinguishFireTasks();

	}
	@Override
	protected void think(int time, ChangeSet changed, Collection<Command> heard) {
		super.think(time, changed, heard);
		if (time == startTime) {
			int[] channels;
			setCommunicationCondition();
			channels = getChannelsToSubscribe();
			sendSubscribe(time, channels);
		}
		if(timeStep%10==0){
			buildings.clear();
			agents.clear();
			//locations.clear();;
			fierynesses.clear();;
		}
		if(timeStep%10==2 && !buildings.isEmpty()){
			agents=getAllFirebrigades();
			costMatrix = makingCostMatrix(buildings, agents);//花費矩阵
			 int[] assignment = computeVectorAssignments(costMatrix);//与房屋列表对应的FB烈表 
			try{
			 for(int i=0;i<assignment.length;i++){
	//输出对应关系反了？
				 AssignImportantBuildingsMessage message=new AssignImportantBuildingsMessage (
						 buildings.get(assignment[i]),transidoldtonew(agents.get(i).getID().getValue()),timeStep);
				
				 System.out.println("send AssignImportantBuildingsMessage!!!   bID "+
				 message.getBuildingId()+" AID "+	agents.get(i).getID()); 
				 sendMessage(message, MessagePriority.High);
			 }
			}catch(Exception e){
				System.err.println("array index outof bound -1");
			}
		}
		
	}
	
	public int[] computeVectorAssignments(double[][] costMatrix) {
        HungarianAssignment hungarianAssignment = new HungarianAssignment(costMatrix);
        int[] temp = hungarianAssignment.execute();
        System.out.println("temp length "+temp.length); 
        int[] result = null;
        List<Integer> tempList = new ArrayList<Integer>();

        if (temp != null) {
            for (int i : temp) {
                if (i != -1) {
                	//System.out.println("temp  "+i); 
                    tempList.add(i);
                }
            }

            result = new int[tempList.size()];
            System.out.println("tempList.size() ??"+tempList.size()); 
            try{
                for(int i=0;i<tempList.size();i++){
                	
                	result[i]=tempList.get(i);
                	System.out.println("shishenme ??  "+"  i="+i+" templist i="+tempList.get(i)+" bid "+buildings.get(tempList.get(i))+" aid "+agents.get(i)); 
                }
                }catch(Exception e){}

        }
//对应于房屋列表的智能体序号列表（如第1个房屋分配给第x个智能体）
        return result;
       

    }
    public double[][] makingCostMatrix( List<EntityID> buildings,  List<FireBrigade> agents) {
        int n = agents.size();
        int m = buildings.size();
        System.out.println("huafei");
        double[][] costMatrix = new double[n][m];
        
        Fieryness f=StandardEntityConstants.Fieryness.HEATING;
        for (int i = 0; i < n; i++) {
        	StandardEntity location=agents.get(i).getPosition(world);
            for (int j = 0; j < m; j++) {
            	
                double distance=model.getDistance(location.getID(), buildings.get(j));
                Building b=(Building)world.getEntity(buildings.get(j));
                
                double buildingimportance=b.getTotalArea()*(fierynesses.get(j).equals(f) ? 100:10);
                costMatrix[i][j] = buildingimportance/distance;
                //System.out.println("cost"+i+"  "+j+costMatrix[i][j]);
            }
        }


        return costMatrix;
    }
	public void sendExtinguishFireTasks() {//unused
		List<FirePartition> underFireRegions = fireTaskManager
				.getUnderFirePartitions();
		for (FirePartition nextRegion : underFireRegions) {
			List<EntityID> assignedAgents = nextRegion.getAssignedAgents();
			Building tagetBuilding = nextRegion.getTagetBuilding();
			for (EntityID next : assignedAgents) {
				ExtinguishFireTaskMessage message = new ExtinguishFireTaskMessage(
						tagetBuilding.getID().getValue(), transidoldtonew(next
								.getValue()));
				// System.out.println(" Sending ExtinguishFireTaskMessage message to agent "
				// + next.getValue());
				sendMessage(message, seu.message.MessagePriority.High);
			}

		}
	}

	@Override
	protected void processMessage(Message message) {
		if (message instanceof BuildingIsExploredMessage) {
			BuildingIsExploredMessage explorationMessage = (BuildingIsExploredMessage) message;
			processExplorationMessage(explorationMessage);
		}

		if (message instanceof BuildingIsBurningMessage) {
			//System.out.println("hraed BuildingIsburningMessage ");
			BuildingIsBurningMessage buildingIsBurningMessage = (BuildingIsBurningMessage) message;
			processBuildingIsBurningMessage(buildingIsBurningMessage);
		}
	
		if (message instanceof BuildingIsExtinguishedMessage) {
			BuildingIsExtinguishedMessage buildingIsExtinguishedMessage = (BuildingIsExtinguishedMessage) message;
			processBuildingIsExtinguishedMessage(buildingIsExtinguishedMessage);
		}
		if(message instanceof ImportantBuildingsMessage ){
			//if( timeStep-((ImportantBuildingsMessage) message).getTimeStep()<5 && timeStep%10<5){//延迟消息不处理
			
			ImportantBuildingsMessage importantBuildingsMessage=(ImportantBuildingsMessage) message;
			processImportantBuildingsMessage(importantBuildingsMessage);
			//}
		}
//		if (message instanceof AgentPositionMessage) {
//			AgentPositionMessage agentPositionMessage = (AgentPositionMessage) message;
//			processAgentPositionMessage(agentPositionMessage);
//		}
	}
	protected void processImportantBuildingsMessage(ImportantBuildingsMessage message){//buchuli chaoshi?
		//jia ru lie biao
		System.out.println(timeStep+"FS process  ImportantBuildingsMessage "+getID());
		EntityID buildingID=message.getBuildingId();	
		int fieryness=message.getFieryness();
		EntityID agentID=transidnewtoold(message.getAgentId().getValue());
		EntityID locationID=message.getLocationId();

		//System.out.println("agentID"+ agentID+"add agent ");
		if(!buildings.contains(buildingID)){
			
			buildings.add(buildingID);
			fierynesses.add(StandardEntityConstants.Fieryness.values()[fieryness]);
		}
//		if(!agents.contains(agentID)){
//			System.out.println("agentID"+ agentID+"add agent "+message.getAgentId());
//			agents.add(agentID);             
//			locations.add(locationID);
//		}
	}
	/*@Override
	public List<MessageType> getMessagesToListen() {
		List<MessageType> types;

		types = new ArrayList<MessageType>();
		
		types.add(MessageType.ImportantBuildingsMessage);//niu 
		types.add(MessageType.AgentPositionMessage);//niu
//		types.add(MessageType.AssignFiredBuildingMessage );//niu 分配消息
//		types.add(MessageType.BuildingIsExtinguishedMessage);
//		types.add(MessageType.BuildingIsBurningMessage);
		return types;
	}*/
	@Override
	protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.FIRE_STATION);
	}

}
