package seu.agent.Firebrigade;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

//import javolution.util.FastMap;


import org.uncommons.maths.number.NumberGenerator;
import org.uncommons.maths.random.GaussianGenerator;

import rescuecore2.standard.entities.Refuge;
import seu.message.CommunicationCondition;
import seu.object.SEUWorldModel;
import seu.object.Building.SEUBuilding;

public class Simulator {

    private SEUWorldModel world;

    public static float GAMMA = 0.2f;
    public static float WATER_COEFFICIENT = 20f;
    private static final float AIR_TO_BUILDING_COEFFICIENT = 45f;
    NumberGenerator<Double> burnRate = new GaussianGenerator(0.15, 0.02, new java.util.Random(23));


    public Simulator(SEUWorldModel world) {
        this.world = world;


    }

    /**
     * this method update building fuel and energy.
     * and get new fieriness and temperature like main fireSimulator.
     */
    public void update() {
        burn();
        cool();
        exchangeBuilding();
        cool();
    }

    private void burn() {
        double burnRate = this.burnRate.nextValue();
        for (SEUBuilding b : world.getSeuBuildings()) {
          
            if (b.getEstimatedTemperature() >= b.getIgnitionPoint() && b.isFlammable() && b.getFuel() > 0) {
                float consumed = b.getConsume(burnRate);
                if (consumed > b.getFuel()) {
                    consumed = b.getFuel();
                }
                b.setEnergy(b.getEnergy() + consumed);
//                energyHistory.registerBurn(b, consumed);
                b.setFuel(b.getFuel() - consumed);
                b.setPrevBurned(consumed);
            } else {
                b.setPrevBurned(0f);
            }
            
        }
    }

    private void exchangeBuilding() {
        for (SEUBuilding b : world.getSeuBuildings()) {
            exchangeWithAir(b);
        }
        Map<SEUBuilding, Double> radiation = new HashMap<SEUBuilding, Double>();
        for (SEUBuilding b : world.getSeuBuildings()) {
            double radEn = b.getRadiationEnergy();
            radiation.put(b, radEn);
        }
        for (SEUBuilding b : world.getSeuBuildings()) {
            
            double radEn = radiation.get(b);
//            if (world.isCommunicationLess() || world.isCommunicationMedium() || world.isCommunicationLow()){
//            		
//                radEn /= 2.311738394;
//            } 和通信有关?
            List<SEUBuilding> bs = b.getConnectedBuilding();
            List<Float> vs = b.getConnectedValues();

            for (int c = 0; c < vs.size(); c++) {
                
                double oldEnergy = bs.get(c).getEnergy();
                double connectionValue = vs.get(c);
                double a = radEn * connectionValue;
                double sum = oldEnergy + a;
                bs.get(c).setEnergy(sum);

            }
            b.setEnergy(b.getEnergy() - radEn);

        }
    }

    private void exchangeWithAir(SEUBuilding b) {
// Give/take heat to/from air cells

        double oldTemperature = b.getEstimatedTemperature();
        double oldEnergy = b.getEnergy();



        if (oldTemperature > 100) {
            b.setEnergy(oldEnergy - (oldEnergy * 0.042));
        }
    }

    private void cool() {
        for (SEUBuilding building : world.getSeuBuildings()) {
            waterCooling(building);
        }
    }

    private void waterCooling(SEUBuilding b) {
        double lWATER_COEFFICIENT = (b.getEstimatedFieryness() > 0 && b.getEstimatedFieryness() < 4 ? WATER_COEFFICIENT : WATER_COEFFICIENT * GAMMA);
        if (b.getWaterQuantity() > 0) {
            double dE = b.getEstimatedTemperature() * b.getCapacity();
            if (dE <= 0) {
                return;
            }
            double effect = b.getWaterQuantity() * lWATER_COEFFICIENT;
            int consumed = b.getWaterQuantity();
            if (effect > dE) {
                double pc = 1 - ((effect - dE) / effect);
                effect *= pc;
                consumed *= pc;
            }
            b.setWaterQuantity(b.getWaterQuantity() - consumed);
            b.setEnergy(b.getEnergy() - effect);
        }
    }


    

}
