package seu.agent.Path;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.PriorityQueue;

import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.Police.ClearPathTaskComparator;
import seu.agent.Police.PoliceAgentTask;
import seu.message.Type.InformTeamExtinguishingFireMessage;
import seu.message.Type.InformTeamSavingCivilianMessage;
import seu.object.SEUWorldModel;

public class ClearPathTaskManager {

	private final List<PoliceAgentTask> policeForceTasks;
	private final AmbulanceTeamPathManager ambulanceTeamPathManager;
	private final FireBrigadePathManager fireBrigadePathManager;

	private final PriorityQueue<RequiredClearPath> queue;
	Comparator<RequiredClearPath> comparator;

	public ClearPathTaskManager(SEUWorldModel world) {
		policeForceTasks = new ArrayList<PoliceAgentTask>();
		Collection<StandardEntity> polices = world
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE);
		for (StandardEntity next : polices) {
			EntityID p = next.getID();
			PoliceAgentTask police = new PoliceAgentTask(p);
			policeForceTasks.add(police);
		}

		ambulanceTeamPathManager = new AmbulanceTeamPathManager();
		fireBrigadePathManager = new FireBrigadePathManager();

		comparator = new ClearPathTaskComparator();
		queue = new PriorityQueue<RequiredClearPath>(10, comparator);

	}

	public void updateState(SEUWorldModel world, int timeStep) {
		ambulanceTeamPathManager.updateState(world, timeStep);
		fireBrigadePathManager.updateState(world, timeStep);

		queue.clear();

		addRequiredClearPaths(ambulanceTeamPathManager.getDesiredPaths());
		addRequiredClearPaths(fireBrigadePathManager.getDesiredPaths());

		assignAgentsToClearPathTasks();
	}

	private void addRequiredClearPaths(List<RequiredClearPath> requiredPaths) {
		for (RequiredClearPath next : requiredPaths) {
			queue.add(next);
		}
	}

	public void addSavingCivilianMessage(
			InformTeamSavingCivilianMessage message, int timeStep) {
		ambulanceTeamPathManager.addMessage(message, timeStep);
	}

	public void addExtinguishingFireMessage(
			InformTeamExtinguishingFireMessage message, int timeStep) {
		fireBrigadePathManager.addMessage(message, timeStep);
	}

	@SuppressWarnings("unused")
	private void assignAgentsToClearPathTasks() {
		for (PoliceAgentTask next : policeForceTasks) {
			RequiredClearPath reqPath = queue.poll();
		}

	}

	public List<PoliceAgentTask> getPoliceForceTasks() {
		return policeForceTasks;
	}

}
