package seu.agent.Police;

import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import rescuecore2.messages.Command;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUCentre;
import seu.agent.Path.ClearPathTaskManager;
import seu.message.Type.AgentIsBuriedMessage;
import seu.message.Type.AgentPositionMessage;
import seu.message.Type.AssignFiredBuildingMessage;
import seu.message.Type.AssignRefugeMessage;
import seu.message.Type.BuildingIsBurningMessage;
import seu.message.Type.GasStationIsBurningMessage;
import seu.message.Type.BuildingCloseToGasStationIsBurningMessage;//niu
import seu.message.Type.ClearPathIsAssignedMessage;
import seu.message.Type.InformTeamMessage;
import seu.message.Type.Message;
import seu.message.Type.MessageType;
import seu.object.SEUWorldModel;
import seu.object.Building.BuildingInfo;
import seu.utilities.FireSite;

public class SEUPoliceOffice extends SEUCentre {

	private ClearPathTaskManager clearPathTaskManager;
	private final ArrayList<StandardEntity> refuge = new ArrayList<StandardEntity>();
	private final ArrayList<StandardEntity> pf = new ArrayList<StandardEntity>();
	private final HashSet<StandardEntity> assignedPF = new HashSet<StandardEntity>();
	protected List<Building> buringbuilding = new ArrayList<Building>();
	public ArrayList<FireSite> fireSites = new ArrayList<FireSite>();
	private boolean isAssigned = false;

	public SEUPoliceOffice() {
		super();
	}

	@Override
	public String toString() {
		return "SEU Police Office";
	}

	@Override
	protected void postConnect() {
		super.postConnect();

		this.world = (SEUWorldModel) super.model;

		world.indexClass(StandardEntityURN.BUILDING, StandardEntityURN.REFUGE);

		clearPathTaskManager = new ClearPathTaskManager(world);

	}

	@Override
	protected void think(int time, ChangeSet changed, Collection<Command> heard) {
		try {
			if (time == startTime) {
				int[] channels;
				setCommunicationCondition();
				channels = getChannelsToSubscribe();
				idtoid();
				sendSubscribe(time, channels);
			}
			if (time >= startTime) {
				if (time < 10) {
					idtoid();
				}
				processHearing(heard);
			}
			setTimeStep(time);
			if (time > startTime + 2) {
				if (!isAssigned) {
					AssignRefuge();
				}
				updateFireSites();
				sendMessagesFromQueue();
			}
		} catch (Exception e) {
		}
	}

	public void AssignFiredBuilding(FireSite fireSite) {

		HashSet<Building> temp;
		ArrayList<Building> neighborBuilding = new ArrayList<Building>();
		temp = fireSite.getSiteNighbors();
		if (temp != null && !temp.isEmpty()) {
			Iterator<Building> it = temp.iterator();
			while (it.hasNext()) {
				Building t = it.next();
				neighborBuilding.add(t);
			}
		} else {
			temp = fireSite.getFires();
			Iterator<Building> it = temp.iterator();
			while (it.hasNext()) {
				Building t = it.next();
				neighborBuilding.add(t);
			}
		}
		// Building targetBuilding = null;
		EntityID targetID = null;
		if (neighborBuilding != null && !neighborBuilding.isEmpty()) {
			for (Building next : neighborBuilding) {
				Road entrance = null;
				if (this.getEntranceRoad(next) != null) {
					entrance = this.getEntranceRoad(next);
					targetID = entrance.getID();
					continue;
				}
			}
		}
		if (targetID == null) {
			// System.out.println("TargetBuilding is null");
			return;
		}
		StandardEntity targetPF = null;
		int dis = Integer.MAX_VALUE;
		for (StandardEntity p : pf) {
			Human man = (Human) p;
			if (man.isBuriednessDefined() && man.getBuriedness() > 0) {
				continue;
			}
			// System.out.println(assignedPF);
			if (assignedPF.contains(p))
				continue;
			if (dis > world.getDistance(targetID, p.getID())) {
				targetPF = p;
				dis = world.getDistance(targetID, p.getID());

			}
		}
		if (targetPF != null && !assignedPF.contains(targetPF)) {
			AssignFiredBuildingMessage message = new AssignFiredBuildingMessage(
					transidoldtonew(targetPF.getID().getValue()), targetID);
			// System.out.println(timeStep + "Send" + message);
			// assignedPF.add(targetPF);
			sendMessage(message, seu.message.MessagePriority.High);
			// System.out.println(message);
		} else {
			// System.out.println("No PF is Assigned!");
		}

	}

	public void AssignRefuge() {
		isAssigned = true;
		HashSet<StandardEntity> temp;
		temp = (HashSet<StandardEntity>) world
				.getEntitiesOfType(StandardEntityURN.REFUGE);
		Iterator<StandardEntity> it = temp.iterator();
		while (it.hasNext()) {
			StandardEntity t = it.next();
			refuge.add(t);
		}
		temp = (HashSet<StandardEntity>) world
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE);
		it = temp.iterator();
		while (it.hasNext()) {
			StandardEntity t = it.next();
			pf.add(t);
		}
		for (StandardEntity r : refuge) {

			StandardEntity targetPF = null;
			int dis = Integer.MAX_VALUE;
			for (StandardEntity p : pf) {
				Human man = (Human) p;
				if (man.isBuriednessDefined() && man.getBuriedness() > 0) {
					continue;
				}
				if (assignedPF.contains(p))
					continue;
				if (dis > world.getDistance(r.getID(), p.getID())) {
					targetPF = p;
					dis = world.getDistance(r.getID(), p.getID());

				}
			}
			if (targetPF != null && !assignedPF.contains(targetPF)) {
				AssignRefugeMessage message = new AssignRefugeMessage(
						transidoldtonew(targetPF.getID().getValue()), r.getID());
				// System.out.println(timeStep + "Send" + message);
				assignedPF.add(targetPF);
				sendMessage(message, seu.message.MessagePriority.High);
			}
		}
	}

	@Override
	protected void thinkAndAct() {

		// if (getTimeStep() % 7 == 0 && getTimeStep() > 3) {
		clearPathTaskManager.updateState(world, getTimeStep());
		sendClearPathTasks();
		// fireTaskManager.printFireRegions();
		// }
	}

	public void sendClearPathTasks() {
		List<PoliceAgentTask> policeForceTasks = clearPathTaskManager
				.getPoliceForceTasks();
		for (PoliceAgentTask nextPolice : policeForceTasks) {
			ClearPathIsAssignedMessage message = new ClearPathIsAssignedMessage(
					nextPolice.getClearPathTask().getRoadIdStart(), nextPolice
							.getClearPathTask().getRoadIdFinish(),
					transidoldtonew(nextPolice.getID().getValue()));
			sendMessage(message, seu.message.MessagePriority.High);
		}
	}

	@Override
	protected void processMessage(Message message) {
		if (message instanceof AgentIsBuriedMessage) {

			AgentIsBuriedMessage agentIsBuriedMessage = (AgentIsBuriedMessage) message;
			try {
				processAgentIsBuriedMessage(agentIsBuriedMessage);
			} catch (Exception e) {

			}
		}
		if (message instanceof InformTeamMessage) {
			InformTeamMessage inforTeamMessage = (InformTeamMessage) message;
			try {
				processInformTeamMessage(inforTeamMessage);
			} catch (Exception e) {

			}

		}
		if (message instanceof AgentPositionMessage) {
			AgentPositionMessage agentPositionMessage = (AgentPositionMessage) message;
			processAgentPositionMessage(agentPositionMessage);
		}
		if (message instanceof BuildingIsBurningMessage) {
			BuildingIsBurningMessage buildingIsBurningMessage = (BuildingIsBurningMessage) message;
			processBuildingIsBurningMessage(buildingIsBurningMessage);
		}
		if (message instanceof GasStationIsBurningMessage) {
			GasStationIsBurningMessage gasStationIsBurningMessage = (GasStationIsBurningMessage) message;
			processGasStationIsBurningMessage(gasStationIsBurningMessage);
		}
		if (message instanceof BuildingCloseToGasStationIsBurningMessage) {//niu
			BuildingCloseToGasStationIsBurningMessage buildingCloseToGasStationIsBurningMessage = (BuildingCloseToGasStationIsBurningMessage) message;
			processBuildingIsBurningMessage(buildingCloseToGasStationIsBurningMessage);
		}
	}

	@Override
	protected void processBuildingIsBurningMessage(
			BuildingIsBurningMessage message) {
		// System.out.println(timeStep + message.toString());

		try {
			EntityID buildingId = message.getBuildingId();
			StandardEntity s = world.getEntity(buildingId);
			if (s instanceof Building) {
				Building building = (Building) world.getEntity(buildingId);
				if (building != null) {
					building.setFieryness(message.getFieryness());
					building.setTemperature(message.getTemperature());
					if (!buringbuilding.contains(building))
						buringbuilding.add(building);
				}
			}
		} catch (Exception e) {
			System.err.println("There is No Such Building");
		}

	}
	
	@Override
	protected void processInformTeamMessage(InformTeamMessage message) {
		// System.out.println(timeStep + message.toString());
		try {
			EntityID agentId = transidnewtoold(message.getAgentId().getValue());
			// System.out.print(agentId);
			EntityID location = message.getLocationId();
			StandardEntity e = world.getEntity(agentId);

			Human man = (Human) e;
			man.setPosition(location);
			man.setBuriedness(message.getBuriedness());

		} catch (Exception e) {
			System.err.println(this.getTimeStep() + message.toString()
					+ "is error");
		}
	}

	protected void processAgentIsBuriedMessage(AgentIsBuriedMessage message) {
		EntityID id = transidnewtoold(message.getAgentId().getValue());
		StandardEntity entity = world.getEntity(id);
		Human man = (Human) entity;
		man.setBuriedness(message.getBuriedness());
	}

	@Override
	protected EnumSet<StandardEntityURN> getRequestedEntityURNsEnum() {
		return EnumSet.of(StandardEntityURN.POLICE_OFFICE);
	}

	/*@Override
	public List<MessageType> getMessagesToListen() {
		List<MessageType> types;

		types = new ArrayList<MessageType>();

		// types.add(MessageType.AgentIsStuckMessage);
		types.add(MessageType.BuildingIsBurningMessage);
		types.add(MessageType.GasStationIsBurningMessage);
		types.add(MessageType.InformTeamMessage);
		types.add(MessageType.AgentIsBuriedMessage);
		types.add(MessageType.AgentPositionMessage);//niu
		return types;
	}*/

	private void updateFireSites() {
		// 1, add fire or construct some new fire sites
		if (buringbuilding != null) {
			for (Building b : buringbuilding) {
				if (b.getFieryness() >= 5 && b.getFieryness() <= 7)
					continue;
				boolean isNewSite = true;
				BuildingInfo bi = new BuildingInfo(b);
				ArrayList<Integer> siteHaveThisFire = new ArrayList<Integer>();
				for (int i = 0; i < fireSites.size(); i++) {
					FireSite fs = fireSites.get(i);
					if (fs.addFire(bi) != -1) {
						siteHaveThisFire.add(new Integer(i));
						isNewSite = false;
					}
				}
				if (isNewSite) {
					FireSite newSite = new FireSite(bi, getModel());
					fireSites.add(newSite);
					AssignFiredBuilding(newSite);
					continue;
				}
				// 2, merge any overlapped sites
				if (siteHaveThisFire.size() > 1) {
					FireSite mergeSite = fireSites.get(siteHaveThisFire.get(0)
							.intValue());
					for (int i = 1; i < siteHaveThisFire.size(); i++) {
						mergeSite.mergeWith(fireSites.get(siteHaveThisFire.get(
								i).intValue()));
					}
					ArrayList<FireSite> needToRemove = new ArrayList<FireSite>();
					for (int i = 0; i < siteHaveThisFire.size(); i++)
						needToRemove.add(fireSites.get(siteHaveThisFire.get(i)
								.intValue()));
					fireSites.removeAll(needToRemove);
					fireSites.add(mergeSite);
				}
			}
		}
		// if (fireSites != null) {
		// for (FireSite p : fireSites) {
		// gg++;
		// System.out.println("    time  " + timeStep + "       " + gg
		// + "   getSiteNighbors  " + p.getSiteNighbors()
		// + "    fires  " + p.getFires());
		// }
		// }

	}
}
