package seu.agent;

import java.awt.Point;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import rescuecore2.Constants;
import rescuecore2.config.Config;
import rescuecore2.messages.Command;
import rescuecore2.messages.control.KASense;
import rescuecore2.misc.Pair;
import rescuecore2.misc.geometry.GeometryTools2D;
import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.misc.geometry.Vector2D;
import rescuecore2.standard.components.StandardAgent;
import rescuecore2.standard.entities.AmbulanceCentre;
import rescuecore2.standard.entities.AmbulanceTeam;
import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Blockade;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.Edge;
import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.FireStation;
import rescuecore2.standard.entities.GasStation;
import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.Hydrant;
import rescuecore2.standard.entities.PoliceForce;
import rescuecore2.standard.entities.PoliceOffice;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.Road;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.standard.entities.StandardPropertyURN;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.standard.messages.AKSpeak;
import rescuecore2.worldmodel.ChangeSet;
import rescuecore2.worldmodel.Entity;
import rescuecore2.worldmodel.EntityID;
import rescuecore2.worldmodel.Property;
import seu.agent.BasicTask.BasicTask;
import seu.agent.BasicTask.CheckGasStationTask;
import seu.agent.BasicTask.CheckRefugeTask;
import seu.agent.BasicTask.ClearPath;
import seu.agent.BasicTask.RunAwayFromGasStationTask;
import seu.agent.BasicTask.State;
import seu.agent.command.seuClear;
import seu.agent.command.seuCommand;
import seu.agent.command.seuLoad;
import seu.agent.command.seuMove;
import seu.agent.command.seuMoveXY;
import seu.agent.command.seuRescue;
import seu.agent.command.seuRest;
import seu.agent.command.seuUnload;
import seu.config.Configuration;
import seu.message.CommunicationCondition;
import seu.message.MessageController;
import seu.message.MessageMemory;
import seu.message.MessagePriority;
import seu.message.MessageTranslator;
import seu.message.QueueMessage;
import seu.message.messageKeeper;
import seu.message.Type.AgentIsBuriedMessage;
import seu.message.Type.AgentIsStuckMessage;
import seu.message.Type.AgentPositionMessage;
import seu.message.Type.BuildingCloseToGasStationIsBurningMessage;
import seu.message.Type.BuildingIsBurningMessage;
import seu.message.Type.BuildingIsExploreMessage_at;
import seu.message.Type.BuildingIsExploreMessage_fb;
import seu.message.Type.BuildingIsExploreMessage_pf;
import seu.message.Type.BuildingIsExploredMessage;
import seu.message.Type.BuildingIsExtinguishedMessage;
import seu.message.Type.ChannelCondition;
import seu.message.Type.CivilianInformationMessage;
import seu.message.Type.CivilianIsSavedOrDeadMessage;
import seu.message.Type.CivilianMaybeMessage;
import seu.message.Type.ClearPathReason;
import seu.message.Type.GasStationIsBurningMessage;
import seu.message.Type.InformTeamMessage;
import seu.message.Type.Message;
import seu.message.Type.MessageCount;
import seu.message.Type.MessageType;
import seu.message.Type.PartitionAssignmentMessage;
import seu.message.newPart.MessageCount1;
import seu.message.newPart.MessageCount2;
import seu.message.newPart.MessageCount3;
import seu.message.newPart.MessageCount4;
import seu.message.newPart.MessageCount5;
import seu.message.newPart.MessageCount6;
import seu.message.newPart.MessageCount7;
import seu.message.newPart.MessageCount8;
import seu.message.newPart.MessageCount9;
import seu.object.Exploration;
import seu.object.SEUWorldModel;
import seu.object.Building.BuildingInfo;
import seu.object.Building.SEUBuilding;
import seu.object.Partition.EntityInPartition;
import seu.object.Partition.GroupingType;
import seu.object.Partition.PartitionMap;
import seu.object.Road.MultiBlockDistanceComparator;
import seu.object.Road.RoadInfo;
import seu.object.Road.RoadUtilities;
import seu.utilities.ACAMainRoad;
import seu.utilities.DistanceComparator;
import seu.utilities.FireSite;
import seu.utilities.Path;
import seu.utilities.PositionHistory;
import seu.utilities.PredictDamageTool;
import seu.utilities.Geometry.PositionLocate;
import seu.utilities.Search.AbstractSearch;
import seu.utilities.Search.AStar.AStarSearch;
import seu.utilities.Search.Cost.PathType;
import seu.utilities.Search.Node.Node;

/**
 * SEU's Top Level Agent
 */
public abstract class SEUAgent<E extends StandardEntity> extends
		StandardAgent<E> {
	public List<FireSite> fire;
	public static final double AGENT_RADIUS = 500;
	private static final int RANDOM_WALK_LENGTH = 3;
	private static final int MIN_DAMAGE_GO_TO_REFUGE = 0;
	private static final int DEFAULT_AGENT_THINK_TIME = 1000;
	private static final double AGENT_THINK_TIME_TOLERANCE = 0.8;
	private static final int DEFAULT_VIEW_DISTANCE = 30000;
	private static final int DEFAULT_RANDOM_WALK_PENALTY = 3;
	private static final int IGNORE_COMMANDS_AFTER = 2;
	protected static final int MAX_PATH_LENGTH = 30;

	@SuppressWarnings("unused")
	private static final String SAY_COMMUNICATION_MODEL = "kernel.standard.StandardCommunicationModel";
	private static final String MAX_PLATOON_CHANNELS_KEY = "comms.channels.max.platoon";
	private static final String SPEAK_COMMUNICATION_MODEL = "kernel.standard.ChannelCommunicationModel";
	protected static final String START_TIME_KEY = "kernel.agents.ignoreuntil";
	private static final String AGENT_THINK_TIME_KEY = "kernel.agents.think-time";
	private static final String VIEW_DISTANCE_KEY = "perception.los.max-distance";
	private static final String TIMESTEPS = "kernel.timesteps";
	private static final String DISTANCE_KEY = "clear.repair.distance";
	protected int viewDistance;
	// 只有Say通信时边走边说
	protected MessageTranslator sayMessageTransfer;
	protected Map<Integer, seuCommand> commands;
	protected int timeStep;
	protected int startTime;
	private int maxDistance;
	public ACAMainRoad aca;
	public List<Building> nofoundbuilding;
	public List<GasStation> GasStationBurned =new ArrayList<GasStation>();//niu着过火的,不再定期檢查 
	public List<GasStation> VisitedGasStation =new ArrayList<GasStation>();
	public List<Hydrant> ReachedHydrant=new ArrayList<Hydrant>();//niu
	private int FailInHydrantTime=0;//niu
	private Hydrant lastFailHydrant;//niu
	
	
	public int getStartTime() {
		return startTime;
		
		
	}

	public int reachGStime=0;//niu
	
	protected int totalSent;
	protected int totalReceived;
	protected int agentThinkTime;
	protected int randomWalkPenalty;
	protected int desiredPath;
	protected int simulationTime;
	protected SEUWorldModel world;
	protected State state;
	protected Collection<StandardEntity> unexploredBuildings;
	protected Collection<Pair<Building, Fieryness>> informedBurningBuildings;
	public int aftershocktime;
	protected Collection<Pair<Building, Integer>> oldbuildings;
	private KASense lastSense;
	protected MessageController messageController;
	protected MessageMemory messageMemory;
	protected ChangeSet changeSet;
	protected Set<EntityID> changedEntities;
	protected List<EntityID> observableBuildings;
	public List<EntityID> observableRefuge=new ArrayList<EntityID>();  //yuan
	
	@SuppressWarnings({ "rawtypes" })
	protected List<BasicTask> tasks;
	protected StandardEntity locationAtLastCycle;
	protected StandardEntityURN agentType;
	protected List<Building> burningBuildingsInChangeSet;
	
	public  List<Human> foundAgents;                 ////wan
	public Map<EntityID,EntityID> agentFirstPositionMap;///////wan 14.7.15
	public List<EntityID> ATFirstTasks;////////////wan
	public List<EntityID> PFFirstTasks;
	
	protected List<Building> heatingBuildings;
	
	protected Date decisionStartTime;
	protected List<Civilian> civiliansAround;
	public List<HumanInfo> foundCivilians;
	public Map<Integer, EntityID> locationHistory;
	public testChannel tChannel;
	public messageKeeper mKeeper;
	public CommunicationCondition communicationCondition;
	public List<Building> temp_maybebuilding;
	
	public List<Building> maybebu;
	public List<EntityID> NUMOfATOCCUR;
	public List<Road> safeRoad;
	public List<HumanInfo> getFoundCivilians() {
		return foundCivilians;
	}

	public PredictDamageTool predictDamageTool;
	// 死亡的智能体和Civilain
	private List<EntityID> deadHumans;

	public List<EntityID> getDeadHumans() {
		return deadHumans;
	}

	public void setSavedCivAndAgents(List<EntityID> savedCivAndAgents) {
	}

	// 记录到达的地点和时间
	PositionHistory positionHistory;
	/**
	 * The search algorithm.
	 */
	protected AbstractSearch search;
	/**
	 * Whether to use AKSpeak messages or not.
	 */
	protected boolean useSpeak;

	/**
	 * Construct a SEU Agent.
	 */
	protected SEUAgent() {
	}

	public int getNo() {
		if (world == null) {
			return 0;
		}
		List<EntityID> team = world.getTeam();
		if (team.contains(getID())) {
			return team.indexOf(getID());
		} else {
			return 0;
		}
	}

	public void superPostConnect() {
		super.postConnect();
	}

	// 记录上上周期，上一周期，当前周期智能体位置
	public Area latestPosition;
	public Area lastPosition;
	public Area currentPostion;

	//
	private State latestState;
	private State lastState;
	private State currentState;

	// 两个移动标志
	// 连续两个周期随机移动
	boolean moveOnce = false;
	boolean moveAgain = false;

	// 智能体状态信息
	State temp;

	// 重复次数
	int sum = 0;

	// 至少保留智能体三周期的位置
	private void checkPostion() {

		// 位置信息
		latestPosition = lastPosition;
		lastPosition = currentPostion;
		currentPostion = (Area) positionHistory.getCurrentLocation();
		//System.out.println(this.getTimeStep()+" time "+this.getID()+" CurrentLocation()"+currentPostion);

		// 状态信息
		latestState = lastState;
		lastState = currentState;
		currentState = state;

		// 在比赛开始之后进行状态赋值
		if (getMeAsHuman().isHPDefined()) {
			if (timeStep >= startTime && getMeAsHuman().getHP() > 0) {

				if (latestPosition != null && lastPosition != null
						&& currentPostion != null
						&& currentState != State.Inactive) {
					if ((latestPosition == lastPosition)
							&& (latestPosition == currentPostion)
							&& (latestState == lastState)
							&& (latestState == currentState)) {

						sum++;
						//
						// if (sum > 3)
						// System.out.println(me() + " Stay One Place For "
						// + (sum + 2) + " Cycles In " + timeStep);

					} else {
						sum = 0;
					}
				} else {
					sum = 0;
				}
			}
		}

	}

	private void moveOutOfThePlace() {//下列情况可以不动 

		// 如果正在加水
		if (me() instanceof FireBrigade && (location() instanceof Refuge ))
			return;
		// 如果正在灭火
		if (me() instanceof FireBrigade
				&& (state == State.Extinguishing || state == State.FillingWaterInRefuge||
						state == State.FillingWaterInHydrant))
			return;
		// 如果正在清障
		if (me() instanceof PoliceForce
				&& (state == State.ClearingAllBlocksAtLocation
						|| state == State.ClearingAnnoyingBlock
						|| state == State.ClearingBlock
						|| state == State.ClearingBlocksToPath
						|| state == State.ClearingCurrentRoad
						|| state == State.ClearingPath || state == State.ClearingRefugeEntrance)
				|| state == State.CleanAllAround || state == State.CleanNearest)
			return;
		// 如果正在救援
		if (me() instanceof AmbulanceTeam && (state == State.RescuingHuman))
			return;
		// 如果被埋或者被堵
		if (state == State.StuckResting || state == State.BuriedResting
				|| state == State.Waiting)
			return;

		if (moveOnce == false && moveAgain == false)
			temp = state;
		// 位置不变，且不满足上面的条件，判断是被卡住
		// 使用Random Walk碰撞走出卡住的位置
		if (moveAgain) {

			if (moveOnce) {
				moveOnce = false;
			} else {
				moveAgain = false;
			}

			List<EntityID> path = randomWalk();
			//System.out.println("move out of the palce");
			if (path == null)
				System.err.println("Random Walk Failed");
			sendMove(timeStep, path);

		}

		
		// 移动到当前位置的中心
		// 目前根据Berlin地图的道路长度大致设定了6个周期作为移动阈值
		// 最佳思路是得到该地图最大路径长度，除以智能体的最大移动速度
		// 不同地图设定不同的被卡阈值

		if (sum >= 4) {
			System.err.println(this.getMeAsHuman().getURN() + me()
					+ " Move to Centre in " + timeStep);

			state = State.MoveOutOfThePlace;

			moveOnce = true;
			moveAgain = true;

			List<EntityID> t = new ArrayList<EntityID>();
			t.add(currentPostion.getID());
			sendMove(timeStep, t, currentPostion.getX(), currentPostion.getY());
		//	 System.out.println(this.getTimeStep()+" time " +this.getID()+" sendmove "+t);
		//	 for(int i=0;i<t.size();i++) {
		//		 System.out.println(t.get(i));
		//	 }
		 
		} else {
			state = temp;
		}
	}

	@Override
	protected void postConnect() {

		super.postConnect();

		mergeConfigFiles();
		tChannel = new testChannel(this);
		mKeeper = new messageKeeper(this);
		// 解决indexClass空指针的问题
		this.world = (SEUWorldModel) super.model;

		search = new AStarSearch(world);

		useSpeak = config.getValue(Constants.COMMUNICATION_MODEL_KEY).equals(
				SPEAK_COMMUNICATION_MODEL);
		agentThinkTime = config.getIntValue(AGENT_THINK_TIME_KEY,
				DEFAULT_AGENT_THINK_TIME);
		sayMessageTransfer = new MessageTranslator(config);
		startTime = config.getIntValue(START_TIME_KEY);
		viewDistance = config.getIntValue(VIEW_DISTANCE_KEY,
				DEFAULT_VIEW_DISTANCE);
		simulationTime = config.getIntValue(TIMESTEPS);
		maxDistance = config.getIntValue(DISTANCE_KEY);

		state = State.Waiting;
		world.indexClass(StandardEntityURN.CIVILIAN,
				StandardEntityURN.FIRE_BRIGADE, StandardEntityURN.POLICE_FORCE,
				StandardEntityURN.AMBULANCE_TEAM, StandardEntityURN.REFUGE,
				StandardEntityURN.BUILDING);

		unexploredBuildings = world
				.getEntitiesOfType(StandardEntityURN.BUILDING);

		messageMemory = new MessageMemory(IGNORE_COMMANDS_AFTER);
		messageController = new MessageController();
		informedBurningBuildings = new ArrayList<Pair<Building, Fieryness>>();
		oldbuildings=new ArrayList<Pair<Building,Integer>>();
		observableBuildings = new ArrayList<EntityID>();
		
		locationAtLastCycle = location();
		burningBuildingsInChangeSet = new ArrayList<Building>();
		
		heatingBuildings = new ArrayList<Building>();

		
		civiliansAround = new ArrayList<Civilian>();
		commands = new HashMap<Integer, seuCommand>();
		world.initializeMapSearching(getGroupingType());
		
		// 对这个方法进行修改
		world.createNodeCaches();
		world. createSEUBuildings();//niu

		foundAgents = new ArrayList<Human>();

		// 初始化
		deadHumans = new ArrayList<EntityID>();
		predictDamageTool = new PredictDamageTool();
		foundCivilians = new ArrayList<HumanInfo>();
		locationHistory = new HashMap<Integer, EntityID>();
		fire = new ArrayList<FireSite>();
		nofoundbuilding = new ArrayList<Building>();
		maybebu = new ArrayList<Building>();
		temp_maybebuilding = new ArrayList<Building>();
		safeRoad = new ArrayList<Road>();
		//HydrantInUse= new ArrayList<Hydrant>();
		ATFirstTasks=new ArrayList<EntityID>();
		PFFirstTasks=new ArrayList<EntityID>();
		agentFirstPositionMap= new HashMap<EntityID, EntityID>();
		initializePositionHistory();

	}

	public void initializePositionHistory() {
		StandardEntity me;

		me = me();
		if (me instanceof Human) {
			Human human = (Human) me;

			positionHistory = new PositionHistory(human, world);
		} else {
			positionHistory = new PositionHistory();
		}
	}

	/**
	 * Get the location of the entity controlled by this agent.
	 * 
	 * @return The location of the entity controlled by this agent.
	 */
	@Override
	public StandardEntity location() {
		E me = me();
		if (me instanceof Human) {
			return ((Human) me).getPosition(world);
		}
		return me;
	}

	@Override
	// @SuppressWarnings("unchecked")
	public E me() {
		return super.me();
	}

	/**
	 * Construct a random walk starting from this agent's current location.
	 * Buildings will only be entered at the end of the walk.
	 * 
	 * @return A random walk.
	 */
	public List<EntityID> randomWalk() {
		List<EntityID> result = new ArrayList<EntityID>(RANDOM_WALK_LENGTH);//记位置
		Set<StandardEntity> seen = new HashSet<StandardEntity>();
		StandardEntity current = location();
		for (int i = 0; i < RANDOM_WALK_LENGTH; ++i) {
			result.add(current.getID());
			seen.add(current);
			List<StandardEntity> neighbours = new ArrayList<StandardEntity>(
					search.findNeighbours(current));
			Collections.shuffle(neighbours, random);
			boolean found = false;
			for (StandardEntity next : neighbours) {
				if (seen.contains(next)) {
					continue;
				}
				if (next instanceof Building && i < RANDOM_WALK_LENGTH - 1) {
					continue;
				}
				current = next;
				found = true;
				break;
			}
			if (!found) {
				// We reached a dead-end.
				break;
			}
		}
		return result;
	}

	/**
	 * Construct a random walk starting from this agent's current location.
	 * Buildings will only be entered at the end of the walk.
	 * 
	 * @param partition
	 *            The region of the agent in the world model.
	 * @return A random walk.
	 */
	protected List<EntityID> randomWalk(EntityInPartition partition) {//异构
		List<EntityID> result = new ArrayList<EntityID>(RANDOM_WALK_LENGTH);
		Set<StandardEntity> seen = new HashSet<StandardEntity>();
		StandardEntity current = location();
		for (int i = 0; i < RANDOM_WALK_LENGTH; ++i) {
			result.add(current.getID());
			seen.add(current);
			List<StandardEntity> neighbours = new ArrayList<StandardEntity>(
					search.findNeighbours(current));
			Collections.shuffle(neighbours, random);
			boolean found = false;
			for (StandardEntity next : neighbours) {
				if (seen.contains(next)) {
					continue;
				}
				if (next instanceof Building && i < RANDOM_WALK_LENGTH - 1) {
					continue;
				}
				current = next;
				found = true;
				break;
			}
			if (!found) {
				// We reached a dead-end.
				break;
			}
		}
		return result;
	}

	/**
	 * Get an EnumSet containing requested entity URNs.
	 * 
	 * @return An EnumSet containing requested entity URNs.
	 */
	@Override
	protected abstract EnumSet<StandardEntityURN> getRequestedEntityURNsEnum();

	@Override
	protected StandardWorldModel createWorldModel() {
		return new SEUWorldModel();
	}

	protected boolean shouldProcessCommand(Command command) {
		if (command.getAgentID().equals(getID())) {//自己的 
			return false;
		}
		if (getTimeStep() - command.getTime() > IGNORE_COMMANDS_AFTER) {
			return false;
		}
		return true;
	}

	protected void processHearing(Collection<Command> heard) {

		messageMemory.clearOldMessages(timeStep);
		for (Command command : heard) {
			if (command instanceof AKSpeak) {
				AKSpeak speakCommand;

				speakCommand = (AKSpeak) command;
				if (isCivilianCommand(speakCommand)) {
					processCivilianCommand(speakCommand);
				}
				if (shouldProcessCommand(command)) {
					processHearing((AKSpeak) command);
				} else {
				}
			} else {
			}
		}
	}

	protected void processObserving(ChangeSet changed) {
		observableBuildings.clear();
		observableRefuge.clear();
		burningBuildingsInChangeSet.clear();
		foundAgents.clear();
		heatingBuildings.clear();
		civiliansAround.clear();
		changedEntities = changed.getChangedEntities();
		NUMOfATOCCUR=new ArrayList<EntityID>();//////LB0422
		
		AgentPositionMessage agentpositionmessage=new AgentPositionMessage(location().getID(),
				transidoldtonew(getMeAsHuman().getID().getValue()), getTimeStep());
		sendMessage(agentpositionmessage,MessagePriority.High);
		
		for (EntityID next : changedEntities) {
			StandardEntity entity = world.getEntity(next);
			if (entity instanceof Building) {
				if(entity instanceof GasStation)
				{
					
					if(getMeAsHuman() instanceof FireBrigade){
					if( ((GasStation) entity).getFieryness()==0){//PRE WATER
						BuildingIsBurningMessage m =new BuildingIsBurningMessage(entity.getID(),1,150,5,this.getTimeStep()) ;
						processBuildingIsBurningMessage(m);
						}
					float x=((Area)entity).getLocation(model).first();
					float y=((Area)entity).getLocation(model).second();
					Collection<StandardEntity>entities=model.getObjectsInRange((int)x, (int)y, 20000);//buildings near gasstation prewater	
//					if(!entities.isEmpty()){
//					for (StandardEntity e : entities){
//						
//						if(e instanceof Building){
//							Building B=(Building) e;
//							if(B.isFierynessDefined())
//							 System.out.println("building next to gas"+B.getFieryness());
//							 if(B.isFierynessDefined() && B.getFieryness()==0){
//						BuildingIsBurningMessage m =new BuildingIsBurningMessage(B.getID(),1,150,5,this.getTimeStep()) ;
//						processBuildingIsBurningMessage(m);
//							 }
//						}
//						}
//					}
					if(((GasStation) entity).getFieryness()!=0 && ((GasStation) entity).getFieryness()!=4&&((GasStation) entity).getFieryness()!=1){
						//System.out.println("gas burned "+entity.getID()+"             "+((GasStation) entity).getFieryness());
						if(!GasStationBurned.contains((GasStation) entity))
						GasStationBurned.add((GasStation) entity);
					}
					}
					
				
					if(((GasStation) entity).getTemperature()>35 && 
							(((GasStation) entity).getFieryness()==0||((GasStation) entity).getFieryness()==4)){
						//tasks.add(0, new RunAwayFromGasStationTask(this));//niu13.10.15
						
						
						AgentIsBuriedMessage agentIsBuriedMessage = new AgentIsBuriedMessage(
							location().getID().getValue(),
							transidoldtonew(getMeAsHuman().getID().getValue()), 
							getTimeStep(), 
							getMeAsHuman().getHP(), 
							getMeAsHuman().getBuriedness(), 
							getMeAsHuman().getDamage());
					sendMessage(agentIsBuriedMessage,
							MessagePriority.High);
					
					GasStationIsBurningMessage message = new GasStationIsBurningMessage(
							entity.getID(), 1, 150,5,
							this.getTimeStep());
					sendMessage(message, MessagePriority.High);
					}
					
					
					
				}
				/*
				if(entity instanceof Refuge){
					Building b = (Building) entity;
					Road entrance = getEntranceRoad(b);
					if (entrance != null) {
						Blockade blockadeInEntrance = getClosestBlockadeInRoad(entrance);
					if( me() instanceof PoliceForce && blockadeInEntrance!=null){
						
						tasks.add(0, new CheckRefugeTask(this));//13.10.16
						
					}
					}
				}*/
				
				Building building = (Building) entity;
				observableBuildings.add(next);
			if(getMeAsHuman().getStandardURN() == StandardEntityURN.AMBULANCE_TEAM ){	
				for (EntityID next1 : changedEntities) {   //////wan   remove buildings with blockade
					StandardEntity entity1 = world.getEntity(next1);
					
					if (entity1 instanceof Blockade) {
						Blockade blockade;
						EntityID roadId;
						 
						blockade = (Blockade) entity1;
						roadId = blockade.getPosition();
					 
						Road entrance=getEntranceRoad(building);
						if(entrance!=null&&entrance.getID().equals(roadId))  {
						//	System.out.println(getID()+" !!!!!!!!!!!!!!!!!!!!!!!!!"+building.getID());
							nofoundbuilding.remove(building);
						}
					}
					}
			}
			 
				if (building.isOnFire()) {//13.10.15
					 
					  SEUBuilding seuBuilding = world.getSEUBuilding(building.getID());
					  seuBuilding.setEnergy(building.getTemperature() * seuBuilding.getCapacity());
			            switch (building.getFieryness()) {
			                case 0:
			                    seuBuilding.setFuel(seuBuilding.getInitialFuel());
			                    
			                    break;
			                case 1:
			                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.66) {
			                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.75));
			                    } else if (seuBuilding.getFuel() == seuBuilding.getInitialFuel()) {
			                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.90));
			                    }
			                    break;

			                case 2:
			                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.33
			                            || seuBuilding.getFuel() > seuBuilding.getInitialFuel() * 0.66) {
			                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.50));
			                    }
			                    break;

			                case 3:
			                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.01
			                            ||seuBuilding.getFuel() > seuBuilding.getInitialFuel() * 0.33) {
			                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.15));
			                    }
			                    break;

			                case 8:
			                	seuBuilding.setFuel(0);
			                    break;
			            }
			            
					
			            BuildingIsExploreMessage_at message1 = new BuildingIsExploreMessage_at(     //////wan
							building.getID());
					sendMessage(message1, MessagePriority.High);
					BuildingIsExploreMessage_fb message2 = new BuildingIsExploreMessage_fb( ////wan
							building.getID());
					sendMessage(message2, MessagePriority.High);
					BuildingIsExploreMessage_pf message3 = new BuildingIsExploreMessage_pf(///wan
							building.getID());
					sendMessage(message3, MessagePriority.High);
					nofoundbuilding.remove(building);
					
					
					addFireBuilding(building);

					Pair<Building, Fieryness> pair;
					burningBuildingsInChangeSet.add(building);
					pair = new Pair<Building, Fieryness>(building,
							building.getFierynessEnum());
					if (building.isOnFire()
							&& !informedBurningBuildings.contains(pair)) {
						informedBurningBuildings.add(pair);
						int fieryness = 0;
						int temperature = 0;
						int importance=building.getImportance();
						if (building.isFierynessDefined()) {
							fieryness = building.getFieryness();
							if(fieryness==4 || fieryness==5){//niu 14.6.26
								seuBuilding.setWasEverWatered(true);
								seuBuilding.setEnergy(0);
							}
						}
						if (building.isTemperatureDefined()) {
							temperature = building.getTemperature();
							
						}
					 if(this.getAllGasStations().isEmpty()){
						 BuildingIsBurningMessage message = new BuildingIsBurningMessage(//niu importance
									building.getID(), fieryness, temperature,importance,
									this.getTimeStep());
						 //System.out.println("send   buildingisburning");
							        sendMessage(message, MessagePriority.High);
					 }
						//到gasstation拒离   niu
						for(GasStation g:this.getAllGasStations()){
							if(!GasStationBurned.contains(g)||!g.isFierynessDefined()){
								
								int dis=world.getDistance(g.getID(), building.getID());
							 if(dis<50000 )//暂定40米 
							 {if( building.getFieryness()==1){
								 building.setImportance(3);
								 BuildingInfo buildingInfo = new BuildingInfo(building);
									buildingInfo.setLastUpdateTime(timeStep);
								 importance=building.getImportance();
								 BuildingIsBurningMessage message = new BuildingIsBurningMessage(
											building.getID(), fieryness, temperature,importance,
											this.getTimeStep());
									sendMessage(message, MessagePriority.High); 
									//System.out.println("buildingclosetoGSisburning");
									if(g.isFierynessDefined() && g.getFieryness()!=1 ){             //niu 14.3.26 20米内发送气站着火
										BuildingIsBurningMessage m =new BuildingIsBurningMessage
												(g.getID(),1,150,5,this.getTimeStep()) ;
										sendMessage(m, MessagePriority.High);
									}
							  }else{building.setImportance(1);}
							 }else{
								// System.out.println("buildingisburning"+entity.getID());
						        BuildingIsBurningMessage message = new BuildingIsBurningMessage(//niu importance
								building.getID(), fieryness, temperature,importance,
								this.getTimeStep());
						        sendMessage(message, MessagePriority.High);
							 }
							}else{
								//building.setImportance(1);
								continue;}
						}
					}
				} else {
					for (Fieryness f : Building.BURNING) {
						Pair<Building, Fieryness> pair;

						pair = new Pair<Building, Fieryness>(building, f);
						if (informedBurningBuildings.contains(pair)) {
							informedBurningBuildings.remove(pair);
							 SEUBuilding seuBuilding = world.getSEUBuilding(building.getID());
							if (building.isFierynessDefined()) {
								int fieryness = building.getFieryness();
								if(fieryness==4 || fieryness==5){//niu 14.6.26
									seuBuilding.setWasEverWatered(true);
									seuBuilding.setEnergy(0);
								}
							}
							BuildingIsExtinguishedMessage message;
							//System.out.println("building is extinguished"+entity.getID());
							message = new BuildingIsExtinguishedMessage(
									building.getID(), building.getFieryness(),timeStep);
							sendMessage(message, MessagePriority.Medium);
						}
					}

				}
				processBuildingHeat(building);
				BuildingInfo bi = world.getBuildingInfo(entity.getID());
				bi.setLastUpdateTime(this.getTimeStep());
				
				if(entity instanceof Refuge){          //yuan7.8
					observableRefuge.add(entity.getID());
				}
			} else if (entity instanceof Human) {
				Human human = (Human) entity;
				if ((human.getStandardURN() == StandardEntityURN.FIRE_BRIGADE
						|| human.getStandardURN() == StandardEntityURN.POLICE_FORCE || human
						.getStandardURN() == StandardEntityURN.AMBULANCE_TEAM)
						&& human.getID().getValue() != me().getID().getValue()) {

					// 死亡的智能体
					if (human.isHPDefined())
						if (human.getHP() == 0) {
							if (!deadHumans.contains(human.getID()))
								deadHumans.add(human.getID());
						}
					foundAgents.add(human);
					if(human.isDamageDefined() ){//niu14.2.21
						if(human.getDamage()>0 && human.getPosition(world) instanceof Road){
							AgentIsBuriedMessage message = new AgentIsBuriedMessage(
									human.getPosition(model).getID().getValue(),
									human.getID(), timeStep,
									human.getHP(),
									human.getBuriedness(),
									human.getDamage());
							if (canRescue()) {
								processMessage(message);
							}
							sendMessage(message, MessagePriority.High);
						}
					}
				
					if(human.getStandardURN() == StandardEntityURN.AMBULANCE_TEAM)
						NUMOfATOCCUR.add(human.getID());
					
				} else if (human.getStandardURN() == StandardEntityURN.CIVILIAN) {

					Civilian civilian = null;
					StandardEntity location;
					HumanInfo h;
					civilian = (Civilian) entity;
					civiliansAround.add(civilian);
					location = civilian.getPosition(model);
					if (location instanceof Area) {
						Area area = (Area) civilian.getPosition(model);
						if (foundCivilians.contains(civilian.getID())) {
							h = foundCivilians.get(foundCivilians
									.indexOf(civilian.getID()));
						} else {
							h = new HumanInfo(civilian.getID());
						}
						if (area instanceof Building) {
							getExploration().addToExplorationHistory(
									(Building) area);
						}
						if (civilian.getHP() == 0) {

							// 死亡的市民 if (!deadHumans.contains(human.getID()))
							deadHumans.add(human.getID());
							civiliansAround.remove(civilian.getID());
							CivilianIsSavedOrDeadMessage civilianIsSavedOrDeadMessage = new CivilianIsSavedOrDeadMessage(
									civilian.getID());
							sendMessage(civilianIsSavedOrDeadMessage,
									MessagePriority.Medium);
						} else {
							if (civilian.getDamage() == 0
									&& civilian.getBuriedness() == 0) {
								foundCivilians.remove(civilian.getID());
							} else {
								if (area instanceof Building) {
									Building b = (Building) area;
									if (b.isOnFire()) {
										// h.setDeadTime(predictDamageTool
										// .calculateDeadTime(
										// civilian.getHP(),
										// civilian.getDamage(),
										// timeStep, true),
										// timeStep, civilian.getDamage(),
										// true);
									} else {
										// h.setDeadTime(predictDamageTool
										// .calculateDeadTime(
										// civilian.getHP(),
										// civilian.getDamage(),
										// timeStep, false),
										// timeStep, civilian.getDamage(),
										// false);
									}
								}
								if (!foundCivilians.contains(civilian.getID())) {
									foundCivilians.add(h);

								}
								CivilianInformationMessage message = new CivilianInformationMessage(
										area.getID().getValue(),
										civilian.getID(), timeStep,
										civilian.getHP(),
										civilian.getBuriedness(),
										civilian.getDamage());
								if (canRescue()) {
									processMessage(message);
								}
								sendMessage(message, MessagePriority.High);
							}
						}

					}
				}

			} else if (entity instanceof Blockade) {
				Blockade blockade;
				EntityID roadId;
				RoadInfo roadInfo;
			

				blockade = (Blockade) entity;
				roadId = blockade.getPosition();
				roadInfo = world.getRoadInfo(roadId);
				roadInfo.resetBlockadeCache();
			} else if (entity instanceof Road) {
				if(entity instanceof Hydrant ){
					Hydrant h=(Hydrant) entity;
					if(getMeAsHuman().getPosition(world).equals(h) ){//niu
					ReachedHydrant.add(h);
				}
				}
				
				Property property;
				property = changed.getChangedProperty(entity.getID(),
						StandardPropertyURN.BLOCKADES.toString());
				if (property != null) {
					RoadInfo roadInfo;

					roadInfo = world.getRoadInfo(entity.getID());
					roadInfo.resetBlockadeCache();
				}
			}
		}
		
	}

	/**
	 * This function is used to process AKSpeak commands.
	 * 
	 * @param command
	 *            AKSpeak command which comes from other agents.
	 */
	protected void processHearing(AKSpeak command) {
		byte[] bytes = command.getContent();
		//System.out.println(this.getTimeStep()+"time : "+command);System.out.println(bytes);
		try {
			if(command.getAgentID().equals(getID()))
				return;
			Message message = sayMessageTransfer.receive(bytes);
			message.setSender(command.getAgentID());
			message.setReceivedStep(timeStep);
			// if (this.getTimeStep() <= 5)
			// System.out.println(message.getType());
/*
			
			 * if (message instanceof AgentIsBuriedMessage)
			 * System.out.println("TIMESTEP" + this.getTimeStep() + "#" +
			 * getID() + " Receive: " + message.getType() + " " +
			 * message.getId());
			 */
			/*
			 * if (message instanceof CivilianIsSavedOrDeadMessage)
			 * System.out.println("TIMESTEP" + this.getTimeStep() + "#" +
			 * this.getMeAsHuman().getID().getValue() + " Receive: " +
			 * message.getType() + " " + message.getId());
			 */

			/*
			 * if (message instanceof BuildingIsBurningMessage)
			 * System.out.println("TIMESTEP" + this.getTimeStep() + "#" +
			 * "timeStep:" + ((BuildingIsBurningMessage) message).getTimeStep()
			 * + " sender:" + message.getSender() + " rev:" + getID() +
			 * " Receive: " + message.getType() + " " + message.getId());
			 * 
			 * if (message instanceof AgentIsStuckMessage)
			 * System.out.println("TIMESTEP" + this.getTimeStep() + "#" +
			 * this.getMeAsHuman().getID().getValue() + " Sender: " +
			 * message.getSender() + message.getType() + " " + message.getId());
			 */
			/*
			 * if (message instanceof BuildingIsExtinguishedMessage)
			 * System.out.println("TIMESTEP" + this.getTimeStep() + " Receive: "
			 * + "#" + this.getMeAsHuman().getID().getValue() +
			 * message.getType() + message.getId());
			 */
			/*
			 * if (message instanceof BuildingIsBurningMessage) if (agentType ==
			 * StandardEntityURN.FIRE_BRIGADE) System.out.println("TIMESTEP" +
			 * this.getTimeStep() + " Receive: "+"  #" + getID() +" Type:"+
			 * message.getType()+" ID:" + message.getId()+" SENDER:" +
			 * message.getSender());
			 */
			/*
			 * if (message instanceof ClearPathIsNeededMessage)
			 * System.out.println("TIMESTEP" + this.getTimeStep() + "#" +
			 * this.getMeAsHuman().getID().getValue() + " Receive: " +
			 * message.getType());
			 */

			/*
			 * if (message instanceof BuildingIsExploredMessage)
			 * System.out.println("TIMESTEP" + this.getTimeStep() + "#" +
			 * this.getMeAsHuman().getID().getValue() + " Receive: " +
			 * message.getType() + " " + message.getId());
			 */
			if (messageMemory.shouldProcessMessage(message)) {    //wan
				totalReceived++;
				if (message instanceof AgentIsStuckMessage){
					AgentIsStuckMessage m=(AgentIsStuckMessage) message;
					mKeeper.addMessage(message);
					//System.out.println(getID()+"---AgentIsStuckMessage-----------喊话接受--------------");
					processMessage(message);
					//sendVoiceMessage(m,MessagePriority.Medium);	
					
				}
				if (message instanceof AgentIsBuriedMessage){    ////wan
					AgentIsBuriedMessage agentIsBuried=(AgentIsBuriedMessage) message;
					mKeeper.addMessage(message);
					//System.out.println(getID()+"----AgentIsBuriedMessage----------喊话接受--------------");
					
					//sendVoiceMessage(agentIsBuried,
						//	MessagePriority.Medium);	
					processMessage(message);
					 
				}
				 if (message instanceof BuildingIsBurningMessage) {                 //wan
						BuildingIsBurningMessage m = (BuildingIsBurningMessage) message;
						mKeeper.addMessage(m);
						//sendVoiceMessage(m,MessagePriority.Medium);
						//System.out.println(getID()+"----BuildingIsBurningMessage---------喊话接受--------------");
						processBuildingIsBurningMessage(m);
					}
				 if (message instanceof BuildingIsExtinguishedMessage) {
						BuildingIsExtinguishedMessage m = (BuildingIsExtinguishedMessage) message;
						mKeeper.addMessage(m);
						//sendVoiceMessage(m,MessagePriority.Medium);
						//System.out.println(getID()+"----BuildingIsExtinguishedMessage---------喊话接受--------------");
						processBuildingIsExtinguishedMessage(m);
					 } 
				if (message instanceof PartitionAssignmentMessage) {
					PartitionAssignmentMessage m = (PartitionAssignmentMessage) message;
					processMessage(m);
				   } 
                if (message instanceof InformTeamMessage) {
					InformTeamMessage m = (InformTeamMessage) message;
					processInformTeamMessage(m);
				    } 
                if (message instanceof MessageCount) {
					MessageCount m = (MessageCount) message;
					processMessage(m);
				   }
                 
				
				 if (message instanceof GasStationIsBurningMessage) {
						GasStationIsBurningMessage m = (GasStationIsBurningMessage) message;
						processGasStationIsBurningMessage(m);
				} 
                  if (message instanceof BuildingCloseToGasStationIsBurningMessage) {
					BuildingCloseToGasStationIsBurningMessage m = (BuildingCloseToGasStationIsBurningMessage) message;
					processBuildingIsBurningMessage(m);
				}
				 
					processMessage(message);
				
				messageMemory.setMessageAsProcessed(message);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * This function processes the walkie talkie messages.
	 * 
	 * @param message
	 *            Message which comes from walkie talkie
	 */
	protected abstract void processMessage(Message message);

	public int tot_message = 0;

	public void sendMessage(Message message, MessagePriority priority) {
		int channel;
		tot_message = tot_message + message.getbyte();
		// if (timeStep == 295)
		// System.out.print(me().getID().getValue() + "   " + tot_message);
		channel = sayMessageTransfer.getChannel(message.getType());
		// Message message1=new Message();

		// Message message1=null;
		/*if (this.getCommCondition() == CommunicationCondition.NoCommunication
				|| this.getCommCondition() == CommunicationCondition.Limited || )*/   //wan
			mKeeper.addMessage(message);
		// System.out.println(message.getType());
		if (timeStep <= 5)

		{
			if (message instanceof ChannelCondition
					|| message instanceof MessageCount)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount1)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount2)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount3)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount4)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount5)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount6)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount7)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount8)
				messageController.addMessage(channel, message, priority);
			if (message instanceof MessageCount9)
				messageController.addMessage(channel, message, priority);

		} else {
			if (channel != 0)
				messageController.addMessage(channel, message, priority);
		}
	}

	public void sendVoiceMessage(Message message, MessagePriority priority) {

		messageController.addMessage(0, message, priority);
	}

	public void sendMessage(Message message, int channel) {
		int id = sayMessageTransfer.getNextMessageId();
		message.setId(id);
		// System.err.println(this.getTimeStep()+" channel:"+channel+message);
		byte[] bytes = sayMessageTransfer.transmit(message);

		int n = sayMessageTransfer.getRepeatCount(channel);

		for (int i = 0; i < n; i++) {
			sendSpeak(getTimeStep(), channel, bytes);
		}

		sayMessageTransfer.incrementMessageId();
		messageController.clearQueue();
	}

	protected void sendMessagesFromQueue() {
		// System.out.println(" Queue size : " + messageController.queueSize());
		Collection<QueueMessage> importantMessages = messageController.getMostImportantMessages();
		// System.err.println(this.getTimeStep()+" channel:"+channel+message);
		if (importantMessages == null) {
		} else {
			for (QueueMessage next : importantMessages) {
				Message message = next.message;
				// if (message.getType() == MessageType.AgentIsStuckMessage
				// || message.getType() == MessageType.ClearPathIsNeededMessage)
				// {
				// System.out.println(this.getTimeStep() + "---"
				// + this.getID() + "Send" + message);
				// }
				int channel = next.channel;
				/*if (this.getCommCondition() == CommunicationCondition.NoCommunication
						|| this.getCommCondition() == CommunicationCondition.Limited)
					if (channel != 0)
						return;            */                    ///////////wan
				try {
					if(message==null)return ;
					
					sendMessage(message, channel);
					// System.out.println(channel);
				} catch (Exception e) {
					System.err.println("Send" + message + "error!!!");
				}

			}

		}
	}

	public int getTimeStep() {
		return timeStep;
	}

	public void setTimeStep(int timeStep) {
		this.timeStep = timeStep;
	}

	protected void addFireBuilding(Building building) {
		if (me().getStandardURN() != StandardEntityURN.FIRE_BRIGADE)
			return;
		// System.out.println("ssssssssssssssssssssssssssssss" + "  id   "
		// + building);
		BuildingInfo b = new BuildingInfo(building);
		List<FireSite> f1 = new ArrayList<FireSite>();
		for (FireSite ff : fire)
			if (ff.containsFire(building))
				return;
		for (FireSite f : fire) {
			if (f.iscontain(building)) {//火堆?
				f1.add(f);
				f.addFire(b);
			}
		}
		if (f1.size() == 0) {
			FireSite f2 = new FireSite(new BuildingInfo(building), world);
			fire.add(f2);
		}
		if (f1.size() > 1) {
			for (FireSite f3 : f1)
				f1.get(0).mergeWith(f3);
			for (FireSite f4 : f1)
				if (f4 != f1.get(0)) {
					fire.remove(f4);

				}
		}

	}

	protected void addFireGasStation(GasStation gasStation) {
		//niu
	}
	@Override
	protected void think(int time, ChangeSet changed, Collection<Command> heard) {
	/*	if(getNo()==1 && timeStep==startTime){
		 
		System.out.println("!!!!!!!!!!!!!!!!!!!        "+ world.getTotalArea());
		}*/
		//System.out.println("!!!!!!!!!!!!!!!!!!!"+getCommCondition()+"!!!!!!!!!!!!!!!!!!!!!!");
		//System.out.println(time+" "+this.getID()); 
		//System.out.println(" "+changed);
		//try {
			Date decisionEndTime;

			decisionStartTime = new Date();

			setTimeStep(time);
			
			if(time==1)  updateAgentStates();  ///wan
			//System.out.println(time+" "+this.getID()+" getLocation "+(Area) positionHistory.getCurrentLocation()+" "+(time-this.timeStep));
			// System.out.println("time  " + timeStep + "   " + heard);
			if (getMeAsHuman().isHPDefined()) {
				if (getMeAsHuman().getHP() == 0) {
					sendRest();
					return;
				}
			}
			//防止因为超时而不执行
			maybebu.removeAll(temp_maybebuilding);
			if(agentType==StandardEntityURN.AMBULANCE_TEAM){
				maybebu = new ArrayList<Building>();
			//System.out.println(timeStep+" "+this.getID()+" temp_maybebuilding "+temp_maybebuilding);
			maybebu.addAll(temp_maybebuilding);
			}
			if (timeStep%20==5) {
				temp_maybebuilding.clear();
				maybebu.clear();
			}
			
	
			
			//防止因为超时而不执行
			
			positionHistory.process(timeStep);
			///positionHistory.process(time);
			//System.out.println(this.getTimeStep()+" time "+this.getID()+" CurrentLocation()"+(Area) positionHistory.getCurrentLocation()+"later");
			if (time == 1) {

			}

			if (time == startTime) {
				int[] channels;
				addTeamMembers();
				assignQuartations();
				//if(agentType==StandardEntityURN.AMBULANCE_TEAM)//lbll
				//System.err.println(this.getID()+" "+getAssignedPartitions());
				
				setCommunicationCondition();//此函数什么也没做//lb
				// channels = getChannelsToSubscribe();
				TranslateID();
				idtoid();
				 
				// System.out.println(this + "my neighbour partitionmap"
				// + getNeighbourAssignedPartitinMap());
				// System.err.println(this + "My assigned partition"
				// + getAssignedPartitionMap());
				// sendSubscribe(time, channels);
				// System.err.println(this.getID());
				// for (StandardEntity s : world
				// .getEntitiesOfType(StandardEntityURN.CIVILIAN)) {
				// Human h = (Human) s;
				// System.out.println(h.getID());
				// }
				// Civilian h = new Civilian(new EntityID(9));
				// h.setHP(10000);
				// h.setDamage(0);
				// h.setBuriedness(0);
				// h.setPosition(new EntityID(959));
				// merge(h);

			}
			if (time >= startTime) {
				// System.out.println(this.getID()
				// + " "
				// + timeStep
				// + " "
				// + world.getEntitiesOfType(StandardEntityURN.CIVILIAN)
				// .size());
				// for (StandardEntity s : world
				// .getEntitiesOfType(StandardEntityURN.CIVILIAN)) {
				// Human h = (Human) s;
				// System.out.println(h.getID() + " " + h.getHP());
				// if (time % 2 == 0) {
				// h.setHP(h.getHP() - 10);
				// }
				// }
				
				changeSet = changed;
				//System.out.println("temp_maybebuilding_before_clear"+temp_maybebuilding);
				//temp_maybebuilding.clear();
				//System.out.println("temp_maybebuilding_after_clear"+temp_maybebuilding);
				
				processObserving(changeSet);
				// checkNeighbor();
				// numberOfBuildings() ;

				// if (me() instanceof AmbulanceTeam)
				// System.out.println(me() + " get Dead Civ & Agents: "
				// + getDeadHumans() + " in " + timeStep);

				// processHearing(heard);
				updateExploredBuilding();
                world.updateSEUBuilding();//niu 14.5.5火仿真更新
				if (timeStep == startTime) {
					nofoundbuilding.clear();
					for (StandardEntity s : world.getAllEntities()) {
						if (s instanceof Building) {
							Building b = (Building) s;
							nofoundbuilding.add(b);
						}
					}
				}
				/*
				maybebuilding.removeAll(temp_maybebuilding);
				System.out.println("temp_maybebuilding_after_remove_all"+temp_maybebuilding);
				maybebuilding.addAll(temp_maybebuilding);
				if (timeStep%10==5) {
					temp_maybebuilding.clear();
					maybebuilding.clear();
				}
				*/
				if (temp_maybebuilding.size() > 20) {/////////////////之前为10
					maybebu.clear();
					maybebu.addAll(temp_maybebuilding);
					System.out.println("maybebuilding"+maybebu);
				}
				
				//System.err.println(timeStep+" "+this.getID()+" maybebu_addall_agent "+me().getURN()+" "+maybebu.size() );
				//if(maybebu.size()>0&&maybebu.size()<100)
				//for(int i=0;i<maybebu.size();i++)
				//System.err.print(timeStep+" "+this.getID()+" "+maybebu.get(i) );
				// System.out
				// .println("time  " + timeStep + "  id   " + getID()
				// + "   size   " + nofoundbuilding.size()
				// + "  building ");

				// 处理碰撞被堵住的问题
				// 更新3周期智能体位置
				checkPostion();
				// 试图走出卡住的位置
				moveOutOfThePlace();
				// if (timeStep == 280)
				// System.out.print(me().getID().getValue() + "   " +
				// tot_message
				// + "  ....    ");
				// System.out.println(me() + "Time: " + timeStep
				// + " Latest Position: " + latestPosition + " Last Position "
				// + lastPosition + " Current Position: " + currentPostion);
				locationHistory.put(timeStep, getMeAsHuman().getPosition());
				if (timeStep <= 5) {

					// if (timeStep == 3) {
					// System.out.println(getCommCondition());
					//
					// }
					if (timeStep >= 3)
						tChannel.distributeChannel(heard);
					
				}
				// if (getID().getValue() == 34946
				// && location().getID().getValue() == 31045) {
				// return;
				// }
				if (timeStep > 5) {
					
					processHearing(heard);
					
					if(timeStep%20 ==1){// niu13.10.15
						
						tasks.add(0, new CheckGasStationTask(this));
					
						
					}
					
					if((timeStep>100 && timeStep%30==1)){//niu13.10.15
					
					tasks.add(0,new CheckRefugeTask(this));
					}
				
//					if(timeStep==world.getLastAfterShockTime() && timeStep!=0){//niu13.10.16
//						
//					tasks.add(0,new CheckRefugeTask(this));
//					//	tasks.add(0,new CheckGasStationTask(this));
//						System.out.println("aftershock!!!!!!!!!!!!!"+timeStep);
//						}
				/*if (this.getCommCondition() == CommunicationCondition.NoCommunication
						|| this.getCommCondition() == CommunicationCondition.Limited)*/      //wan
						this.sendVoiceMessageRepeatly();                       
				}
				processObserving(changeSet);
				
				handleTasks();
				if (me().getStandardURN() == StandardEntityURN.POLICE_FORCE
						&& timeStep % 3 == 0) {
					informTeam();
				}
				sendMessagesFromQueue();
				decisionEndTime = new Date();
				double dt = decisionEndTime.getTime()
						- decisionStartTime.getTime();

				// System.out.println(dt);
				if (dt > agentThinkTime / 2) {
					System.err.println(this.getMeAsHuman().getURN() + me()
							+ " Decision Time Needs " + dt + " In " + time);
				}
			}
		//	System.out.println(this.getTimeStep()+" time "+this.getID()+" CurrentLocation()"+(Area) positionHistory.getCurrentLocation()+"later");

			/* System.out.println(this.getTimeStep()+" time " +this.getID()+" "+this.getName()+
					 " latestPosition:"+latestPosition+" "+latestPosition.getX() +" "+latestPosition.getY()+
					 " lastPosition:"+lastPosition+" "+lastPosition.getX()+" "+lastPosition.getY()+
					 " currentPostion:"+currentPostion+" "+currentPostion.getX()+" "+currentPostion.getY()+
					 " latestState:"+latestState+" latestState "+lastState+" currentState "+currentState);
*/
			//positionHistory.process(timeStep);
			///positionHistory.process(time);
			//System.out.println(this.getTimeStep()+" time "+this.getID()+" CurrentLocation()"+(Area) positionHistory.getCurrentLocation()+"lat");
		//} catch (Exception e) {
		//	System.err.println(this.toString() + "Think err in "
		//			+ this.getTimeStep());
		//}

	}

	protected void addTeamMembers() {
		agentType = me().getStandardURN();
		world.addTeamMembers(agentType);
	}

	public void assignQuartations() {
		//if(me() instanceof FireBrigade && timeStep<50){//niu
		//	return;
		//}
		world.getPartitionModel().assignPartitionsToTeam();
		List<EntityInPartition> regions = getAssignedPartitions();
		for (EntityInPartition region : regions) {
			getExploration().addEntitiesToExplore(region.getBuildings());
		}
	}

	protected void processInformTeamMessage(InformTeamMessage message) {
		EntityID agentId = message.getSender();
		world.addToTeam(agentId);
	}
	protected void processAgentPositionMessage(AgentPositionMessage message) {
		 //System.out.println("process agentpositionmessage");
		
			EntityID agentId = transidnewtoold(message.getAgentId().getValue());
			
			EntityID location = message.getLocationId();
			StandardEntity e = world.getEntity(agentId);

			Human man = (Human) e;
			man.setPosition(location);

	}
	protected abstract void thinkAndAct();

	@SuppressWarnings("unused")
	private void informTeam() {
		if (this.getMeAsHuman().isBuriednessDefined()) {
			InformTeamMessage message = new InformTeamMessage(
					transidoldtonew(this.getID().getValue()), this.location()
							.getID(), this.getMeAsHuman().getBuriedness());
			sendMessage(message, MessagePriority.Medium);
		} else {
			InformTeamMessage message = new InformTeamMessage(
					transidoldtonew(this.getID().getValue()), this.location()
							.getID(), 0);
			sendMessage(message, MessagePriority.Medium);
		}
	}

	public List<EntityInPartition> getAssignedPartitions() {
		if (world == null) {
			return null;
		}else {
			return getAssignedPartitionMap().getPartitions();
		}
	}

	public PartitionMap getAssignedPartitionMap() {
		if (world == null) {
			return null;
		} else {
			return world.getPartitionModel().getAssignedPartitions(getID());
		}
	}

	public PartitionMap getNeighbourAssignedPartitinMap() {
		if (world == null) {
			return null;
		} else {
			return world.getPartitionModel().getPartitionGroups()
					.get((this.getNo() + 1) % world.getTeam().size());
		}
	}

	public List<Area> getMyNeighbourPartitionEntities() {
		List<Area> entities;
		List<EntityInPartition> regions;

		entities = new ArrayList<Area>();
		regions = getNeighbourAssignedPartitinMap().getPartitions();
		if (regions == null) {
			return null;
		}
		for (EntityInPartition region : regions) {
			entities.addAll(region.getBuildings());
			entities.addAll(region.getRoads());
		}
		return entities;
	}

	public List<Building> getBuildingsInNeighbourPartition() {
		List<Building> entities;
		List<EntityInPartition> partitions;

		entities = new ArrayList<Building>();
		partitions = getNeighbourAssignedPartitinMap().getPartitions();
		if (partitions == null) {
			return null;
		}
		for (EntityInPartition region : partitions) {
			entities.addAll(region.getBuildings());
		}
		return entities;
	}

	public List<Road> getRoadsInNeighbourPartition() {
		List<Road> entities;
		List<EntityInPartition> partitions;

		entities = new ArrayList<Road>();
		partitions = getNeighbourAssignedPartitinMap().getPartitions();
		if (partitions == null) {
			return null;
		}
		for (EntityInPartition region : partitions) {
			entities.addAll(region.getRoads());
		}
		return entities;
	}

	public List<Area> getMyPartitionEntities() {
		List<Area> entities;
		List<EntityInPartition> regions;

		entities = new ArrayList<Area>();
		regions = getAssignedPartitions();
		if (regions == null) {
			return null;
		}
		for (EntityInPartition region : regions) {
			entities.addAll(region.getBuildings());
			entities.addAll(region.getRoads());
		}
		return entities;
	}

	public List<Road> getRoadsInPartition() {
		List<Road> entities;
		List<EntityInPartition> partitions;

		entities = new ArrayList<Road>();
		partitions = getAssignedPartitions();
		if (partitions == null) {
			return null;
		}
		for (EntityInPartition region : partitions) {
			entities.addAll(region.getRoads());
		}
		return entities;
	}

	public List<Building> getBuildingsInPartition() {
		List<Building> entities;
		List<EntityInPartition> partitions;

		entities = new ArrayList<Building>();
		partitions = getAssignedPartitions();
		if (partitions == null) {
			return null;
		}
		for (EntityInPartition region : partitions) {
			entities.addAll(region.getBuildings());
		}
		return entities;
	}

	/**
	 * 从当前位置寻找一条到Refuge的道路
	 * 
	 * @return The path from current location to nearest refuge.
	 */
	public Path getPathToRefuge(PathType pathType) {
		// TODO: 如果该路径不存在，如何处理
		Path path = search.getPath(getMeAsHuman(), getAllRefuges(), pathType);
		return path;
	}
	public Path getPathToRefuge() {
		return getPathToRefuge(PathType.EmptyAndSafe);
	}
	public Path getPathToRefuge1111(PathType pathType) {  ////wan 
		// TODO: 如果该路径不存在，如何处理
		 if(getAllRefuges1111().size()>0){
		 return  search.getPath(getMeAsHuman(),getAllRefuges1111().get(0), pathType);
		 }else
			return  search.getPath(getMeAsHuman(),getAllRefuges(), pathType);
		
	} 
	public Path getPathToRefuge1111() {                ////wan 
		return getPathToRefuge1111(PathType.EmptyAndSafe);
	}
	
	
	public Path getPathToHydrant(PathType pathType, Hydrant hydrant) {//niu
		// TODO: 如果该路径不存在，如何处理
		
		Path path = search.getPath(getMeAsHuman(), hydrant, pathType);
		return path;
	}
	
	public Path getPathToGasStation(GasStation gasStation) {//niu
		return getPathToGasStation(PathType.EmptyAndSafe, gasStation);
	}
	public Path getPathToGasStation(PathType pathType, GasStation gasStation) {//niu
		// TODO: 如果该路径不存在，如何处理
		
		Path path = search.getPath(getMeAsHuman(), gasStation, pathType);
		return path;
	}
	
	public Path getPathToHydrant(Hydrant hydrant) {//niu
		return getPathToHydrant(PathType.EmptyAndSafe, hydrant);
	}
	public Path getPathToGasStation(PathType pathType) {
		// TODO: 如果该路径不存在，如何处理
		
//		Path path = search.getPath(getMeAsHuman(), getAllGasStations(), pathType);
//		return path;
		Path path=null;
		for(GasStation g:getAllGasStations()){//niu 
			//Building b=(Building)g;
			if( !GasStationBurned.contains(g) && (isInMyPartition(g)||isInNeighborPartition(g))){
				 Path path1 = search.getPath(getMeAsHuman(), g, pathType);
				 if(path1.size()<15){
				 path=path1;
				 }
			}
		}

		return path;
	}
	public Path getPathToHydrant1() {//niu
		return getPathToHydrant1(PathType.EmptyAndSafe);
	}
	public Path getPathToHydrant1(PathType pathType) {//niu
		// TODO: 如果该路径不存在，如何处理
		
		Path path = search.getPath(getMeAsHuman(), getFarHydrants(), pathType);
		return path;
	}
	 
	 
	public Path getPathToGasStation() {//niu
		return getPathToGasStation(PathType.EmptyAndSafe);
	}
	public Path getPathToFire(PathType pathType) {//niu
		// TODO: 如果该路径不存在，如何处理
		Path path = search.getPath(getMeAsHuman(), getAllBigFires(), pathType);
		return path;
	}
	public Path getPathToSafePlace() {                  /////wan
		 
//		int i=0;
 		Path path = null;
//		for(;i<=3;i++){
//			path=search.getPath(getMeAsHuman(), getSafePlace().get(i), PathType.EmptyAndSafe);
//			if(path!=null&&!path.isEmpty()&&path.isPassable())
//				return path;
//		}
 		EntityID positionId;
		StandardEntity position;
          Road road=null;
          positionId = getPosition();
		position = getModel().getEntity(positionId);
 		if(position instanceof Building){
 			
 			Building building =(Building) position;
 			road= getEntranceRoad(building);
 		}
 		if(road!=null&&isSafeRoad(road)) 
 			return search.getPath(getMeAsHuman(), road, PathType.EmptyAndSafe);
 		
 		path=search.getPath(getMeAsHuman(), getSafePlace().get(0), PathType.EmptyAndSafe);
 		
		/*if(path==null||path.isEmpty()||!path.isPassable()){
		 if(location() instanceof Building){
			 
			 Road road=getEntranceRoad((Building) location());
			 path=search.getPath(getMeAsHuman(),road, PathType.EmptyAndSafe);	 
		 }
			 
		
		}*/
			
		return 	  path;

	}
	private EntityID getPosition() {
		// TODO Auto-generated method stub
		return null;
	}

	public Path getPathToFire() {//niu
		return getPathToFire(PathType.EmptyAndSafe);
	}
	public Path getPathToPartition() {
		Path path;
		List<Area> entities;

		entities = getMyPartitionEntities();
		path = search.getPath(getMeAsHuman(), entities, PathType.EmptyAndSafe);
		return path;
	}

	/**
	 * Determines the refuge objects in the world model.
	 * 
	 * @return the lise of refuges in the map.
	 */
	public List<Refuge> getAllRefuges() {
		List<Refuge> result = world.getEntitiesOfType(Refuge.class,
				StandardEntityURN.REFUGE);
		return result;
	}
public List<Refuge> getAllRefuges1111() {  ////wan 把没有路障的refuges排序
		
		List<Refuge> result=  new ArrayList<Refuge>();
		
		List<Refuge> refuges = world.getEntitiesOfType(Refuge.class,
				StandardEntityURN.REFUGE);
		
		for(int i=0;i<=refuges.size();i++){
			Refuge one=getNearestRefuge(refuges);
			refuges.remove(one);
			Path path =getPathTo(one.getID(),PathType.EmptyAndSafe);
			
			if(path!=null&&path.isPassable()&&!(path.isEmpty()))
				result.add(one);
		}
		
		 
		
		return result;
	}
	public List<Hydrant> getAllHydrants() {//niu
		List<Hydrant> result = world.getEntitiesOfType(Hydrant.class,
				StandardEntityURN.HYDRANT);
		/*List<FireBrigade> firebrigade = world.getEntitiesOfType(FireBrigade.class,
				StandardEntityURN. FIRE_BRIGADE);
		 StandardEntity position;
		 
		for(FireBrigade fb:firebrigade){
			position=fb.getPosition(world);
			if(position instanceof Hydrant){//简单判断占用,还需改进 
				result.remove((Hydrant)position);
			}
		}*/
		return result;
	}
	public List<Hydrant> getFarHydrants() {//niu paris4
		List<Hydrant> result1 = world.getEntitiesOfType(Hydrant.class,
				StandardEntityURN.HYDRANT);
		List<Hydrant> result =new ArrayList<Hydrant>();
		for(Hydrant h: result1){
			int temp1 = world.getDistance(getID(), h.getID());
			if(temp1<maxDistance/2 && temp1>maxDistance/3){
				result.add(h);
			}
		}
		return result;
	}
//	public List<Building> getSafePlaces() {//niu paris4
//		List<Building> result = new ArrayList<Building>() ;
//		List<Building> result1 = world.getEntitiesOfType(Building.class,
//				StandardEntityURN.BUILDING);
//		for(Building b: result1){
//			
//			//if(){       远离火点和气站，靠近主干道
//				result.add(b);
//			//}
//		}
//		return result;
//	}
	 
	public  List<Road> getSafePlace() {            //wan 
		
		Road road = null;	
		 if(safeRoad.isEmpty()){
		List<Road> result  = world.getEntitiesOfType(Road.class,
				StandardEntityURN.ROAD);
	 
	
		int dis2;
	
	 boolean isSafeRoad=true;
		for(Road b: result){
			 
			if(!getAllGasStations().isEmpty()){
			 
			for(GasStation next:getAllGasStations()){
 
			 
		 	dis2=world.getDistance(b, next);//distance(b.getX(), b.getY(), next.getX(), next.getY());
				//System.out.println(dis2);
			if(dis2<=60000)  
				
				isSafeRoad=false;
			}
			}
			if(isSafeRoad)   safeRoad.add(b);
			
		}
		 }
			/*int dis;
			int dis1=1999999999 ;
			
			for(Road r :safeRoad) {
				
				dis=world.distance(getMeAsHuman().getX(), getMeAsHuman().getY(), r.getX(), r.getY());
			 
			if(dis<dis1){
				road=r; 
				dis1=dis;
			
			}
			}*/
			
			 DistanceComparator comparator;
			comparator = new DistanceComparator(location(), getModel());
			Collections.sort(safeRoad,comparator);
		 
		return safeRoad;
	}
	public List<GasStation> getAllGasStations() {//niu
		List<GasStation> result = world.getEntitiesOfType(GasStation.class,
				StandardEntityURN.GAS_STATION);
		return result;
	}
	public List<Building> getAllBigFires() {//niu
		Collection<Building> e = world.getEntitiesOfType(Building.class,
				StandardEntityURN.BUILDING);
		List<Building> result = new ArrayList<Building>();
		for (Building b : e) {
			if (b.isOnFire()&& (world.getTotalBurningArea(b)>500 )) {
				result.add(b);
				}
			}
		return result;
	}
	// unused
	protected void updateUnexploredBuildings(Collection<EntityID> changed) {
		for (EntityID next : changed) {
			StandardEntity e = world.getEntity(next);
			if (e != null) {
				unexploredBuildings.remove(e);
			}
		}
	}

	protected void updateExploredBuilding() {
		StandardEntity location = location();
		if (location instanceof Building) {
			Building b1 = (Building) location;

			if (nofoundbuilding.contains(location)) {
				BuildingIsExploreMessage_at  message1 = new BuildingIsExploreMessage_at(
						b1.getID());
				sendMessage(message1, MessagePriority.Medium);
				BuildingIsExploreMessage_fb message2 = new BuildingIsExploreMessage_fb(
						b1.getID());
				sendMessage(message2, MessagePriority.Medium);
				BuildingIsExploreMessage_pf message3 = new BuildingIsExploreMessage_pf(
						b1.getID());
				sendMessage(message3, MessagePriority.Medium);
				nofoundbuilding.remove(b1);
			}
			// if ()
			if (maybebu.contains(b1))
				maybebu.remove(b1);
			Human human = getCivilianOnCurrentBuilding();
			if (human == null) {
				Building building = (Building) location;
				getExploration().addToExplorationHistory(building);
				BuildingIsExploredMessage message = new BuildingIsExploredMessage(
						building.getID());
				sendMessage(message, MessagePriority.Medium);
			} else {
				// DecisionManager.getHealthState(human);
				if (!canRescue()) {
					Building building = (Building) location;
					getExploration().addToExplorationHistory(building);
					try {
						CivilianInformationMessage message = new CivilianInformationMessage(
								building.getID().getValue(), human.getID(),
								timeStep, human.getHP(), human.getBuriedness(),
								human.getDamage());
						sendMessage(message, MessagePriority.High);
					} catch (Exception e) {
						System.err.println(me() + " Get HP Error");
						// e.printStackTrace();
					}

				}
			}
		}
	}

	protected Human getCivilianOnCurrentBuilding() {
		StandardEntity location = location();
		if (location instanceof Building) {
			if (location instanceof Refuge) {
				return null;
			}
			Building building = (Building) location;
			for (StandardEntity next : world.getHumans()) {
				if(next instanceof Human){//niu handle classexception
				Human human = (Human) next;
				
				HealthState health = DecisionManager.getHealthState(human,
						world, timeStep);
				if (human.getID().equals(getID())) {
					continue;
				}
				if (health == HealthState.Dead) {
					continue;
				}
				if (health == HealthState.Burning) {
					continue;
				}
				if (health == HealthState.Helpless) {
					continue;
				}
				if (!(human instanceof Civilian) && human.isBuriednessDefined()
						&& human.getBuriedness() == 0) {
					continue;
				}
				if (health == HealthState.WillDie
						|| health == HealthState.WillDieSoon
						|| health == HealthState.WillLive) {
					if (human.getPosition().equals(building.getID())) {
						return human;
					}
				} else {
				}
			}
			}
		}
		return null;
	}

	public Exploration<Building> getExploration() {
		return world.getBuildingExploration();
	}

	public Exploration<Building> getMyPartitionExploration() {
		return world.getBuildingExploration();
	}

	protected void processExplorationMessage(BuildingIsExploredMessage message) {
		Exploration<Building> exploration = getExploration();
		EntityID buildingId = message.getBuildingId();

		// 解决类的强制转换问题，伊斯坦布尔地图出现该类问题
		try {
			StandardEntity s = world.getEntity(buildingId);
			if (s instanceof Building) {
				Building building = (Building) world.getEntity(buildingId);
				if (building == null) {
					building = new Building(buildingId);
					merge(building);
				} else {
					if (maybebu.contains(building))
						maybebu.remove(building);
					if (nofoundbuilding.contains(building)) {
						nofoundbuilding.remove(building);

					}
				}

				if (exploration.shouldExplore(building)) {
					exploration.addToExplorationHistory(building);
				}
			}

		} catch (Exception e) {
			System.err.println("There is No Such Building");
		}
	}

	// 没有被调用
	protected boolean exploreRandomly() {
		List<EntityID> path = null;
		List<EntityInPartition> regions = getAssignedPartitions();
		if (regions.size() == 0) {
			path = randomWalk();
		} else {
			path = randomWalk(regions.get(0));
		}
		if (path == null || path.isEmpty()) {
			return false;
		} else {
			sendMove(getTimeStep(), path);
			return true;
		}
	}

	protected Path getPathTo(EntityID id, PathType type) {
		StandardEntity entity = world.getEntity(id);
		if (entity instanceof Area) {
			Path path = search.getPath(getMeAsHuman(), (Area) entity, type);
			return path;
		} else {
			return null;
		}

	}

	protected abstract boolean canRescue();

	public boolean isInMyPartition(StandardEntity entity) {
		return getAssignedPartitionMap().contains(entity);
	}

	public boolean isInNeighborPartition(StandardEntity entity) {
		Set<PartitionMap> neighbors;

		neighbors = world.getPartitionModel().getNeighbors(
				getAssignedPartitionMap(), 1);
		for (PartitionMap regionGroup : neighbors) {
			if (regionGroup.contains(entity)) {
				return true;
			}
		}
		return false;
	}

	private void mergeConfigFiles() {
		Configuration merger;
		Config simulatorConfig, mergedConfig;

		simulatorConfig = this.config;
		merger = new Configuration();
		mergedConfig = merger.getMergedConfig(simulatorConfig);
		this.config = mergedConfig;
	}

	public boolean shouldGoToRefuge() {
		Human me = getMeAsHuman();
		if (me.isDamageDefined()) {
			return me.getDamage() > MIN_DAMAGE_GO_TO_REFUGE;
		}
		return false;
	}

	public Human getMeAsHuman() {
		StandardEntity entiy = world.getEntity(me().getID());
		Human me = (Human) entiy;
		return me;
	}

	protected Path getPathToArea(Building center, int distance) {
		// Try to get to anything within maxDistance of the target
		Collection<StandardEntity> targets = world.getObjectsInRange(center,
				distance);
		Collection<Area> temp;

		temp = new ArrayList<Area>();
		for (StandardEntity entity : targets) {
			if (entity instanceof Area) {
				temp.add((Area) entity);
			}
		}
		if (temp.isEmpty()) {
			return null;
		}

		Path path = search.getPath(getMeAsHuman(), temp, PathType.EmptyAndSafe);
		if (path == null || path.isEmpty()) {
		}
		return path;
	}

	protected void setCommunicationCondition() {
		messageController.setCommunicationCondition(world);
	}

	// 基于任务的决策流程
	@SuppressWarnings({ "rawtypes" })
	public abstract List<BasicTask> getAgentTasks();
	//public abstract List<BasicTask> getAgentTasks1();//niu

	public Random getRandomSource() {
		return random;
	}

	public AbstractSearch getSearch() {
		return search;
	}

	public void sendMove(Path path) {
		Point target;
		if (path == null) {
			return;
		} else if (!path.isPassable()) {
			return;
		}
		path = fixBuildingExitPath(path);

		target = path.getTarget();
		if (target == null) {
			sendMove(timeStep, path.getEntities());
			seuMove command = new seuMove(timeStep, path.getEntities());
			commands.put(timeStep, command);
		} else {
			sendMove(timeStep, path.getEntities(), target.x, target.y);
			seuMoveXY command = new seuMoveXY(timeStep, path.getEntities(),
					target.x, target.y);
			commands.put(timeStep, command);
		}
		positionHistory.setDesiredLocation(path.getDestination());
		positionHistory.setDesiredPath(path);
	}

	public void sendClear(EntityID targetId) {
		Blockade blockade;

		blockade = world.getEntity(targetId, Blockade.class);
		if (blockade == null) {
		}
		if (!RoadUtilities.doesBlockExists(blockade, world)) {
		}
		seuClear command = new seuClear(timeStep, targetId);
		commands.put(timeStep, command);
		super.sendClear(timeStep, targetId);
	
		// System.out.println(this.getTimeStep()+" time "+this.getID()+" Clear"+targetId);
	}
	
	public void sendClearArea(EntityID targetId) {//niu
		Blockade blockade;

		blockade = world.getEntity(targetId, Blockade.class);
		if (blockade == null) {
		}
		if (!RoadUtilities.doesBlockExists(blockade, world)) {
		}
		/*
		int destX;
		int destY;
		  int dis=world.getDistance(getID(), targetId);
		  if(2*dis<8000){
			 destX=blockade.getX()+(int)dis;
			 destY=blockade.getY()+(int)dis;
		  }else{
			   destX=blockade.getX()+(int)0.5*dis;
			   destY=blockade.getY()+(int)0.5*dis;
		  }
	   */
		 List<Line2D> lines = GeometryTools2D.pointsToLines(GeometryTools2D.vertexArrayToPoints(blockade.getApexes()), true);
		double best = Double.MAX_VALUE;
        Point2D bestPoint = null;
        Point2D origin = new Point2D(me().getLocation(world).first(), me().getLocation(world).second());
        for (Line2D next : lines) {
            Point2D closest = GeometryTools2D.getClosestPointOnSegment(next, origin);
            double d = GeometryTools2D.getDistance(origin, closest);
            if (d < best) {
                best = d;
                bestPoint = closest;
            }
        }
        Vector2D v = bestPoint.minus(new Point2D(me().getLocation(world).first(), me().getLocation(world).second()));
		v = v.normalised().scale(1000000);
	
			int destX=(int)(me().getLocation(world).first()+v.getX());
			int destY=(int)(me().getLocation(world).second()+v.getY());
			//int destX=(int)(me().getLocation(world).first()+8000);
			//int destY=(int)(me().getLocation(world).second()+8000);
		seuClear command = new seuClear(timeStep, destX,  destY);
		commands.put(timeStep, command);
		super.sendClear(timeStep,  destX,  destY);
		
		
	     
	}
	/*
   public void sendClearArea(int destX, int destY) {//niu
		
	Area a = (Area) model.getEntity(((Human) me()).getPosition());
	
	// Area targetArea = (Area) target;
	//sendClearArea(time, x, y);
		seuClear command = new seuClear(timeStep, destX,  destY);
		commands.put(timeStep, command);
		super.sendClearArea(timeStep,  destX,  destY);
	     }
	*/
	

	@Override
	public void sendRest(int time) {
		// TODO Auto-generated method stub
		seuRest command = new seuRest(timeStep);
		commands.put(timeStep, command);
		super.sendRest(time);

	}

	@Override
	protected void sendLoad(int time, EntityID target) {
		// TODO Auto-generated method stub
		seuLoad command = new seuLoad(timeStep, target);
		commands.put(timeStep, command);
		super.sendLoad(time, target);
	}

	@Override
	protected void sendUnload(int time) {
		// TODO Auto-generated method stub
		seuUnload command = new seuUnload(timeStep);
		commands.put(timeStep, command);
		super.sendUnload(time);
	}

	public void sendRescue(Human human) {
		sendRescue(timeStep, human.getID());
		seuRescue command = new seuRescue(timeStep, human.getID());
		commands.put(timeStep, command);
	}

	public Path fixBuildingExitPath(Path path) {
		Path fixedPath;
		StandardEntity entity;

		entity = path.getStart(world);
		if (entity instanceof Building) {
			if (path.size() > 1) {
				Path p;
				Edge e;
				Area a2, a3;

				p = new Path();
				p.add(path.getStart());
				p.add(path.getEntities().get(1));
				if (path.size() > 2) {
					Point target;

					a2 = world.getEntity(path.getEntities().get(1), Area.class);
					a3 = world.getEntity(path.getEntities().get(2), Area.class);
					e = a2.getEdgeTo(a3.getID());
					target = PositionLocate.getCenter(e);
					p.setTarget(target);
				}

				fixedPath = p;
			} else {
				fixedPath = path;
			}
		} else {
			fixedPath = path;
		}
		return fixedPath;
	}

	void teshu(int e, int r1, int r2) {
		if (this.getID().getValue() == e
				& this.location().getID().getValue() == r1) {
			ClearPath cp = new ClearPath(new EntityID(r1), new EntityID(r2),
					ClearPathReason.AgentIsStuck);
			addAgentIsStuckTask(cp);
		}
	}

	// 处理任务
	@SuppressWarnings("rawtypes")
	public void handleTasks() {
		
		boolean executed;
		executed = false;
		
		if (tasks == null) {
			tasks = getAgentTasks();
			
			// if (this.getID().getValue() == 36545
			// & this.location().getID().getValue() == 352) {
			// ClearPath cp = new ClearPath(new EntityID(352), new EntityID(
			// 38012), ClearPathReason.AgentIsStuck);
			// addAgentIsStuckTask(cp);
			// }
			// System.out.println("sssssssssssssssssssssssssssssssss");
			// }
		}

		for (BasicTask task : tasks) {

			boolean shouldExecute;

			// 任务已经被执行过，是否需要重复执行
			if (executed) {
				task.taskDone();
			} else {
				Date time;
				long timeDifference;
				boolean runningLate;

				// 计算时间
				time = new Date();
				timeDifference = time.getTime() - decisionStartTime.getTime();
				runningLate = timeDifference > agentThinkTime
						* AGENT_THINK_TIME_TOLERANCE;

				// 超出容许时间，当前周期无任务执行
				if (runningLate) {
					randomWalkPenalty = DEFAULT_RANDOM_WALK_PENALTY;

					BasicTask lastTask;
					lastTask = tasks.get(tasks.size() - 1);

					// 如果当前任务不是最后任务
					if (!task.equals(lastTask)) {
						continue;
					}
				} else if (randomWalkPenalty > 0) {

					BasicTask lastTask;
					lastTask = tasks.get(tasks.size() - 1);

					// 如果当前任务是系统最后一个任务，随机行走任务
					if (!task.equals(lastTask)) {
						continue;
					} else {
						randomWalkPenalty--;
					}
				}

				shouldExecute = task.shouldExecute();
				//if(agentType==StandardEntityURN.AMBULANCE_TEAM)
				//System.out.println(timeStep+" "+me().getID()+"AT tasks "+task );
				if (shouldExecute) {
					state = task.getState();
					// System.out.println("  time  " + timeStep + "  id  "
					// + getID() + "   state  " + state);
					task.execute();
					executed = true;
				}
			}
		}
		if (!executed) {
			System.err.println(this + "Task Not Executed" + "In" + timeStep);
		}

	}
	
	public Road getStuckEntrance(Human human){//wujie
		StandardEntity location = human.getPosition(world);
       Road entrance = null;
       if(location instanceof Building){
	       entrance=getEntranceRoad((Building) location);
	       if(entrance!=null)
	        if(entrance.isBlockadesDefined()&&entrance.getBlockades()!=null&&!entrance.getBlockades().isEmpty()){
	    		return entrance;
	         }
        }
		return null;
	}
	
	public Road getStuckRoad(Human human, boolean isArea) {
		StandardEntity location = human.getPosition(world);
		if (location instanceof Road) {
			Road road = (Road) location;
			if (road.isBlockadesDefined()) {
				List<EntityID> blockades = road.getBlockades();
				for (EntityID blockadeID : blockades) {
					Blockade blockade = (Blockade) world.getEntity(blockadeID);
					if (isArea) {
						if(blockade!=null&& blockade.getShape()!=null){//niu13.10.14
							
						if (blockade.getShape().intersects(
								PositionLocate.getBounds(human, world))) {
							return road;
						}
						}
					} else {
						if(blockade!=null && blockade.getShape()!=null){//niu13.10.14
						if (blockade.getShape()
								.contains(
										PositionLocate.getPosition(
												human.getID(), world))) {
							return road;
						}
						}
					}
				}
			}
		}
		return null;
	}

	public Blockade getStuckBlockade(Human human, boolean isArea) {
		StandardEntity location = human.getPosition(world);
		if (location instanceof Road) {
			Road road = (Road) location;
			if (road.isBlockadesDefined()) {
				List<EntityID> blockades = road.getBlockades();
				try{
				for (EntityID blockadeID : blockades) {
					Blockade blockade = (Blockade) world.getEntity(blockadeID);
					if (isArea) {
						if(blockade.getShape()!=null){//niu 13.10.14
						  if(PositionLocate.getBounds(human, world)!=null){
						     if (blockade.getShape().intersects(
								PositionLocate.getBounds(human, world))) {
							return blockade;
						     }
						   }
						}
					} else {
						if(blockade!=null&& blockade.getShape()!=null){//niu 13.10.14
						if (blockade.getShape()
								.contains(
										PositionLocate.getPosition(
												human.getID(), world))) {
							return blockade;
						}
					}
					}
				}
				}catch(Exception e){}
			}
		}
		return null;
	}

	public boolean isStuck(Human human, boolean isArea) {
		Blockade blocakde = getStuckBlockade(human, isArea);
		if (blocakde == null) {
			return false;
		} else {
			return true;
		}
	}

	public boolean isBuried() {
		Human me = getMeAsHuman();
		if (me.isBuriednessDefined()) {
			if (me.getBuriedness() > 0) {
				return true;
			}
		}
		return false;

	}

	protected boolean isBuriedCallHelp() {
		Human me = getMeAsHuman();
		if (me.isBuriednessDefined()) {
			if (me.getBuriedness() > 0) {
				AgentIsBuriedMessage agentIsBuriedMessage = new AgentIsBuriedMessage(
						location().getID().getValue(), transidoldtonew(me
								.getID().getValue()), timeStep, me.getHP(),
						me.getBuriedness(), me.getDamage());
				sendMessage(agentIsBuriedMessage, MessagePriority.High);
				// System.out.println(me() + " I AM BURIEDDD !!!!!!!!!!!!!!!!");
				return true;
			}
		}
		return false;

	}

	public PositionHistory getTravelMemory() {
		return positionHistory;
	}

	public boolean isInRefuge() {
		boolean inRefuge;
		StandardEntity location;

		location = location();
		inRefuge = location instanceof Refuge;
		return inRefuge;
	}
	public boolean hasnoRefuge() {//niu13.10.16
		if(getAllRefuges().isEmpty()){
			return true;
		}else{
			return false;
		}
	}
	public boolean isAtSafePlace() {//niu paris4
		
		StandardEntity location;
		location = location();
		//if()
		//{
			//return true;
		//}
		return false;
		//HydrantInUseMessage hydrantInUseMessage = new HydrantInUseMessage(
		//		location.getID(),timeStep);
		//sendMessage(hydrantInUseMessage,MessagePriority.High);
		//System.out.println(" sendHydrantInUsemessage ");
		
		
	}
	
	
	
	
	public SEUWorldModel getModel() {
		return world;
	}

	public void sendLoad(Human human) {
		sendLoad(timeStep, human.getID());
	}

	public void sendUnload() {
		sendUnload(timeStep);
	}

	public void sendRest() {
		sendRest(timeStep);
	}

	public boolean canSee(EntityID id) {
		Set<EntityID> ids;
		boolean canSee;

		ids = changeSet.getChangedEntities();
		canSee = ids.contains(id);
		return canSee;
	}

	@Override
	public String toString() {
		return this.getID().toString();
	}

	@SuppressWarnings("unchecked")
	public List<E> getNearbyTeamAgents() {
		Set<EntityID> nearEntities;
		List<EntityID> team;
		List<E> teamAgents;

		team = world.getTeam();
		nearEntities = changeSet.getChangedEntities();
		teamAgents = new ArrayList<E>();
		for (EntityID id : team) {
			if (nearEntities.contains(id)) {
				E agent;
				agent = (E) world.getEntity(id);
				teamAgents.add(agent);
			}
		}
		return teamAgents;
	}

	public void sendGetTaskAssigmentMessage() {
		// GetTaskAssignmentMessage getTaskAssignmentMessage = new
		// GetTaskAssignmentMessage();
		// sendMessage(getTaskAssignmentMessage, getMessageChannel(),
		// Priority.High);
	}

	public Path getPathToBlock(Blockade blockade) {
		Path path;
		EntityID id;
		Area destination;
		Point p, humanPoint;
		Human human;

		human = getMeAsHuman();
		id = blockade.getPosition();
		destination = getModel().getEntity(id, Area.class);
		humanPoint = PositionLocate.getPosition(human, getModel());
		p = RoadUtilities.closestPointToBlock(blockade, humanPoint.x,
				humanPoint.y);
		path = getSearch().getPath(human, destination, p, PathType.Shortest);
		return path;
	}

	public GroupingType getGroupingType() {
		return GroupingType.getGroupingType(me().getStandardURN());
	}

	/**
	 * Return type is CommunicationCondition,including states like
	 * NoCommunication,Limited,Medium,Good total bandwith<400 limit 400<total
	 * bandwith<5000 medium total bandwith>5000 good
	 */
	public abstract List<MessageType> getMessagesToListen();

	/*public CommunicationCondition getCommCondition() {
		if (this.sayMessageTransfer.getChannelCount() == 1)
			communicationCondition = CommunicationCondition.NoCommunication;
		else if (getmaxbandwidth() <=128)
			communicationCondition = CommunicationCondition.Limited;
		else if (getmaxbandwidth() <=1024)
			communicationCondition = CommunicationCondition.Medium;
		else
			communicationCondition = CommunicationCondition.Good;
		return communicationCondition;

	}*/
public CommunicationCondition getCommCondition() {
		
		int i;
		int  goodchannels=0;
		if(getTimeStep()>=6){  ////6周期后再调用getchannelRealConditionList
		
       int [] CommRealConditionList=getchannelRealConditionList();
      for(i=0;i<CommRealConditionList.length;i++){
    	  if(CommRealConditionList[i]>50) goodchannels++;
      }
		}
		if (this.sayMessageTransfer.getChannelCount() == 1||(goodchannels==0&&getTimeStep()>=6))
			communicationCondition = CommunicationCondition.NoCommunication;
		else if (getmaxbandwidth() <=128||(goodchannels==1&&getTimeStep()>=6))
			communicationCondition = CommunicationCondition.Limited;
		else if (getmaxbandwidth() <=1024||(goodchannels>1&&getTimeStep()>=6))
			communicationCondition = CommunicationCondition.Medium;
		 		else
			communicationCondition = CommunicationCondition.Good;
		
		//System.out.println("CommunicationCondition:  "+communicationCondition);
		
		
		return communicationCondition;

	}
	public int getmaxbandwidth(){     ///wan
		int bandwidths;
		int maxBandwidth=0;
		if( this.sayMessageTransfer.getChannelCount()>1){
			for (int i = 1; i < this.sayMessageTransfer.getChannelCount(); i++) {
	        bandwidths = config.getIntValue("comms.channels." + i + ".bandwidth");
	        if (bandwidths> maxBandwidth) {
	            maxBandwidth = bandwidths;
	        }
	      }
		}
		return maxBandwidth;
	}
	private int totalBandWith() {
		double sum = 0;
		int i, n = (sayMessageTransfer.getChannelCount() - 1);
		if (n == 0)
			return 0;
		for (i = 1; i <= n; i++)
			sum += sayMessageTransfer.getBandWith(i);
		return (int) sum;

	}

	public int[] getChannelsToSubscribe() {
		List<Integer> channelList;
		int[] channels;
		int max;

		max = config.getIntValue(MAX_PLATOON_CHANNELS_KEY);
		channelList = new ArrayList<Integer>();
		for (MessageType type : getMessagesToListen()) {
			if (channelList.size() >= max) {
				break;
			}
			int channel;
			// channelList.add(0);
			channel = sayMessageTransfer.getChannel(type);
			if (!channelList.contains(channel)) {
				channelList.add(channel);
			}
		}
		channels = new int[channelList.size()];
		for (int i = 0; i < channels.length; i++) {
			channels[i] = channelList.get(i);
		}
		return channels;
	}

	public List<Building> getBurningBuildingsFromChangeSet() {
		return getBurningBuildingsFromChangeSet(true, false);
	}

	public List<Building> getBurningBuildingsFromChangeSet(boolean sort,
			boolean noInfono) {
		List<Building> buildings;

		buildings = burningBuildingsInChangeSet;
		if (noInfono) {
			buildings = removeInfono(buildings);
		}
		if (sort) {
			Collections.sort(buildings, new DistanceComparator(location(),
					world));
		}
		return buildings;
	}

	// 移出没有着火的房屋
	public List<Building> removeInfono(List<Building> buildings) {
		List<Building> list;

		list = new ArrayList<Building>();
		for (Building building : buildings) {
			if (building.isFierynessDefined()) {
				list.add(building);
			} else {
				if (building.getFierynessEnum() != Fieryness.INFERNO) {
					list.add(building);
				}
			}
		}
		return list;
	}

	boolean isCivilianCommand(AKSpeak command) {
		int channel;
		EntityID id;

		channel = command.getChannel();
		id = command.getAgentID();
		if (channel == 0) {
			StandardEntity entity;

			entity = model.getEntity(id);
			if (entity == null || entity instanceof Civilian) {
				return true;
			}
		}
		return false;
	}

	public void processCivilianCommand(AKSpeak speakCommand) {
		// if (me().getStandardURN() == StandardEntityURN.POLICE_FORCE) {
		// System.out
		// .println("llllllllllllllllllllllllllllllllllllllllllllllllllllll");
		// }
		// if (me().getStandardURN() == StandardEntityURN.FIRE_BRIGADE) {
		// System.out
		// .println("ffffffffffffffffffffffffffffffffffffffffffffffffffff");
		// }
		CivilianMaybeMessage message = new CivilianMaybeMessage(getMeAsHuman()
				.getPosition());
		sendMessage(message, MessagePriority.Medium);
		//System.out.println("llllllllllllllllllllllllllllllllllllllllllllllllllllll"+getMeAsHuman().getPosition());
	}

	protected void processBuildingIsBurningMessage(
			BuildingIsBurningMessage message) {
		EntityID buildingId = message.getBuildingId();
		//if(message.getImportance()==5){
		//System.out.println("process gasstation"+buildingId);
		//}
		//System.out.println("process BuildingIsBurningMessage"+getMeAsHuman().getID()+"      "+buildingId);
		// mKeeper.addMessage(message);
		try {

			StandardEntity s = world.getEntity(buildingId);
			if (s instanceof Building) {
				Building building = (Building) world.getEntity(buildingId);
				BuildingInfo bi = world.getBuildingInfo(building.getID());
				if (message.getTimeStep() < bi.getLastUpdateTime()) {
					return;
				}

				if (building == null) {
					building = new Building(buildingId);
					building.setFieryness(message.getFieryness());
					building.setTemperature(message.getTemperature());
					world.addEntity(building);
				} else {
					building.setFieryness(message.getFieryness());
					building.setTemperature(message.getTemperature());
					building.setImportance(message.getImportance());//niu
					addFireBuilding(building);
					if(timeStep-message.getTimeStep()<10){//家入喊话要改,消息延迟.
					SEUBuilding seuBuilding = world.getSEUBuilding(building.getID());
					seuBuilding.setEnergy(building.getTemperature() * seuBuilding.getCapacity());
					switch (building.getFieryness()) {
		                case 0:
		                    seuBuilding.setFuel(seuBuilding.getInitialFuel());
		                    break;
		                case 1:
		                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.66) {
		                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.75));
		                    } else if (seuBuilding.getFuel() == seuBuilding.getInitialFuel()) {
		                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.90));
		                    }
		                    break;

		                case 2:
		                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.33
		                            || seuBuilding.getFuel() > seuBuilding.getInitialFuel() * 0.66) {
		                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.50));
		                    }
		                    break;

		                case 3:
		                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.01
		                            ||seuBuilding.getFuel() > seuBuilding.getInitialFuel() * 0.33) {
		                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.15));
		                    }
		                    break;

		                case 8:
		                	seuBuilding.setFuel(0);
		                    break;
		            }
					if(message.getFieryness()==4 || message.getFieryness()==5){//niu 14.6.26
						seuBuilding.setWasEverWatered(true);
						seuBuilding.setEnergy(0);
					}
					}
				}
			}
		} catch (Exception e) {
			System.err.println("There is No Such Building");
		}

	}


	protected void processGasStationIsBurningMessage(GasStationIsBurningMessage message){//NIU 
		EntityID GSId = message.getBuildingId();
	
		Building g=(Building)world.getEntity(GSId);
		BuildingInfo gs=new BuildingInfo(g);
		
	}
	
	
	
	protected void processBuildingIsExtinguishedMessage(
			BuildingIsExtinguishedMessage message) {
		EntityID buildingId = message.getBuildingId();

		Building building = null;

		try {
			StandardEntity s = world.getEntity(buildingId);
			if (s instanceof Building) {
				building = (Building) world.getEntity(buildingId);
				mKeeper.clear(building);
				
				if (building == null) {
					building = new Building(buildingId);
					building.setFieryness(message.getFieryness());
					world.addEntity(building);
				} else {
					building.setFieryness(message.getFieryness());
					if(timeStep-message.getTimeStep()<10){
					SEUBuilding seuBuilding = world.getSEUBuilding(building.getID());
					switch (building.getFieryness()) {
	                case 0:
	                    seuBuilding.setFuel(seuBuilding.getInitialFuel());
	                    break;
	                case 1:
	                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.66) {
	                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.75));
	                    } else if (seuBuilding.getFuel() == seuBuilding.getInitialFuel()) {
	                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.90));
	                    }
	                    break;

	                case 2:
	                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.33
	                            || seuBuilding.getFuel() > seuBuilding.getInitialFuel() * 0.66) {
	                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.50));
	                    }
	                    break;

	                case 3:
	                    if (seuBuilding.getFuel() < seuBuilding.getInitialFuel() * 0.01
	                            ||seuBuilding.getFuel() > seuBuilding.getInitialFuel() * 0.33) {
	                    	seuBuilding.setFuel((float) (seuBuilding.getInitialFuel() * 0.15));
	                    }
	                    break;

	                case 8:
	                	seuBuilding.setFuel(0);
	                    break;
	            }
	            seuBuilding.setEnergy(building.getTemperature() * seuBuilding.getCapacity());
	            if(message.getFieryness()==4 || message.getFieryness()==5){
	            	seuBuilding.setEnergy(0);
	            	seuBuilding.setWasEverWatered(true);
	            }
				}
				}
			}
		} catch (Exception e) {
			System.err.println("There Is No Such Building");
		}

	}

	public List<Building> getObserveBuilding() {
		List<Building> building = new ArrayList<Building>();
		for (EntityID e : observableBuildings) {
			StandardEntity s = world.getEntity(e);
			building.add((Building) s);
		}
		return building;
	}

	public List<Human> getFoundAgents() {
		return foundAgents;
	}

	private void processBuildingHeat(Building building) {

		BuildingInfo buildingInfo;

		buildingInfo = world.getBuildingInfo(building.getID());
		if (buildingInfo.isHeating()) {
			if (!(building instanceof Refuge)) {
				
				heatingBuildings.add(building);
			}
		}
		buildingInfo.updateTemperature(timeStep);
	}
	
	public List<Building> getHeatingBuildings() {
		return heatingBuildings;
	}
	
	public List<Civilian> getCiviliansAround() {
		return civiliansAround;
	}

	public Road getEntranceRoad(Building building) {
		for (EntityID next : building.getNeighbours()) {
			StandardEntity entity = world.getEntity(next);
			if (entity instanceof Road) {
				return (Road) entity;
			}
		}
		return null;
	}

	public Blockade getClosestBlockadeInRoad(Road road) {
		Human human;
		List<Blockade> blockades;
		MultiBlockDistanceComparator comparator;
		Point position;
		Blockade target;

		human = getMeAsHuman();
		position = PositionLocate.getPosition(human, world);
		comparator = new MultiBlockDistanceComparator(getModel(), position);

		if (!road.isBlockadesDefined()) {
			return null;
		}
		blockades = getModel().getBlockades(road);
		if (blockades.isEmpty()) {
			return null;
		}
		Collections.sort(blockades, comparator);
		target = blockades.get(0);
		return target;
	}

	int[] newid;
	// List<EntityID> G;
	int num_entity;

	public void idtoid() {
		newid = new int[200];
		num_entity = -1;
		for (StandardEntity s : world
				.getEntitiesOfType(StandardEntityURN.AMBULANCE_TEAM)) {
			num_entity++;
			newid[num_entity] = s.getID().getValue();
		}
		for (StandardEntity s : world
				.getEntitiesOfType(StandardEntityURN.FIRE_BRIGADE)) {
			num_entity++;
			newid[num_entity] = s.getID().getValue();
		}
		for (StandardEntity s : world
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE)) {
			num_entity++;
			newid[num_entity] = s.getID().getValue();
		}
		// newid = new int[200];
		int i, j, k, k1;
		for (i = 0; i <= num_entity; i++) {
			k = newid[i];
			k1 = i;
			for (j = i; j <= num_entity; j++) {
				if (newid[j] < k) {
					k = newid[j];
					k1 = j;
				}
			}
			newid[k1] = newid[i];
			newid[i] = k;

		}

		// for (i = 0; i <= num_entity; i++) {
		// System.out.printf(i + "    " + newid[i] + "     ");
		//
		// }

	}

	public EntityID transidoldtonew(int old) {
		int i;
		for (i = 0; i <= num_entity; i++)
			if (newid[i] == old)
				return new EntityID(i);
		return new EntityID(-1);
	}

	public EntityID transidnewtoold(int nnew) {
		int i;
		
		if (nnew < 0 || nnew > num_entity) 
			return new EntityID(-1);
		return new EntityID(newid[nnew]);
	}

	public void TranslateID() {
		newid = new int[200];
		num_entity = -1;
		for (StandardEntity s : world
				.getEntitiesOfType(StandardEntityURN.AMBULANCE_TEAM)) {
			num_entity++;
			newid[num_entity] = s.getID().getValue();
		}
		for (StandardEntity s : world
				.getEntitiesOfType(StandardEntityURN.FIRE_BRIGADE)) {
			num_entity++;
			newid[num_entity] = s.getID().getValue();
		}
		for (StandardEntity s : world
				.getEntitiesOfType(StandardEntityURN.POLICE_FORCE)) {
			num_entity++;
			newid[num_entity] = s.getID().getValue();
		}
		// newid = new int[200];
		int i, j, k, k1;
		for (i = 0; i <= num_entity; i++) {//id从小到大 
			k = newid[i];
			k1 = i;
			for (j = i; j <= num_entity; j++) {
				if (newid[j] < k) {
					k = newid[j];
					k1 = j;
				}
			}
			newid[k1] = newid[i];
			newid[i] = k;

		}

		// for (i = 0; i <= num_entity; i++) {
		// System.out.printf(i + "    " + newid[i] + "     ");
		//
		// }

	}

	// 更新世界模型，用于更新接受到的消息中的世界模型
	public void merge(Entity toMerge) {
		Entity existing = world.getEntity(toMerge.getID());
		if (existing == null) {
			world.addEntity(toMerge);
			// System.out
			// .println(timeStep + " " + getID() + " " + toMerge.getID());

		} else {
			Set<Property> props = existing.getProperties();
			for (Property prop : props) {
				Property other = toMerge.getProperty(prop.getURN());
				if (other.isDefined()) {
					prop.takeValue(other);
				}
			}
		}
	}

	public PredictDamageTool getPredictDamageTool() {
		return predictDamageTool;
	}

	public Map<Integer, seuCommand> getCommands() {
		return commands;
	}

	public EntityID getLocation(int time) {
		return locationHistory.get(time);
	}

	public PositionHistory getPositionHistory() {
		return positionHistory;
	}

	public static double getAgentRadius() {
		return AGENT_RADIUS;
	}
	
	 
	public Building getNearestFire() {//niu
		int dis=500000 ;
		Building target = null;
		Collection<Building> e = world.getEntitiesOfType(Building.class,
				StandardEntityURN.BUILDING);
		List<Building> result = new ArrayList<Building>();
		
		for (Building b : e) {
			if (b.isOnFire() && (world
					.getTotalBurningArea(b) > 100 || getTimeStep()<50)) {
				result.add(b);
			}
		}	
		if(result!=null){
			
		for (Building h : result) {
			int temp = world.getDistance(getID(), h.getID());
			if (temp < dis) {
				dis = temp;
				target = h;
			}
		
		}
		return target;
		}else{
			return null;
		}
	}
	public Hydrant getNearestHydrant() {//niu
		int dis = Integer.MAX_VALUE;
		//Hydrant target = null;
		List<Hydrant>result=getAllHydrants();
		Hydrant targetHydrant=null;
		//Collections.sort(result, new DistanceComparator(location(), world));
		for (Hydrant h : result) {
			
			
			if (canfillinhydrant(h) 
					
					&& !getMeAsHuman().getPosition(world).equals(h)) {
				//dis = temp;
				
				targetHydrant = h;
				break;
			
			}
		   // }
		}
		
		//System.out.println("TARGET  "+target.getID());
		//HydrantInUseMessage hydrantInUseMessage = new HydrantInUseMessage(
		//		target.getID(),timeStep);
		//sendMessage(hydrantInUseMessage,MessagePriority.High);
		//System.out.println("sendHydrantinUseMessage"+target.getID() );
		
		return targetHydrant;
	}
	

	public GasStation getNearestGasStation() {//niu
		int dis = Integer.MAX_VALUE;
		GasStation target = null;
		for (GasStation h : this.getAllGasStations()) {
			int temp = world.getDistance(getID(), h.getID());
			if (temp < dis) {
				dis = temp;
				target = h;
			}
		}
		return target;
	}
	
	public List<StandardEntity> getEntityInsight() {
		List<StandardEntity> entitiesInSight = new ArrayList<StandardEntity>();
		for (EntityID id : changeSet.getChangedEntities()) {
			entitiesInSight.add(world.getEntity(id));
		}
		return entitiesInSight;
	}

	public boolean isInClearRange(Blockade b) {
		int x = this.getMeAsHuman().getX();
		int y = this.getMeAsHuman().getY();
		List<Line2D> lines = GeometryTools2D.pointsToLines(
				GeometryTools2D.vertexArrayToPoints(b.getApexes()), true);
		double best = Double.MAX_VALUE;
		Point2D origin = new Point2D(x, y);
		for (Line2D next : lines) {
			Point2D closest = GeometryTools2D.getClosestPointOnSegment(next,
					origin);
			double d = GeometryTools2D.getDistance(origin, closest);
			if (d < best) {
				best = d;
			}
		}
		int intBest = (int) best;
		if (maxDistance < 0 || intBest < maxDistance) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public void sendMove(int time, List<EntityID> path) {
		// TODO Auto-generated method stub
		super.sendMove(time, path);
		// System.out.println(this.getTimeStep()+" time " +this.getID()+this.getName()+  " sendmove "+path);

	}

	@Override
	public void sendMove(int time, List<EntityID> path, int destX, int destY) {
		// TODO Auto-generated method stub
		super.sendMove(time, path, destX, destY);
	}

	/**
	 * ALL THE AGENT GET THE CHANNEL ONE
	 * 
	 * @return
	 */
	public int[] getFirstChannelToSusbscribe() {
		int[] channels;
		channels = new int[1];
		channels[0] = 1;
		return channels;

	}

	void sendVoiceMessageRepeatly() {
		mKeeper.clearOldMessages(this.getTimeStep());
		mKeeper.sendMessageRepeatlly();
	}

	@Override
	public void sendSubscribe(int time, int... channels) {
		// TODO Auto-generated method stub
		super.sendSubscribe(time, channels);
	}

	public void addAgentIsStuckTask(ClearPath cp) {

	}
	
	public boolean refillchoose(Entity refuge,Entity hydrant){//选择refuge or hydrant加水 
		Entity first=refuge;
		Entity second=hydrant;
		//System.out.print("refillchoose\n");
		int temp1 = world.getDistance(getID(), first.getID());
		int temp2 = world.getDistance(getID(), second.getID());
		if(temp1>temp2*2) return true;
		else return false;
	}
	
	public List<FireBrigade> getAllFirebrigades() {   //niu 14.2.9
		List<FireBrigade> result = world.getEntitiesOfType(FireBrigade.class,
				StandardEntityURN.FIRE_BRIGADE);
		return result;
	}
	public boolean canfillinhydrant(Hydrant h) {//比较FB的ID
		
		if (h == null || (getlastHydrant()!=null && h.equals(getlastHydrant()))) {
//			if(getlastHydrant()!=null && h.equals(getlastHydrant())){
//				System.out.println(getID()+"lastfailhydrant"+getlastHydrant());
//			}
			return false;
		}
		Path path =getPathToHydrant(h);
		if(!getAllRefuges().isEmpty()){
			if( path.getEntities().size()>20){
		return false;
			}
		}else{
			if( path.getEntities().size()>40){
				return false;
					}
		}
		List<EntityID> entities=h.getNeighbours();
//		
//		List<Building> ebuildings=new ArrayList<Building>();
//		for (EntityID next : entities){
//			StandardEntity e=world.getEntity(next);
//			if(e instanceof Building){
//				Building B=(Building) e;
//				if(((B.isFierynessDefined() && B.getFieryness()<2) ||
//						 B.isFierynessDefined())
//						&& !ebuildings.contains(B)){
//					ebuildings.add(B);
//				}
//			}
//		}
		//if(!ebuildings.isEmpty()){
		try{
		for (FireBrigade next : getAllFirebrigades()){
			if(next.isPositionDefined() && next.getPosition(world).equals(h)  
					&& next.getID().getValue()<this.getID().getValue())//wj 
				return false;
					
			}
		}catch(Exception e){}
		return true;
//		}else{
//			return false;
//		}
		
		
			
		}
	
	public boolean isbuildinginpartiontraversed(){     //// wan14.3自己分区的房屋是否遍历完
		 
		 List<Building> buildings=  getnofoundbuildinginpartion();
		  if(buildings.isEmpty())
			 return true;
		 return false;
		 }
		
public List<Building> getnofoundbuildinginpartion(){      ////////wan 14.3
	
	List<Building> buildings;
	buildings = new ArrayList<Building>();
	for(Building next: getBuildingsInPartition())
	{
		if(nofoundbuilding.contains(next)) buildings.add(next)	;
	
   }
	return buildings;
}

public int rescuerange(){          //////依照地图的大小分散救援大小范围
	world.calculateMap();
   double mapDimension = Math.hypot(world.getMapWidth(), world.getMapHeight());
   double rate = mapDimension /31445.392;//10000是目测最大距离  31445.392 是大致移动速度  （距离是两地连线）
	if(rate>60) return 3*10000;
	else if(rate>30) return 2*10000;
	else return 1*10000;
	
	
	
}
public  double getdistoceterofpartion(int X,int Y){         //// 分区的中心
   int minX = 0;
   int minY = 0;
   int maxX = 0;
   int maxY = 0;

   List<Area> entities;

    entities = new ArrayList<Area>();
    entities.addAll(getBuildingsInPartition());
   entities.addAll(getRoadsInPartition());
   if(entities.isEmpty()){
	   System.out.println("没有分区");
	   return 1;
   }
   
   
   for (Area area : entities) {
	 
	  int x = area.getX();
	  int y = area.getY();
	 
		minX = Math.min(x, minX);
		maxX = Math.max(x, maxX);
		minY = Math.min(y, minY);
		maxY = Math.max(y, maxY);
	 
   }
   double x = (minX + maxX) / 2.0;
   double y = (minY + maxY) / 2.0;


return  Math.hypot(x-X, y-Y);
}
public double getbenefit(Human human){
double benefit=Math.ceil(human.getHP()/(double)human.getDamage()*0.8)	;
return 100/(human.getBuriedness()*benefit+1);
}
public Refuge getNearestRefuge( List<Refuge>refuges) {             ///////wan
	int dis = Integer.MAX_VALUE;
	Refuge target = null;
	for (Refuge r :refuges) {
		int temp = world.getDistance(getID(), r.getID());
		if (temp < dis) {
			dis = temp;
			target = r;
		}
	}
	return target;
}
public Refuge getNearestRefuge() {           
	int dis = Integer.MAX_VALUE;
	Refuge target = null;
	for (Refuge r : this.getAllRefuges()) {
		int temp = world.getDistance(getID(), r.getID());
		if (temp < dis) {
			dis = temp;
			target = r;
		}
	}
	return target;
}
public double getNearestdistorefuge(Human human) {            /////wan 
	 List<Refuge> refuges;
	 refuges = new ArrayList<Refuge>();
	 int dis = Integer.MAX_VALUE;
	 for (Refuge r : this.getAllRefuges()) {
//		 Path path = search.getPath(human, (Area) r,  PathType.Shortest);
//		 if(path!=null&&!path.isEmpty()&&path.isPassable()){
			  
			 refuges.add(r);
//		 }
	 }
	if(!refuges.isEmpty()){
		for(Refuge next:refuges ) {
			int temp = world.getDistance(getID(), next.getID());
			if (temp < dis) {
				dis = temp;	 
			}
		}
		return dis;
	} 
	return 1;
	 } 
public int gettimeforfire(Human human){      /////  wan  暂时设置1000
	return (int)getNearestdistofire( human)/1000;
	
}
public double getNearestdistofire(Human human){                   ///wan
	int dis=500000000;
	 if(world.getEntity(human.getPosition())instanceof Road)  return dis;
	Collection<Building> e = world.getEntitiesOfType(Building.class,
			StandardEntityURN.BUILDING);
	List<Building> result = new ArrayList<Building>();
	
	for (Building b : e) {
		if (b.isOnFire()) {
			result.add(b);
		}
	}	
	if(result!=null){
		
	for (Building h : result) {
		int temp = world.getDistance(getID(), h.getID());
		if (temp < dis) {
			dis = temp;
		}
	
   }
	
	}
	return dis;
}
public SEUWorldModel getSEUWorldModel()               //// wan 14.02.19     
{ 
	return world;
}
public List<PoliceForce> getAllPoliceForce() {   //niu 
	List<PoliceForce> result = world.getEntitiesOfType(PoliceForce.class,
			StandardEntityURN.POLICE_FORCE);
	return result;
}
public List<PoliceOffice> getAllPoliceOffice() {   //wan 
	List<PoliceOffice> result = world.getEntitiesOfType(PoliceOffice.class,
			StandardEntityURN.POLICE_OFFICE);
	return result;
}
public List<AmbulanceCentre> getAllAmbulanceCenter() {   //wan 
	List<AmbulanceCentre> result = world.getEntitiesOfType(AmbulanceCentre.class,
			StandardEntityURN.AMBULANCE_CENTRE);
	return result;
}
public List<FireStation> getAllFireStation() {   //wan 
	List<FireStation> result = world.getEntitiesOfType(FireStation.class,
			StandardEntityURN.FIRE_STATION);
	return result;
}

public int[] getchannelRealConditionList(){       ////wan
	return tChannel.getchannelRealConditionList();	
		
	}
 public boolean isSafeRoad(Road road){ ////wan
	 if(!safeRoad.isEmpty()){
	 for(Road next:safeRoad){
		 if(road.equals(next)) return true;
	 }
	 }
	 return false;
	 
 }
 private void updateAgentStates() { ////////////////////wan

     Human human;
for (StandardEntity entity : world.getAllAgents()) {
 human = (Human) entity;

 agentFirstPositionMap.put(entity.getID(), human.getPosition());

 if (human.getPosition(world) instanceof Building) {
 	ATFirstTasks.add(entity.getID());


 } else if (!(entity instanceof PoliceForce)) { 
     PFFirstTasks.add(entity.getID());
//     
 }

}
//   System.out.println("ATFirstTasks:      "+ATFirstTasks.size()) ;
} 

 public List<Hydrant> getReachedHydrants(){
	 return ReachedHydrant;
 }
 public int getReachGStime(){
		return reachGStime;
	}
	public int setReachGStime(int time ){
		reachGStime=time;
		return reachGStime;
	}
	public Hydrant getlastHydrant() {
		return lastFailHydrant;
	}
	public Hydrant setlastHydrant(Hydrant hydrant) {
		lastFailHydrant=hydrant;
		return lastFailHydrant;
	}
	public int getFailInHydrantTime() {
		return FailInHydrantTime;
	}
	public int setFailInHydrantTime(int time) {
		FailInHydrantTime=time;
		return FailInHydrantTime;
	}
	public Area getCurrentPostion()	//ljz 14.4.7
	{
		return currentPostion;
	}
	
	
	
	
	
   public int getAfterShockTime(){//wujie 2014.6.16
		if(this.aftershocktime<300&&this.aftershocktime!=0){
			setAfterShockTime(aftershocktime);
			//System.out.println(this.getID()+"aftershock!!!!!!!!!!!!!"+aftershocktime);
			}else{
		Collection<Pair<Building, Integer>> samebuildings = new ArrayList<Pair<Building, Integer>>();
		if(oldbuildings!=null){
			for(Pair<Building, Integer> b : oldbuildings){
				if(getObserveBuilding().contains(b.first())){
					samebuildings.add(b);	
				} 		
			}
		}
		if(oldbuildings!=null){oldbuildings.clear();}
	    try{
		for(Building b:getObserveBuilding()){
	    	if(b.isBrokennessDefined()){
	    	Pair<Building, Integer>pair=new Pair<Building, Integer>(b, b.getBrokenness());
	    	oldbuildings.add(pair);
	    	}
	    }
	    }catch(Exception e){}
		for(Pair<Building, Integer> building :samebuildings){
			if(building.first().isBrokennessDefined()){
				if(building.first().getBrokenness()>building.second()){			
					setAfterShockTime(getTimeStep());					
					//System.out.println(this.getID()+"aftershock!!!!!!!!!!!!!"+aftershocktime);					
				}else{
					setAfterShockTime(300);
				}
			
			}
		}
	}
//		
		return aftershocktime ;
	}
	public void setAfterShockTime(int aftershock) {
		this.aftershocktime = aftershock;
	}
	public <A extends StandardEntity> ArrayList<A> getVisibleEntities(Class<A> type) {//wujie 2014.6.16
		ArrayList<A> changed = new ArrayList<A>();
		if (lastSense == null || lastSense.getChangeSet() == null)
			return changed;
		for (EntityID element : lastSense.getChangeSet().getChangedEntities()) {
			StandardEntity entity = world.getEntity(element);
			if (type.isInstance(entity))
				changed.add((A) entity);
		}
		return changed;
		
	}
	
	public boolean isStuckInBuilding(Human human){//wujie5.12
		EntityID id = human.getPosition();
		Area area = getModel().getEntity(id, Area.class);
		if(area instanceof Building){
			Building b = (Building)area;
			Road enter = getEntranceRoad(b);
			List<EntityID> blks1 = b.getBlockades();
			List<EntityID> blks2 = enter.getBlockades();
			if((blks1!=null&&!blks1.isEmpty())||(blks2!=null&&!blks2.isEmpty())){
				return true;
			}
		}
		return false;
	}
	
}
