 package seu.message;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import rescuecore2.config.Config;
import seu.message.Type.BlankMessage;
import seu.message.Type.Message;
import seu.message.Type.MessageType;

//该类对信息进行转码，将String转为Byte
//同时对消息进行信道分配
public class MessageTranslator {

	// 消息重复次数，默认只发送一次
	public static final int DEFAULT_REPEAT_COUNT = 1;
	public static final int MAX_MESSAGE_ID = 255;
	public static int isTestPeriod = 0;

	private final int byteRepeatCount;
	private int nextMessageId;
	private final Config config;
	private Map<MessageType, Integer> channelMap;
	private HashMap<Integer, Integer> repeatCounts;
	private int Reapeat;

	/**
	 * MessageTranslator构造类
	 */
	public MessageTranslator(Config config) {
		this(config, DEFAULT_REPEAT_COUNT);
	}

	/**
	 * Construct a Message Translator
	 * 
	 * @param repeatCount
	 *            How many times the message will be repeated
	 */
	public MessageTranslator(Config config, int repeatCount) {
		this.setReapeat(1);
		this.byteRepeatCount = repeatCount;
		this.config = config;

		setTempChannelMap();
		setRepeatCounts();

	}

	public int calculateRepeat(double probability) {
		probability = probability / 100;
		if (probability == 1) {
			return 1;
		}
		double top = Math.log(1 - CONFIDENCE);
		double bottom = Math.log(1 - probability);

		int n = (int) (Math.ceil(top / bottom));
		if (n >= 10)
			n = 10;
		if (n <= 1)
			n = 1;

		// System.out.println("sendCounter:"+n);

		return n;

	}

	private void setTempChannelMap() {
		int n;
		Map<MessageType, Integer> map;

		// Channel数量
		n = getChannelCount(); 
		
		// System.err.println("CHANNEL NUM:"+n);
		map = new HashMap<MessageType, Integer>();
		map.put(MessageType.ChannelCondition, 1);
		if (n == 2) {
			map.put(MessageType.MessageCount1, 1);

		} else if (n == 3) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);

		} else if (n == 4) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);

		} else if (n == 5) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
		} else if (n == 6) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
			map.put(MessageType.MessageCount5, 5);
		} else if (n == 7) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
			map.put(MessageType.MessageCount5, 5);
			map.put(MessageType.MessageCount6, 6);
		} else if (n == 8) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
			map.put(MessageType.MessageCount5, 5);
			map.put(MessageType.MessageCount6, 6);
			map.put(MessageType.MessageCount7, 7);
		} else if (n == 9) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
			map.put(MessageType.MessageCount5, 5);
			map.put(MessageType.MessageCount6, 6);
			map.put(MessageType.MessageCount7, 7);
			map.put(MessageType.MessageCount8, 8);
		} else if (n == 10) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
			map.put(MessageType.MessageCount5, 5);
			map.put(MessageType.MessageCount6, 6);
			map.put(MessageType.MessageCount7, 7);
			map.put(MessageType.MessageCount8, 8);
			map.put(MessageType.MessageCount9, 9);

		} else if (n == 11) {
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
			map.put(MessageType.MessageCount5, 5);
			map.put(MessageType.MessageCount6, 6);
			map.put(MessageType.MessageCount7, 7);
			map.put(MessageType.MessageCount8, 8);
			map.put(MessageType.MessageCount9, 9);
			map.put(MessageType.MessageCount10, 10);

		} else // No Radio Channel can be used
		{
			map.put(MessageType.MessageCount1, 1);
			map.put(MessageType.MessageCount2, 2);
			map.put(MessageType.MessageCount3, 3);
			map.put(MessageType.MessageCount4, 4);
			map.put(MessageType.MessageCount5, 5);
			map.put(MessageType.MessageCount6, 6);
			map.put(MessageType.MessageCount7, 7);
			map.put(MessageType.MessageCount8, 8);
			map.put(MessageType.MessageCount9, 9);
			map.put(MessageType.MessageCount10, 10);
			// System.err.println("!!!!!!!!!!! The number of channels are more than 11 !!!!!!!!!!!");

		}
		channelMap = map;

	}

	public void sense() {
	}

	/**
	 * 将Message转为Byte数组
	 * 
	 * @param message
	 *            进行转码的消息
	 * @return 返回转码成功后的Byte Array
	 */
	public byte[] transmit(Message message) {

		byte[] bytes = message.convertToByteArray();

		if (byteRepeatCount == 1) {
			return bytes;
		}
		// byte[] repeatedBytes = getRepeatedBytes(bytes, byteRepeatCount);

		// return repeatedBytes;

		return bytes;
	}

	public byte[] transmit(byte[] message) {
		return message;
	}

	/**
	 * Translates the byte array to Message object
	 * 
	 * @param bytes
	 *            Byte array message.
	 * @return Message objects of the given byte array.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public Message receive(byte[] bytes) throws IOException,
			ClassNotFoundException {
		if (bytes == null || bytes.length == 0) {
			// LOG.warn("Received empty message");
			// if(bytes == null)System.err.println("null");
			// else System.err.println("L=0");
			return new BlankMessage();
		}
		bytes = getNonRepeatedBytes(bytes, byteRepeatCount);
		// logger.info("received bytes:" + bytes.length);
		if (!Message.isValid(bytes)) {
			// LOG.warn("Received corrupted message");
			return new BlankMessage();
		}
		Message message = Message.getMessage(bytes);
		return message;

	}

	/**
	 * Returns the single message from repeated message array.
	 * 
	 * @param msg
	 *            Byte array which contains repeated messages.
	 * @return Returns the extracted byte array.
	 */
	public byte[] getNonRepeatedBytes(byte[] msg) {
		return getNonRepeatedBytes(msg, byteRepeatCount);
	}

	public static byte[] getNonRepeatedBytes(byte[] bytes, int repeatCount) {
		repeatCount = 1;// it must be 1
		if (repeatCount == 1) {
			return bytes;
		}
		byte[] realMessage = new byte[bytes.length / repeatCount];
		for (int i = 0; i < realMessage.length; i++) {
			byte[] temp = new byte[repeatCount];
			for (int j = 0; j < temp.length; j++) {
				int k = i + j * realMessage.length;
				temp[j] = bytes[k];
			}
			realMessage[i] = getCorrectByte(temp);
		}
		return realMessage;
	}

	public static byte getCorrectByte(byte[] bytes) {
		int BYTE_SIZE = 8;
		byte b = 0;
		int base = 1;
		for (int i = 0; i < BYTE_SIZE; i++) {
			int value = 0;
			for (int j = 0; j < bytes.length; j++) {
				if (bytes[j] % 2 != 0) {
					value++;
				}
				bytes[j] = (byte) (bytes[j] >>> 1);
			}
			if (value > bytes.length / 2) {
				b += base;
			}
			base *= 2;
		}
		return b;
	}

	protected static byte[] getRepeatedBytes(byte[] bytes, int repeatCount) {
		byte[] repeatedBytes = new byte[bytes.length * repeatCount];
		for (int i = 0; i < repeatCount; i++) {
			for (int j = 0; j < bytes.length; j++) {
				int k = i * bytes.length + j;
				repeatedBytes[k] = bytes[j];
			}
		}
		return repeatedBytes;
	}

	protected byte[] getRepeatedBytes(byte[] bytes) {
		return getRepeatedBytes(bytes, byteRepeatCount);
	}

	public static final String COMMUNICATIONS_KEY = "comms";
	public static final String CHANNELS_KEY = "channels";
	public static final String NOISE_KEY = "noise";
	public static final String USE_KEY = "use";
	public static final String PROBABILITY_KEY = "p";
	public static final String COUNT_KEY = "count";
	public static final String BANDWITH_KEY = "bandwidth";

	public int getNumofPlatoon()// Add by gq 2012-4-16
	{
		String key = COMMUNICATIONS_KEY + "." + CHANNELS_KEY + ".max.platoon";
		int n = config.getIntValue(key);
		return n;
	}

	public int getChannelCount() {
		String key = COMMUNICATIONS_KEY + "." + CHANNELS_KEY + "." + COUNT_KEY;
		int n = config.getIntValue(key);
		return n;
	}

	public double getBandWith(int channel) {
		String key = COMMUNICATIONS_KEY + "." + CHANNELS_KEY + "." + channel
				+ "." + BANDWITH_KEY;
		double n = config.getFloatValue(key);
		return n;
	}

	protected double getNoise(int channel, String ioType, String noiseType) {
		String key = COMMUNICATIONS_KEY + "." + CHANNELS_KEY + "." + channel
				+ "." + NOISE_KEY + "." + ioType + "." + noiseType;
		String useKey = key + "." + USE_KEY;
		try {
			boolean hasNoise = config.getBooleanValue(useKey);
			if (hasNoise) {
				String probabilityKey = key + "." + PROBABILITY_KEY;
				double p = config.getFloatValue(probabilityKey);
				return p;
			}
		} catch (Exception e) {

		}
		return 0.0;
	}

	public static final String INPUT_KEY = "input";
	public static final String OUTPUT_KEY = "output";
	public static final String DROPOUT_KEY = "dropout";
	public static final String FAILURE_KEY = "failure";

	protected double getTotalNoise(int channel) {
		String[] ioTypes = { INPUT_KEY, OUTPUT_KEY };
		String[] noiseTypes = { DROPOUT_KEY, FAILURE_KEY };
		double sum = 1;
		for (String ioType : ioTypes) {
			for (String noiseType : noiseTypes) {
				double p = getNoise(channel, ioType, noiseType);

				// System.out.print("channel:"+channel+" ioTypes:"+ioType+" noiseType:"+noiseType+" p");
				// System.out.print(p);
				// if(channel==0)System.out.println();
				// else
				// System.out.println(" bandWith:"+this.getBandWith(channel));
				if (p > 0)
					sum = sum * (1 - p);
			}
		}

		return 1 - sum;
	}

	public int getRepeatCount(int channel) {
		return repeatCounts.get(channel);
	}

	public static final double CONFIDENCE = 0.8;

	private int calculateRepeatCount(double probability) {
		if (probability == 0) {
			return 1;
		}
		double top = Math.log(1 - CONFIDENCE);
		double bottom = Math.log(probability);

		int n = (int) (Math.ceil(top / bottom));
		if (n >= 10)
			n = 10;
		// System.out.println("sendCounter:"+n);

		return n;
	}

	private void setRepeatCounts() {
		repeatCounts = new HashMap<Integer, Integer>();
		int n = getChannelCount();
		for (int i = 0; i < n; i++) {
			double p = getTotalNoise(i);
			int repeat = calculateRepeatCount(p);
			repeatCounts.put(i, repeat);
		}
	}

	private void setFinalRepeatCounts(int[] rankChannel,
			int[] channelConditionList) {
		repeatCounts = new HashMap<Integer, Integer>();
		int n = getChannelCount();
		for (int i = 0; i < n - 1; i++) {

			int temp = rankChannel[i] - 1;
			if (temp < 0)
				temp = 0;
			repeatCounts.put(rankChannel[i], channelConditionList[temp]);

		}
		repeatCounts.put(0, 2);
	}

	public void incrementMessageId() {
		nextMessageId = (nextMessageId + 1) % (MAX_MESSAGE_ID + 1);
	}

	public int getNextMessageId() {
		return nextMessageId;
	}
	public void setChannelMap(int[] rankChannel,int n) {  ///m=3||2
		//	int n;
			Map<MessageType, Integer> map;

			// Channel数量
			//n = getChannelCount();

			map = new HashMap<MessageType, Integer>();

			if (n == 2) {
				map.put(MessageType.BuildingIsExploredMessage, rankChannel[0]);
				map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
				map.put(MessageType.ImportantBuildingsMessage, rankChannel[0]);//niu
				map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[0]);//niu
			
				map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[0]);
				map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
				map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
				map.put(MessageType.AgentPositionMessage, rankChannel[0]);
				map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
				map.put(MessageType.ClearPathIsNeededMessage, rankChannel[0]);
				map.put(MessageType.AgentIsStuckMessage, rankChannel[0]);
				map.put(MessageType.AssignRefugeMessage, rankChannel[0]);
				map.put(MessageType.RoadIsClearedMessage, rankChannel[0]);
				map.put(MessageType.AssignFiredBuildingMessage, rankChannel[0]);
				map.put(MessageType.InformTeamMessage, rankChannel[0]);
				map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[0]);
				map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[0]);
				map.put(MessageType.BuildingIsExploreMessage_at, rankChannel[0]);              ///wan
				map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
				map.put(MessageType.BuildingMaybeMessage, rankChannel[0]);
				map.put(MessageType.BuildingMaybeMessage_at, rankChannel[0]);
				map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);
			} else if (n == 3) {
				map.put(MessageType.BuildingIsExploredMessage, rankChannel[1]);
				map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
				
				map.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
				map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
				//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
				map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[1]);//niu
				//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[0]);//niu
				
				map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[1]);
				map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
				map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
				map.put(MessageType.AgentPositionMessage, rankChannel[1]);
				map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[1]);
				map.put(MessageType.ClearPathIsNeededMessage, rankChannel[0]);
				map.put(MessageType.AgentIsStuckMessage, rankChannel[0]);
				map.put(MessageType.AssignRefugeMessage, rankChannel[0]);
				map.put(MessageType.RoadIsClearedMessage, rankChannel[1]);
				map.put(MessageType.AssignFiredBuildingMessage, rankChannel[1]);
				map.put(MessageType.InformTeamMessage, rankChannel[1]);
				map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[1]);
				map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[1]);
				map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
				map.put(MessageType.BuildingMaybeMessage, rankChannel[1]);
				map.put(MessageType.BuildingMaybeMessage_at, rankChannel[1]);
				map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);///lb
				

			} /*else if (n == 4) {
				map.put(MessageType.BuildingIsExploredMessage, rankChannel[1]);
				map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
				map.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
				map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
				//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
				map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[0]);//niu
				//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
				
				map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[1]);
				map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
				map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
				map.put(MessageType.AgentPositionMessage, rankChannel[1]);
				map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
				map.put(MessageType.ClearPathIsNeededMessage, rankChannel[2]);
				map.put(MessageType.AgentIsStuckMessage, rankChannel[2]);
				map.put(MessageType.AssignRefugeMessage, rankChannel[2]);
				map.put(MessageType.RoadIsClearedMessage, rankChannel[2]);
				map.put(MessageType.AssignFiredBuildingMessage, rankChannel[2]);
				map.put(MessageType.InformTeamMessage, rankChannel[1]);
				map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[1]);
				map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[2]);
				map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
				map.put(MessageType.BuildingMaybeMessage, rankChannel[1]);
				map.put(MessageType.BuildingMaybeMessage_at, rankChannel[1]);
				map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);

			} else if (n >= 5) {
				map.put(MessageType.BuildingIsExploredMessage, rankChannel[1]);
				map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
				map.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
				map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
				//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
				map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[0]);//niu
				//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
				
				map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[1]);
				map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
				map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
				map.put(MessageType.AgentPositionMessage, rankChannel[0]);
				map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
				map.put(MessageType.ClearPathIsNeededMessage, rankChannel[2]);
				map.put(MessageType.AgentIsStuckMessage, rankChannel[2]);
				map.put(MessageType.AssignRefugeMessage, rankChannel[2]);
				map.put(MessageType.RoadIsClearedMessage, rankChannel[2]);
				map.put(MessageType.AssignFiredBuildingMessage, rankChannel[1]);
				map.put(MessageType.InformTeamMessage, rankChannel[1]);
				map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[1]);
				map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[2]);
				map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
				map.put(MessageType.BuildingMaybeMessage, rankChannel[1]);
				map.put(MessageType.BuildingMaybeMessage_at, rankChannel[1]);
				map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);

			}*/ 
			if(n==1){
				map.put(MessageType.BuildingIsExploredMessage, 0);
				map.put(MessageType.BuildingIsBurningMessage, 0);
				map.put(MessageType.ImportantBuildingsMessage, 0);//niu
				map.put(MessageType.AssignImportantBuildingsMessage, 0);//niu
				//map.put(MessageType.GasStationIsBurningMessage, 0);//niu
				map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, 0);//niu
				//map.put(MessageType.GasStationIsExtinguishedMessage, 0);//niu
				
				map.put(MessageType.BuildingIsExtinguishedMessage, 0);
				map.put(MessageType.CivilianInformationMessage, 0);
				map.put(MessageType.AgentIsBuriedMessage, 0);
				map.put(MessageType.AgentPositionMessage, 0);
				map.put(MessageType.CivilianIsSavedOrDeadMessage, 0);
				map.put(MessageType.ClearPathIsNeededMessage, 0);
				map.put(MessageType.AgentIsStuckMessage, 0);
				map.put(MessageType.AssignRefugeMessage, 0);
				map.put(MessageType.RoadIsClearedMessage, 0);
				map.put(MessageType.AssignFiredBuildingMessage, 0);
				map.put(MessageType.InformTeamMessage, 0);
				map.put(MessageType.BuildingIsExploreMessage_fb, 0);
				map.put(MessageType.BuildingIsExploreMessage_pf, 0);
				map.put(MessageType.CivilianMaybeMessage, 0);
				map.put(MessageType.BuildingMaybeMessage, 0);
				map.put(MessageType.BuildingMaybeMessage_at, 0);
				map.put(MessageType.BuildingMaybeMessage_pf, 0);
			}

				// System.out
				// .println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~00");

			
			channelMap = map;
		}
		public void setChannelMap1(int[] rankChannel,int n) {  ///m=1  //////////////wan
			//	int n;
				Map<MessageType, Integer> map;

				// Channel数量
				//n = getChannelCount();

				map = new HashMap<MessageType, Integer>();

				if (n == 2) {   ///all is 0 
					map.put(MessageType.BuildingIsExploredMessage, rankChannel[0]);
					map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
					map.put(MessageType.ImportantBuildingsMessage, rankChannel[0]);//niu
					map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[0]);//niu
				
					map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[0]);
					map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
					map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
					map.put(MessageType.AgentPositionMessage, rankChannel[0]);
					map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
					map.put(MessageType.ClearPathIsNeededMessage, rankChannel[0]);
					map.put(MessageType.AgentIsStuckMessage, rankChannel[0]);
					map.put(MessageType.AssignRefugeMessage, rankChannel[0]);
					map.put(MessageType.RoadIsClearedMessage, rankChannel[0]);
					map.put(MessageType.AssignFiredBuildingMessage, rankChannel[0]);
					map.put(MessageType.InformTeamMessage, rankChannel[0]);
					map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[0]);
					map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[0]);
					map.put(MessageType.BuildingIsExploreMessage_at, rankChannel[0]);              ///wan
					map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
					map.put(MessageType.BuildingMaybeMessage, rankChannel[0]);
					map.put(MessageType.BuildingMaybeMessage_at, rankChannel[0]);
					map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);
				} else if (n == 3) {                 ///fb is 0   at/pf  is 1
					map.put(MessageType.BuildingIsExploredMessage, rankChannel[1]);
					map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
					
					map.put(MessageType.ImportantBuildingsMessage, rankChannel[0]);//niu
					map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[0]);//niu
					//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
					map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[0]);//niu
					//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[0]);//niu
					
					map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[0]);
					map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
					map.put(MessageType.AgentIsBuriedMessage, rankChannel[1]);
					map.put(MessageType.AgentPositionMessage, rankChannel[1]);
					map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[1]);
					map.put(MessageType.ClearPathIsNeededMessage, rankChannel[1]);
					map.put(MessageType.AgentIsStuckMessage, rankChannel[1]);
					map.put(MessageType.AssignRefugeMessage, rankChannel[1]);
					map.put(MessageType.RoadIsClearedMessage, rankChannel[1]);
					map.put(MessageType.AssignFiredBuildingMessage, rankChannel[1]);
					map.put(MessageType.InformTeamMessage, rankChannel[1]);
					map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[0]);
					map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[1]);
					map.put(MessageType.CivilianMaybeMessage, rankChannel[1]);
					map.put(MessageType.BuildingMaybeMessage, rankChannel[1]);
					map.put(MessageType.BuildingMaybeMessage_at, rankChannel[1]);
					map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[1]);///lb
					

				} else if (n >= 4) {        ///fb0  pf1  at2 
					map.put(MessageType.BuildingIsExploredMessage, rankChannel[1]);
					map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
					map.put(MessageType.ImportantBuildingsMessage, rankChannel[0]);//niu
					map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[0]);//niu
					//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
					map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[0]);//niu
					//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
					
					map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[0]);
					map.put(MessageType.CivilianInformationMessage, rankChannel[2]);
					map.put(MessageType.AgentIsBuriedMessage, rankChannel[2]);
					map.put(MessageType.AgentPositionMessage, rankChannel[1]);
					map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[2]);
					map.put(MessageType.ClearPathIsNeededMessage, rankChannel[1]);
					map.put(MessageType.AgentIsStuckMessage, rankChannel[1]);
					map.put(MessageType.AssignRefugeMessage, rankChannel[1]);
					map.put(MessageType.RoadIsClearedMessage, rankChannel[1]);
					map.put(MessageType.AssignFiredBuildingMessage, rankChannel[0]);
					map.put(MessageType.InformTeamMessage, rankChannel[0]);
					map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[0]);
					map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[1]);
					map.put(MessageType.CivilianMaybeMessage, rankChannel[2]);
					map.put(MessageType.BuildingMaybeMessage, rankChannel[0]);
					map.put(MessageType.BuildingMaybeMessage_at, rankChannel[2]);
					map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[1]);

				} 
				
			    if(n==1){   //////分配0信道。。。。喊话信道
					map.put(MessageType.BuildingIsExploredMessage, 0);
					map.put(MessageType.BuildingIsBurningMessage, 0);
					map.put(MessageType.ImportantBuildingsMessage, 0);//niu
					map.put(MessageType.AssignImportantBuildingsMessage, 0);//niu
					//map.put(MessageType.GasStationIsBurningMessage, 0);//niu
					map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, 0);//niu
					//map.put(MessageType.GasStationIsExtinguishedMessage, 0);//niu
					
					map.put(MessageType.BuildingIsExtinguishedMessage, 0);
					map.put(MessageType.CivilianInformationMessage, 0);
					map.put(MessageType.AgentIsBuriedMessage, 0);
					map.put(MessageType.AgentPositionMessage, 0);
					map.put(MessageType.CivilianIsSavedOrDeadMessage, 0);
					map.put(MessageType.ClearPathIsNeededMessage, 0);
					map.put(MessageType.AgentIsStuckMessage, 0);
					map.put(MessageType.AssignRefugeMessage, 0);
					map.put(MessageType.RoadIsClearedMessage, 0);
					map.put(MessageType.AssignFiredBuildingMessage, 0);
					map.put(MessageType.InformTeamMessage, 0);
					map.put(MessageType.BuildingIsExploreMessage_fb, 0);
					map.put(MessageType.BuildingIsExploreMessage_pf, 0);
					map.put(MessageType.CivilianMaybeMessage, 0);
					map.put(MessageType.BuildingMaybeMessage, 0);
					map.put(MessageType.BuildingMaybeMessage_at, 0);
					map.put(MessageType.BuildingMaybeMessage_pf, 0);
			         }
					// System.out
					// .println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~00");

				
		
				channelMap = map;
			}
	/*public void setChannelMap(int[] rankChannel) {
		int n;
		Map<MessageType, Integer> map;

		// Channel数量
		n = getChannelCount();

		map = new HashMap<MessageType, Integer>();

		if (n == 2) {
			map.put(MessageType.BuildingIsExploredMessage, rankChannel[0]);
			map.put(MessageType.BuildingIsBurningMessage, rankChannel[0]);
			map.put(MessageType.ImportantBuildingsMessage, rankChannel[0]);//niu
			map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[0]);//niu
		
			map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[0]);
			map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
			map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
			map.put(MessageType.AgentPositionMessage, rankChannel[0]);
			map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
			map.put(MessageType.ClearPathIsNeededMessage, rankChannel[0]);
			map.put(MessageType.AgentIsStuckMessage, rankChannel[0]);
			map.put(MessageType.AssignRefugeMessage, rankChannel[0]);
			map.put(MessageType.RoadIsClearedMessage, rankChannel[0]);
			map.put(MessageType.AssignFiredBuildingMessage, rankChannel[0]);
			map.put(MessageType.InformTeamMessage, rankChannel[0]);
			map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[0]);
			map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[0]);
			map.put(MessageType.BuildingIsExploreMessage_at, rankChannel[0]);              ///wan
			map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
			map.put(MessageType.BuildingMaybeMessage, rankChannel[0]);
			map.put(MessageType.BuildingMaybeMessage_at, rankChannel[0]);
			map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);
		} else if (n == 3) {
			map.put(MessageType.BuildingIsExploredMessage, rankChannel[0]);
			map.put(MessageType.BuildingIsBurningMessage, rankChannel[1]);
			
			map.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
			map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
			//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
			map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[1]);//niu
			//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[0]);//niu
			
			map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[1]);
			map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
			map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
			map.put(MessageType.AgentPositionMessage, rankChannel[1]);
			map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
			map.put(MessageType.ClearPathIsNeededMessage, rankChannel[1]);
			map.put(MessageType.AgentIsStuckMessage, rankChannel[1]);
			map.put(MessageType.AssignRefugeMessage, rankChannel[1]);
			map.put(MessageType.RoadIsClearedMessage, rankChannel[1]);
			map.put(MessageType.AssignFiredBuildingMessage, rankChannel[1]);
			map.put(MessageType.InformTeamMessage, rankChannel[1]);
			map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[1]);
			map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[1]);
			map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
			map.put(MessageType.BuildingMaybeMessage, rankChannel[1]);
			map.put(MessageType.BuildingMaybeMessage_at, rankChannel[1]);
			map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);///lb

		} else if (n == 4) {
			map.put(MessageType.BuildingIsExploredMessage, rankChannel[0]);
			map.put(MessageType.BuildingIsBurningMessage, rankChannel[1]);
			map.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
			map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
			//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
			map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[0]);//niu
			//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
			
			map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[1]);
			map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
			map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
			map.put(MessageType.AgentPositionMessage, rankChannel[1]);
			map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
			map.put(MessageType.ClearPathIsNeededMessage, rankChannel[2]);
			map.put(MessageType.AgentIsStuckMessage, rankChannel[2]);
			map.put(MessageType.AssignRefugeMessage, rankChannel[2]);
			map.put(MessageType.RoadIsClearedMessage, rankChannel[2]);
			map.put(MessageType.AssignFiredBuildingMessage, rankChannel[2]);
			map.put(MessageType.InformTeamMessage, rankChannel[1]);
			map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[1]);
			map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[2]);
			map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
			map.put(MessageType.BuildingMaybeMessage, rankChannel[1]);
			map.put(MessageType.BuildingMaybeMessage_at, rankChannel[1]);
			map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);

		} else if (n >= 5) {
			map.put(MessageType.BuildingIsExploredMessage, rankChannel[0]);
			map.put(MessageType.BuildingIsBurningMessage, rankChannel[1]);
			map.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
			map.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
			//map.put(MessageType.GasStationIsBurningMessage, rankChannel[0]);//niu
			map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[0]);//niu
			//map.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
			
			map.put(MessageType.BuildingIsExtinguishedMessage, rankChannel[1]);
			map.put(MessageType.CivilianInformationMessage, rankChannel[0]);
			map.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
			map.put(MessageType.AgentPositionMessage, rankChannel[0]);
			map.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
			map.put(MessageType.ClearPathIsNeededMessage, rankChannel[2]);
			map.put(MessageType.AgentIsStuckMessage, rankChannel[2]);
			map.put(MessageType.AssignRefugeMessage, rankChannel[2]);
			map.put(MessageType.RoadIsClearedMessage, rankChannel[2]);
			map.put(MessageType.AssignFiredBuildingMessage, rankChannel[1]);
			map.put(MessageType.InformTeamMessage, rankChannel[1]);
			map.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[1]);
			map.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[2]);
			map.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
			map.put(MessageType.BuildingMaybeMessage, rankChannel[1]);
			map.put(MessageType.BuildingMaybeMessage_at, rankChannel[1]);
			map.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);

		} else {
			map.put(MessageType.BuildingIsExploredMessage, 0);
			map.put(MessageType.BuildingIsBurningMessage, 0);
			map.put(MessageType.ImportantBuildingsMessage, 0);//niu
			map.put(MessageType.AssignImportantBuildingsMessage, 0);//niu
			//map.put(MessageType.GasStationIsBurningMessage, 0);//niu
			map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, 0);//niu
			//map.put(MessageType.GasStationIsExtinguishedMessage, 0);//niu
			
			map.put(MessageType.BuildingIsExtinguishedMessage, 0);
			map.put(MessageType.CivilianInformationMessage, 0);
			map.put(MessageType.AgentIsBuriedMessage, 0);
			map.put(MessageType.AgentPositionMessage, 0);
			map.put(MessageType.CivilianIsSavedOrDeadMessage, 0);
			map.put(MessageType.ClearPathIsNeededMessage, 0);
			map.put(MessageType.AgentIsStuckMessage, 0);
			map.put(MessageType.AssignRefugeMessage, 0);
			map.put(MessageType.RoadIsClearedMessage, 0);
			map.put(MessageType.AssignFiredBuildingMessage, 0);
			map.put(MessageType.InformTeamMessage, 0);
			map.put(MessageType.BuildingIsExploreMessage_fb, 0);
			map.put(MessageType.BuildingIsExploreMessage_pf, 0);
			map.put(MessageType.CivilianMaybeMessage, 0);
			map.put(MessageType.BuildingMaybeMessage, 0);
			map.put(MessageType.BuildingMaybeMessage_at, 0);
			map.put(MessageType.BuildingMaybeMessage_pf, 0);

			// System.out
			// .println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~00");

		}
		channelMap = map;
	}*/

	/*public void setChannelMapWhenNoComm() {
		Map<MessageType, Integer> map;
		map = new HashMap<MessageType, Integer>();
		// map.put(MessageType.BuildingIsExploredMessage, 0);
		map.put(MessageType.BuildingIsBurningMessage, 0);
		//map.put(MessageType.GasStationIsBurningMessage, 0);//niu
		//map.put(MessageType.BuildingCloseToGasStationIsBurningMessage, 0);//niu
		// map.put(MessageType.BuildingIsExtinguishedMessage, 0);
		// map.put(MessageType.CivilianInformationMessage, 0);
		 map.put(MessageType.AgentIsBuriedMessage, 0);
		// map.put(MessageType.CivilianIsSavedOrDeadMessage, 0);
		// map.put(MessageType.ClearPathIsNeededMessage, 0);
		 map.put(MessageType.AgentIsStuckMessage, 0);
		// map.put(MessageType.AssignRefugeMessage, 0);
		// map.put(MessageType.RoadIsClearedMessage, 0);
		// map.put(MessageType.AssignFiredBuildingMessage, 0);
		// map.put(MessageType.InformTeamMessage, 0);
		repeatCounts.put(0, 1);
		channelMap = map;

	}*/

	public void setChannelMap(int[] channelRank, int[] channelConditionList,int[] channelRealConditionList
			,int m) {
		int n;
		channelMap.clear();
		Map<MessageType, Integer> map;
		int k;
		int  goodchannels=0;
	      for(k=0;k<channelRealConditionList.length;k++){         ////wan
	    	  if(channelRealConditionList[k]>50) goodchannels++;   ///所有的radio
	      }
		// Channel数量
		n = getChannelCount();
         
		int i, j;
		int[] rankChannel = new int[21];
		for (i = 0; i < n - 1; i++) {
			int point = 0;
			for (j = 0; j < n - 1; j++) {
				if (channelRank[j] == i) {
					rankChannel[i] = j;
				}
			}

		}
		for (i = 0; i < rankChannel.length; i++) {
			rankChannel[i] += 1;
			// System.out.print(rankChannel[i]);

		}
		// System.out.println();
		
		 n=goodchannels+1;  ////wan     只分配好的信道
		 
		this.setFinalRepeatCounts(rankChannel, channelConditionList);
		map = new HashMap<MessageType, Integer>();
		// There are more than 3 channels (n>=3 including channel 0)
		if(m>3)   m=3;  ///////wan
		if (m == 3) {

			switch (n) {
			case 9 + 1:
			case 10 + 1:
			case 11 + 1: 
			case 12 + 1:
			case 13 + 1:
			case 14 + 1:
			case 15 + 1:
			case 16 + 1:
			case 17 + 1:
			case 18 + 1:
			case 19 + 1:
			case 20 + 1:{
				setATChannelMap_3(rankChannel);
				setFBChannelMap_3(rankChannel);
				setPFChannelMap_3(rankChannel);
			}
				break;
			case 8 + 1: {
				setATChannelMap_3(rankChannel);
				setFBChannelMap_3(rankChannel);
				setPFChannelMap_2(rankChannel);

			}
				break;
			case 7 + 1: {
				setATChannelMap_3(rankChannel);
				setFBChannelMap_2(rankChannel);
				setPFChannelMap_2(rankChannel);
			}
				break;
			case 6 + 1: {
				setATChannelMap_2(rankChannel);
				setFBChannelMap_2(rankChannel);
				setPFChannelMap_2(rankChannel);
			}
				break;
			case 5 + 1: {
				setATChannelMap_2(rankChannel);
				setFBChannelMap_2(rankChannel);
				setPFChannelMap_1(rankChannel);
			}
				break;
			case 4 + 1: {
				setATChannelMap_2(rankChannel);
				setFBChannelMap_1(rankChannel);
				setPFChannelMap_1(rankChannel);
			}
				break;
			case 3 + 1: {
				setATChannelMap_1(rankChannel);
				setFBChannelMap_1(rankChannel);
				setPFChannelMap_1(rankChannel);
			}
				break;
			case 2 + 1:
			case 1 + 1:
				setChannelMap(rankChannel,n);
				break;

			}
		} else if (m == 2) {
			switch (n) {
			case 9 + 1:
			case 10 + 1:
			case 11 + 1: 
			case 12 + 1:
			case 13 + 1:
			case 14 + 1:
			case 15 + 1:
			case 16 + 1:
			case 17 + 1:
			case 18 + 1:
			case 19 + 1: 
			case 20 + 1:
			case 8 + 1:
			case 7 + 1:
			case 6 + 1:

			{
				setATChannelMap_2(rankChannel);
				setFBChannelMap_2(rankChannel);
				setPFChannelMap_2(rankChannel);
			}
				break;

			case 5 + 1: {
				setATChannelMap_2(rankChannel);
				setFBChannelMap_2(rankChannel);
				setPFChannelMap_1(rankChannel);
			}
				break;
			case 4 + 1: {
				setATChannelMap_2(rankChannel);
				setFBChannelMap_1(rankChannel);
				setPFChannelMap_1(rankChannel);
			}
				break;
			case 3 + 1: {
				setATChannelMap_1(rankChannel);
				setFBChannelMap_1(rankChannel);
				setPFChannelMap_1(rankChannel);
			}
				break;
			case 2 + 1:
			case 1 + 1:
				setChannelMap(rankChannel,n);
				break;

			}

		} else // m==1
		{
			setChannelMap1(rankChannel,n);

		}
		// System.out.println(channelMap);
		// channelMap = map;
	}

	private void setPFChannelMap_1(int[] rankChannel) {

		// TODO Auto-generated method stub
		channelMap.put(MessageType.AssignImportantBuildingsMessage, rankChannel[2]); //////wan
		channelMap.put(MessageType.ClearPathIsNeededMessage, rankChannel[2]);
		channelMap.put(MessageType.AgentIsStuckMessage, rankChannel[2]);
		channelMap.put(MessageType.AssignRefugeMessage, rankChannel[2]);
		channelMap.put(MessageType.RoadIsClearedMessage, rankChannel[2]);
		channelMap.put(MessageType.InformTeamMessage, rankChannel[2]);
		channelMap.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[2]);
		channelMap.put(MessageType.AssignRefugeMessage, rankChannel[2]);
	///	channelMap.put(MessageType.BuildingMaybeMessage_pf, rankChannel[2]);    //wan

	}

	private void setFBChannelMap_1(int[] rankChannel) {
		// TODO Auto-generated method stub
		channelMap.put(MessageType.BuildingIsBurningMessage, rankChannel[1]);
		channelMap.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
		channelMap.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
		//channelMap.put(MessageType.GasStationIsBurningMessage, rankChannel[1]);//niu
		channelMap.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
		channelMap.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[1]);//niu
		
		channelMap.put(MessageType.BuildingIsExtinguishedMessage,rankChannel[1]);
		channelMap.put(MessageType.BuildingIsExploredMessage, rankChannel[1]);
		channelMap.put(MessageType.AssignFiredBuildingMessage, rankChannel[1]);
		channelMap.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[1]);
	//	channelMap.put(MessageType.BuildingMaybeMessage, rankChannel[1]);

	}

	private void setATChannelMap_1(int[] rankChannel) {
		// TODO Auto-generated method stub
	//	channelMap.put(MessageType.CivilianMaybeMessage, rankChannel[0]);              //wan
		channelMap.put(MessageType.CivilianInformationMessage, rankChannel[0]);
		channelMap.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
		channelMap
				.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
		channelMap.put(MessageType.BuildingIsExploreMessage_at, rankChannel[0]);     //wan
	//	channelMap.put(MessageType.BuildingMaybeMessage_pf, rankChannel[0]);        //wan

	}

	private void setPFChannelMap_2(int[] rankChannel)// 2&&5
	{

		// TODO Auto-generated method stub
		channelMap.put(MessageType.AssignImportantBuildingsMessage, rankChannel[2]);
		channelMap.put(MessageType.ClearPathIsNeededMessage, rankChannel[2]);
		channelMap.put(MessageType.AgentIsStuckMessage, rankChannel[2]);
		channelMap.put(MessageType.AssignRefugeMessage, rankChannel[2]);
		channelMap.put(MessageType.RoadIsClearedMessage, rankChannel[5]);
		channelMap.put(MessageType.InformTeamMessage, rankChannel[5]);
		channelMap.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[5]);
		channelMap.put(MessageType.AssignRefugeMessage, rankChannel[2]);
	//	channelMap.put(MessageType.BuildingMaybeMessage_pf, rankChannel[2]);  //wan
	}

	private void setFBChannelMap_2(int[] rankChannel) // 1&&4
	{
		// TODO Auto-generated method stub
		channelMap.put(MessageType.BuildingIsBurningMessage, rankChannel[1]);
		channelMap.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
		channelMap.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
		//channelMap.put(MessageType.GasStationIsBurningMessage, rankChannel[1]);//niu
		//channelMap.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
		channelMap.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[1]);//niu
		
		channelMap.put(MessageType.BuildingIsExtinguishedMessage,
				rankChannel[1]);
		channelMap.put(MessageType.BuildingIsExploredMessage, rankChannel[4]);
		channelMap.put(MessageType.AssignFiredBuildingMessage, rankChannel[4]);
		channelMap.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[4]);
	//	channelMap.put(MessageType.BuildingMaybeMessage, rankChannel[4]);       //wan

	}

	private void setATChannelMap_2(int[] rankChannel)// 0&&3
	{
		// TODO Auto-generated method stub
		channelMap.put(MessageType.CivilianMaybeMessage, rankChannel[0]);
		channelMap.put(MessageType.CivilianInformationMessage, rankChannel[0]);
		channelMap.put(MessageType.AgentIsBuriedMessage, rankChannel[3]);
		channelMap
				.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[0]);
		channelMap.put(MessageType.BuildingIsExploreMessage_at, rankChannel[3]);  //wan
//		channelMap.put(MessageType.BuildingMaybeMessage_pf, rankChannel[3]);   //wan

	}

	private void setPFChannelMap_3(int[] rankChannel)// 2&&5&&8
	{
		// TODO Auto-generated method stub
		channelMap.put(MessageType.AssignImportantBuildingsMessage, rankChannel[5]);
		channelMap.put(MessageType.AssignRefugeMessage, rankChannel[2]);
		channelMap.put(MessageType.ClearPathIsNeededMessage, rankChannel[2]);
		channelMap.put(MessageType.AgentIsStuckMessage, rankChannel[2]);
		channelMap.put(MessageType.AssignRefugeMessage, rankChannel[2]);
		channelMap.put(MessageType.RoadIsClearedMessage, rankChannel[8]);
		channelMap.put(MessageType.InformTeamMessage, rankChannel[5]);
		channelMap.put(MessageType.BuildingIsExploreMessage_pf, rankChannel[5]);
		//channelMap.put(MessageType.BuildingMaybeMessage_pf, rankChannel[2]);    //wan
	}

	private void setFBChannelMap_3(int[] rankChannel)// 1&&4&&7
	{
		// TODO Auto-generated method stub
		channelMap.put(MessageType.BuildingIsBurningMessage, rankChannel[1]);
		channelMap.put(MessageType.ImportantBuildingsMessage, rankChannel[1]);//niu
		channelMap.put(MessageType.AssignImportantBuildingsMessage, rankChannel[1]);//niu
		//channelMap.put(MessageType.GasStationIsBurningMessage, rankChannel[1]);
		//channelMap.put(MessageType.GasStationIsExtinguishedMessage, rankChannel[1]);//niu
		channelMap.put(MessageType.BuildingCloseToGasStationIsBurningMessage, rankChannel[1]);//niu
		
		channelMap.put(MessageType.BuildingIsExtinguishedMessage,
				rankChannel[1]);
		channelMap.put(MessageType.BuildingIsExploredMessage, rankChannel[4]);
		channelMap.put(MessageType.AssignFiredBuildingMessage, rankChannel[7]);
		channelMap.put(MessageType.BuildingIsExploreMessage_fb, rankChannel[4]);
	//	channelMap.put(MessageType.BuildingMaybeMessage, rankChannel[7]);    //wan
 
	}

	private void setATChannelMap_3(int[] rankChannel)// 0&&3&&6
	{
		// TODO Auto-generated method stub
		channelMap.put(MessageType.CivilianInformationMessage, rankChannel[0]);
		channelMap.put(MessageType.AgentIsBuriedMessage, rankChannel[0]);
		channelMap
				.put(MessageType.CivilianIsSavedOrDeadMessage, rankChannel[3]);
	//	channelMap.put(MessageType.BuildingMaybeMessage_at, rankChannel[6]);    //wan
		channelMap.put(MessageType.BuildingIsExploreMessage_at, rankChannel[6]);   //wan
	//	channelMap.put(MessageType.CivilianMaybeMessage, rankChannel[0]);    //wan

	}

	public int getChannel(MessageType messageType) {
		Integer no;

		no = channelMap.get(messageType);
		// System.out.println(no);
		if (no == null) {

			// System.out.println(messageType);
			return 1;
		} else {
			return no;
		}
	}

	public int getReapeat() {
		return Reapeat;
	}

	public void setReapeat(int reapeat) {
		Reapeat = reapeat;
	}

}
