package seu.object.Building;
import java.util.ArrayList;

import rescuecore2.standard.entities.GasStation;
import rescuecore2.standard.entities.StandardEntityConstants.Fieryness;

public class GasStationInfo {
	private int lastUpdateTime;
	private final GasStation gasStation_;
	// 得到当前房屋
	public GasStation getGasStation() {
		return gasStation_;
	}
	private static final int MIN_HEATING_VALUE = 0;
	int latestTemperature_;
	int temperatureUpdateTime_;
	public GasStationInfo (GasStation gasStation) {
		gasStation_ = gasStation;
		if (gasStation.isTemperatureDefined()) {
			latestTemperature_ = gasStation.getTemperature();
		} else {
			latestTemperature_ = 0;
		}
		lastUpdateTime = 0;
	}
	public int getLatestTemperature() {
		return latestTemperature_;
	}

	public void setLatestTemperature(int latestTemperature) {
		this.latestTemperature_ = latestTemperature;
	}

	public boolean isHeating() {
		if (gasStation_.isFierynessDefined()
				&& gasStation_.getFierynessEnum() != Fieryness.BURNT_OUT) {
			int d;
			boolean heating;

			d = getTemperatureDifference();
			heating = (d > MIN_HEATING_VALUE);
			return heating;
		} else {
			return false;
		}
	}

	public double getHeatingSpeed(int timeStep) {
		int timeDifference;

		timeDifference = timeStep - temperatureUpdateTime_;
		if (timeDifference > 0) {
			int temperatureDifference;
			double r;

			temperatureDifference = getCurrentTemperature()
					- latestTemperature_;
			r = (double) temperatureDifference / timeDifference;
			return r;
		} else {
			return 0;
		}
	}

	public int getCurrentTemperature() {
		if (gasStation_.isTemperatureDefined()) {
			return gasStation_.getTemperature();
		} else {
			return 0;
		}
	}

	public int getTemperatureDifference() {
		int temperature, difference;

		temperature = getCurrentTemperature();
		difference = temperature - latestTemperature_;
		return difference;
	}

	public void updateTemperature(int timeStep) {
		if (gasStation_.isTemperatureDefined()) {
			latestTemperature_ = gasStation_.getTemperature();
			temperatureUpdateTime_ = timeStep;
		}
	}

	public int getLastUpdateTime() {
		return lastUpdateTime;
	}

	public void setLastUpdateTime(int lastUpdateTime) {
		this.lastUpdateTime = lastUpdateTime;
	}
}
