package seu.object.Partition;

import java.awt.Point;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.Building;
import rescuecore2.standard.entities.FireBrigade;
import rescuecore2.standard.entities.Refuge;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.object.Direction;
import seu.object.Line;
import seu.object.SEUWorldModel;
import seu.utilities.Geometry.PositionLocate;

public class PartitionModel {                //分区模型

	public static final int DEFAULT_LINE_COUNT = 1;
	public static final int DEFAULT_REGION_COLUMN_COUNT = 6;
	public static final int DEFAULT_TEAM = 6;

	SEUWorldModel world;
	EntityInPartition[][] entityInPartition;
	List<PartitionMap> partitionGroups;                   //实际分区组
	//Map<PartitionMap, EntityID> partitionAssignements;      //智能体与对应分区键值对
	Map<EntityID, PartitionMap> partitionAssignements;            //yuan_r
	GroupingType groupingType;

	public PartitionModel(SEUWorldModel model, GroupingType type) {
		int teamSize;

		this.world = model;
		groupingType = type;
		teamSize = 36;                                  //yuan_r        
		this.entityInPartition = calculatePartitions(teamSize);           //36个分区，初始分区
		partitionAssignements = new HashMap<EntityID, PartitionMap>();    //yuan_r 与智能体数量相等，实际分区
	}

	/**
	 * 计算分区数量
	 * 
	 * @return Returns the list of regions found.
	 */

	public int getExpectedRegionCount(int teamSize) {           //有队伍大小计算分区数量
		int n;

		n = (int) Math.ceil(Math.sqrt(teamSize) + 1);

		return n;
	}

	public EntityInPartition[][] calculatePartitions(int teamSize) {  //由队伍大小计算初始分区数
		int n;
		EntityInPartition[][] regions;

		n = getExpectedRegionCount(teamSize);
		regions = calculateRegions(n, n);

		return regions;
	}

	//根据队伍大小和类型计算实际分区组
	public List<PartitionMap> calculatePartitionsMap(int teamSize,  
			GroupingType type) {
		List<PartitionMap> groups;
		int c, totalCost, averageCost;
		 int NumofPartitionMap;
		if(teamSize>getNumofRefuges()+1){           //yuan
		    NumofPartitionMap=(teamSize-getNumofRefuges())/2; //偶数情况 wkk
		}else if(teamSize>0){
			NumofPartitionMap=teamSize;
		}else{
			return null;
		}
		groups = new ArrayList<PartitionMap>();		
		if(groupingType==GroupingType.RoadCount){   //警察
			for (int i = 0; i < teamSize; i++) {//36 partitions>teamSize
				   groups.add(new PartitionMap(i));  
			}
			c = 0;
			totalCost = 0;
			for (EntityInPartition[] regionRow : entityInPartition) {
				for (EntityInPartition region : regionRow) {
					int cost;
					cost = region.calculateAssignCost(type);
					totalCost += cost;
				}
			} 
			averageCost = totalCost / teamSize;
			totalCost = 0;
			int j=0;
			EntityInPartition region;
			for(int i=0;i<entityInPartition.length;i++){
				if(i%2==0){
					for(j=0;j<entityInPartition[0].length;j++){
						region=entityInPartition[i][j];
						PartitionMap group;
						group = groups.get(c);
						totalCost += region.getAssignCost();
						group.addRegion(region);
						//totalCost += region.getAssignCost();
						if (totalCost >= averageCost * (c + 1)) {
							if (c < teamSize - 1) {
								c++;
							}
						}
					}
				}else{
					for(j=entityInPartition[0].length-1;j>=0;j--){
						region=entityInPartition[i][j];
						PartitionMap group;
						group = groups.get(c);
						totalCost += region.getAssignCost();
						group.addRegion(region);
						//totalCost += region.getAssignCost();
						if (totalCost >= averageCost * (c + 1)) {
							if (c < teamSize - 1) {
								c++;
							}
						}
					}
				}
			}
		}else{                //医生，火警
		   for (int i = 0; i < NumofPartitionMap; i++) {//36 partitions>teamSize
			   groups.add(new PartitionMap(i));                //wkk
		   }
		   c = 0;
		   totalCost = 0;
		   for (EntityInPartition[] regionRow : entityInPartition) {
			 for (EntityInPartition region : regionRow) {
				int cost;
				cost = region.calculateAssignCost(type);
				totalCost += cost;
			 }
		   } 
		   averageCost = totalCost / NumofPartitionMap;
		   totalCost = 0;		
//		for (EntityInPartition[] regionRow : entityInPartition) {
//			for (EntityInPartition region : regionRow) {
//				PartitionMap group;
//				group = groups.get(c);
//				group.addRegion(region);
//				totalCost += region.getAssignCost();
//				if (totalCost >= averageCost * (c + 1)) {
//					if (c < teamSize/2 - 1) {
//						c++;
//					}
//				}
//			}
//		}			
		//重叠分区
		int j=0;
		EntityInPartition region;
		for(int i=0;i<entityInPartition.length;i++){
			if(i%2==0){
				for(j=0;j<entityInPartition[0].length;j++){
					region=entityInPartition[i][j];
					PartitionMap group;
					group = groups.get(c);
					totalCost += region.getAssignCost();
					group.addRegion(region);				
				    group.addNumOfRefuge(region.getNumOfRefuge());                //wkk				
					//totalCost += region.getAssignCost();
					if (totalCost >= averageCost * (c + 1)) {
						if (c < NumofPartitionMap - 1) {    //wkk
							c++;
						}
					}
				}
			}else{
				for(j=entityInPartition[0].length-1;j>=0;j--){
					region=entityInPartition[i][j];
					PartitionMap group;
					group = groups.get(c);
					totalCost += region.getAssignCost();
					group.addRegion(region);					
					group.addNumOfRefuge(region.getNumOfRefuge());   					
					//totalCost += region.getAssignCost();
					if (totalCost >= averageCost * (c + 1)) {
						if (c < NumofPartitionMap - 1) {   //wkk
							c++;
						}
					}
				}
			}
		}
	  }
	  //System.out.println( "we calculate partitions is size :" +groups.size());
	  return groups;
	}
//		totalCost=0;
//		boolean first=true;
//		int d=0;
//		
//		
//		for(int i=0;i<7;i++){
//			if(i%2==0){
//				for(j=0;j<7;j++){
//					region=entityInPartition[i][j];
//					PartitionMap group;
//					group = groups.get(c);
//					totalCost += region.getAssignCost();
//					if(totalCost<=averageCost/2) continue;
//					if(first) {
//						totalCost= region.getAssignCost();
//						first=false;
//					}
//					group.addRegion(region);
//					totalCost += region.getAssignCost();
//					if (totalCost >= averageCost * (d + 1)) {
//						if (c < teamSize/2 - 1) {
//							c++;
//							d++;
//						}
//					}
//				}
//			}
//			else{
//				for(j=6;j>=0;j--){
//					region=entityInPartition[i][j];
//					PartitionMap group;
//					group = groups.get(c);
//					totalCost += region.getAssignCost();
//					if(totalCost<=averageCost/2) continue;
//					if(first) {
//						totalCost= region.getAssignCost();
//						first=false;
//					}
//					group.addRegion(region);
//					totalCost += region.getAssignCost();
//					if (totalCost >= averageCost * (d + 1)) {
//						if (c < teamSize/2 - 1) {
//							c++;
//							d++;
//						}
//					}
//				}
//			}
//		}
		
		
		
		
		
		
		
		
      /* for (EntityInPartition[] regionRow : entityInPartition) {
			for (EntityInPartition region : regionRow) {
				PartitionMap group;
				group = groups.get(c);
				totalCost += region.getAssignCost();
				if(totalCost<=averageCost/2) continue;
				if(first) {
					totalCost= region.getAssignCost();
					first=false;
				}
				group.addRegion(region);
				totalCost += region.getAssignCost();
				if (totalCost >= averageCost * (d + 1)) {
					if (c < teamSize/2 - 1) {
						c++;
						d++;
					}
				}
			}
		}*/
		
	
	public void ReportMyCalculation(){
		
	}

	// 将World Model内实体对应至区域内
	public EntityInPartition[][] calculateRegions(int row, int column) {
		EntityInPartition[][] regions;
		Collection<StandardEntity> entities;
		int n;

		regions = new EntityInPartition[row][column];
		n = 0;
		for (int r = 0; r < row; r++) {
			for (int c = 0; c < column; c++) {
				EntityInPartition region;

				region = new EntityInPartition(n);
				region.setRow(r);
				region.setColumn(c);
				regions[r][c] = region;
				n++;
			}
		}
		entities = world
				.getEntitiesOfType(StandardEntityURN.BUILDING,
						StandardEntityURN.ROAD, StandardEntityURN.REFUGE,
						StandardEntityURN.AMBULANCE_CENTRE,
						StandardEntityURN.POLICE_OFFICE,
						StandardEntityURN.FIRE_STATION,
						StandardEntityURN.GAS_STATION);
		for (StandardEntity entity : entities) {
			Point point;
			int c, r;

			point = PositionLocate.getPosition(entity, world);
			c = getColumn((int) world.getBounds().getMinX(), (int) world
					.getBounds().getWidth(), column, point.x);
			r = getColumn((int) world.getBounds().getMinY(), (int) world
					.getBounds().getHeight(), row, point.y);
			if (r >= 0 && c >= 0 && r < regions.length && c < regions[r].length) {
				regions[r][c].add(entity);
			}
		}
		entities.clear();                                    //wkk
		entities = world
				.getEntitiesOfType(StandardEntityURN.REFUGE);
		for (StandardEntity entity : entities) {
			Point point;
			int c, r;

			point = PositionLocate.getPosition(entity, world);
			c = getColumn((int) world.getBounds().getMinX(), (int) world
					.getBounds().getWidth(), column, point.x);
			r = getColumn((int) world.getBounds().getMinY(), (int) world
					.getBounds().getHeight(), row, point.y);
			if (r >= 0 && c >= 0 && r < regions.length && c < regions[r].length) {
				//regions[r][c].add(entity);
				regions[r][c].addRefuge();     //wkk
			}
		}
		return regions;
	}

	protected int getColumn(int left, int width, int columnCount, int value) {
		return (value - left) * columnCount / width;
	}

	/**
	 * Retunrs the list of regions
	 * 
	 * @return
	 */

	public EntityInPartition[][] getRegions() {
		return entityInPartition;
	}

	/**
	 * Sets the regions
	 * 
	 * @param regions
	 */

	public void setRegions(EntityInPartition[][] regions) {
		this.entityInPartition = regions;
	}

	/**
	 * This function is used to get diagonal lines of the map.
	 * 
	 * @return the list of two main diagonal lines of the map.
	 */
	protected List<Line> getDiagonalLines() {            //yuan_nu 得到对角线
		Rectangle2D bounds = world.getBounds();
		List<Line> lines = new ArrayList<Line>();
		lines.add(new Line(bounds.getMinX(), bounds.getMinY(),
				bounds.getMaxX(), bounds.getMaxY()));
		lines.add(new Line(bounds.getMinX(), bounds.getMaxY(),
				bounds.getMaxX(), bounds.getMinY()));
		return lines;
	}

	protected List<Line> getGridLines(int n) {             //yuan_nu 得到网格线
		Rectangle2D bounds = world.getBounds();
		List<Line> lines = new ArrayList<Line>();
		double h = bounds.getHeight();
		double w = bounds.getWidth();
		double dx = w / (n + 1);
		double dy = h / (n + 1);
		for (int i = 0; i < n; i++) {
			lines.add(new Line(bounds.getMinX(), bounds.getMinY() + (i + 1)
					* dy, bounds.getMaxX(), bounds.getMinY() + (i + 1) * dy));
			lines.add(new Line(bounds.getMinX() + (i + 1) * dx, bounds
					.getMinY(), bounds.getMinX() + (i + 1) * dx, bounds
					.getMaxY()));
		}
		return lines;
	}

	/**
	 * 
	 * @param <T>
	 * @param entities
	 * @param lines
	 * @return
	 */

	protected <T extends StandardEntity> List<String> getLineStrings(        //yuan_nu 
			List<T> entities, List<Line> lines) {
		List<String> strings = new ArrayList<String>();
		for (StandardEntity entity : entities) {
			strings.add(getLineString(entity, lines));
		}
		return strings;
	}

	/**
	 * 
	 * @param entity
	 * @param lines
	 * @return
	 */

	protected String getLineString(StandardEntity entity, List<Line> lines) {   //yuan_nu
		String s = "";
		int x, y;
		if (entity instanceof Area) {
			Area building = (Area) entity;
			x = building.getX();
			y = building.getY();
		} else {
			return "";
		}
		for (Line line : lines) {
			if (line.isOnLeft(x, y)) {
				s += Direction.West.toChar();
			} else {
				s += Direction.East.toChar();
			}
		}
		return s;
	}

	/**
	 * 
	 * @return
	 */

	protected int getTotalBuildingArea() {                   //yuan_nu
		List<Building> buildings = world.getEntitiesOfType(Building.class,
				StandardEntityURN.BUILDING);
		int totalArea = 0;
		for (Building building : buildings) {
			totalArea += building.getTotalArea();
		}
		return totalArea;
	}

	/**
	 * 
	 * @return
	 */

	protected List<Building> getBuildings() {         //yuan_nu
		return world.getEntitiesOfType(Building.class,
				StandardEntityURN.BUILDING);
	}

	public EntityInPartition getRegion(StandardEntity entity) {       

		for (EntityInPartition[] regionRow : entityInPartition) {
			for (EntityInPartition region : regionRow) {
				if (region.contains(entity)) {
					return region;
				}
			}
		}
		return null;
	}

	public PartitionMap getRegionGroup(StandardEntity entity) {
		EntityInPartition region;
		PartitionMap group;

		region = getRegion(entity);
		if (region == null) {
			return null;
		}
		group = region.getRegionGroup();
		return group;
	}

	/**
	 * 根据实体获得该实体在第几个分区
	 * 
	 * @param entity
	 *            Given StandardEntity
	 * @return Returns corresponding region number.
	 */

	public int getRegionNo(StandardEntity entity) {
		EntityInPartition region = getRegion(entity);
		if (region == null) {
			return -1;
		} else {
			return region.getNo();
		}
	}

	public EntityInPartition getSmallest(List<EntityInPartition> regions,  //yuan_nu 得到前limit个分区中最小大小的分区
			int limit) {
		List<EntityInPartition> temp = new ArrayList<EntityInPartition>();
		for (int i = 0; i < limit; i++) {
			if (i < regions.size()) {
				temp.add(regions.get(i));
			}
		}
		Comparator<EntityInPartition> comparator = Collections
				.reverseOrder(new RegionSizeComparator());
		Collections.sort(temp, comparator);
		return temp.get(0);
	}

	public void assignPartitionsToTeam() {
		assignPartitionsToEntities(world.getTeam());
	}

	public void assignPartitionsToTeam(List<EntityID> team) {
		assignPartitionsToEntities(team);
	}
	//指派分区到实体组
	public void assignPartitionsToEntities(List<EntityID> entities) {  //yuan 
		int i = 0;
		int NumofPartitionMap;
		if(entities.size()>getNumofRefuges()+1){    
			NumofPartitionMap=(entities.size()-getNumofRefuges())/2;
		}else if(entities.size()>0){
			NumofPartitionMap=entities.size();
		}else{
			return;
		}
		List<PartitionMap> partitionHasRefuge = new ArrayList<PartitionMap>(); 	
		if (partitionGroups == null) {
			partitionGroups = calculatePartitionsMap(entities.size(),groupingType);
		}
		if(groupingType==GroupingType.RoadCount){   //警察
			for (PartitionMap group : partitionGroups) {              					
				if (i < entities.size()) {
					EntityID id = entities.get(i % entities.size());
					partitionAssignements.put(id,group);           
					i++;
				}
			 }
		}else{                //医生，火警           
			if(entities.size()>getNumofRefuges()+1){    
				for (PartitionMap group : partitionGroups){                //wkk
					if(group.getNumOfRefuge()!=0){
						partitionHasRefuge.add(group);
					}
				}
				//System.out.println("partitionHasRefuge nums is"+partitionHasRefuge.size());			
				for (PartitionMap group : partitionGroups) {              										
				    if (i < NumofPartitionMap*2) {          //wkk
					   EntityID id1 = entities.get(i % entities.size());
					   EntityID id2 = entities.get(i+1 % entities.size());				    
					   partitionAssignements.put(id1,group);             
					   partitionAssignements.put(id2,group);
					   i=i+2;					
				    }					
			    }
                for (PartitionMap group : partitionHasRefuge){					
					for(int a=0;a<group.getNumOfRefuge();a++){
						if(i<entities.size()){
					        EntityID id1 = entities.get(i % entities.size());
					        partitionAssignements.put(id1,group);
					        i=i+1;
					    }
					    else continue;
					}					
			    }
			    if(i==entities.size()-1){
				   EntityID id1 = entities.get(i % entities.size());
				   partitionAssignements.put(id1,partitionHasRefuge.get(0));
			    }          
				
			}else{
				for (PartitionMap group : partitionGroups) {              										
					    if (i < NumofPartitionMap) {          //wkk
						   EntityID id1 = entities.get(i % entities.size());						  		    
						   partitionAssignements.put(id1,group);             						 
						   i++;					
					    }					
				    }
			}			
		}
	}
/*
	public void assignPartitionsToEntities(List<EntityID> entities) {//lb04211519
		int i = 0;
		if (partitionGroups == null) {
			partitionGroups = calculatePartitionsMap(entities.size(),
					groupingType);
		}
		
		// List< > line=new ArrayList< >();
		//List<List<StandardEntity>> assign = new ArrayList<List<StandardEntity>>();
		List<EntityID> EsNeedReassign=new ArrayList<EntityID >();
		 List<StandardEntity> assign = new ArrayList<StandardEntity>(36);
		 //EntityID ent=new EntityID(3);
		 StandardEntity nopentity=null;
		 for(i=0;i<36;i++)
			 assign.add(nopentity);
		//先确定每个在此分区中到智能体，分区中多的智能体以此为中心向周围分区膨胀lb
		//if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM){
		//try{	
		for(i=0;i<entities.size();i++) {	//首次确定每个在此分区中到智能体lb	
		StandardEntity a=world.getEntity(entities.get(i));
		int n=6;		 
		Point point;
		int c, r;
		point = PositionLocate.getPosition(a, world);
		c = getColumn((int) world.getBounds().getMinY(), (int) world //表示行! !
				.getBounds().getHeight(), n, point.y);
		r = getColumn((int) world.getBounds().getMinX(), (int) world //表示列！！
				.getBounds().getWidth(), n, point.x);
		
		
		if (r >= 0 && c >= 0 && r < entityInPartition.length && c < entityInPartition[r].length) {
			
			if(assign.get(entityInPartition[c][r].no)==null){
			assign.set(entityInPartition[c][r].no, a);
			
			}
			else EsNeedReassign.add(a.getID());
		}
		if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM)
			System.out.print (entities.get(i)+" c "+c+"r "+r+"  ");
		}
		 
		
		if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM){
			System.out.println ("first assign "+assign);
		//	System.out.println ("entityInPartition");
		//for(int j=0;j<entityInPartition.length;j++ )
		//	for(int k=0;k<entityInPartition[j].length;k++ )
		//	System.out.print(entityInPartition[j][k].no+" Row"+entityInPartition[j][k].getRow()+" Column"+entityInPartition[j][k].getColumn() );
		//	//System.out.println(" ");
		}
		
	 //}catch(Exception e){System.err.print ("bbbbbbbbbbbbb "+e.getMessage());}
		//if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM)
		//	System.out.println ("EsNeedReassign "+EsNeedReassign);
		//每个分区中多的智能体以此为中心向周围分区膨胀,此时不计代价，这样可以尽量增加智能体对地图信息到获取lb		
			for(i=0;i<EsNeedReassign.size();i++){
			StandardEntity a=world.getEntity( EsNeedReassign.get(i));
			int n=6;		 
			Point point;
			int c, r;
			point = PositionLocate.getPosition(a, world);
			r = getColumn((int) world.getBounds().getMinX(), (int) world 
					.getBounds().getWidth(), n, point.x);
			c = getColumn((int) world.getBounds().getMinY(), (int) world 
					.getBounds().getHeight(), n, point.y);		
			//try{		
			int k=1;
				
				while(k<=5) {//////////这种分配有问题，不是按照格子数的no
				
				if(((c+k)<=5)&&assign.get(entityInPartition[c+k][r].no)==null){ 
					assign.set(entityInPartition[c+k][r].no, a); break;}
				
				else if(((r+k)<=5)&&assign.get(entityInPartition[c][r+k].no)==null){
					assign.set(entityInPartition[c][r+k].no, a); break;}
				 
				else if(((c+k)<=5&&(r+k)<=5)&&assign.get(entityInPartition[c+k][r+k].no)==null) {
					assign.set(entityInPartition[c+k][r+k].no, a); break;}
				 
				else if(((c-k)>=0&&(r-k)>=0)&&assign.get(entityInPartition[c-k][r-k].no)==null) {
					assign.set(entityInPartition[c-k][r-k].no, a); break;}
				 
				else if(((r-k)>=0)&&assign.get(entityInPartition[c][r-k].no)==null) {
				    assign.set(entityInPartition[c][r-k].no, a); break;}
				 
				else if(((c-k)>=0)&&assign.get(entityInPartition[c-k][r].no)==null){
					assign.set(entityInPartition[c-k][r].no, a); break;}
				 
				else if(((c+k)<=5&&(r-k)>=0)&&assign.get(entityInPartition[c+k][r-k].no)==null) {
					assign.set(entityInPartition[c+k][r-k].no, a); break;}
				 
				else if(((c-k)>=0&&(r+k)<=5)&&assign.get(entityInPartition[c-k][r+k].no)==null) {
					assign.set(entityInPartition[c-k][r+k].no, a); break;}
		 
				else k++;
			}
				
		//	}catch(Exception e){System.err.println ("ffffff "+e.getMessage());}
		}
			if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM)
			System.out.println ("second assign "+assign);
			
		//给group做分配，然后将每个group中的多余的智能体分给没有智能体的group
			//group.partition.get(i).getNo()是group中小分区原来的no
		//try{
			//第一次遍历先确定每个有智能体在的group，将没有智能体的group和有多于一个智能体的group保存出来
			int num=0;
			int count=0;
			//List<PartitionMap> tempNoEGroups=new ArrayList<PartitionMap>();
			List<Integer>tempNoEGroups=new ArrayList<Integer>();
			 List<StandardEntity> tempEntityID = new ArrayList<StandardEntity>();
		for (PartitionMap group : partitionGroups) {
			num=0;
			 
			i=0;
			while(i<group.size){//将有对应的智能体的group先确保对应上一个，注意这里并非每个group的第一个Partition的序号就能在assign列表中对应位置有智能体对应的，所以需要循环，如果循环后还是没有，那么
				if((assign.get(group.partition.get(i).getNo())!=null)){
				partitionAssignements.put(group, assign.get(group.partition.get(i).getNo()).getID());
				
				break;
				}
				i++;//if(i==group.size)tempNoEGroups.add(count);
			}
			
			for(i=0;i<group.partition.size();i++){//统计每个group中智能体的数量
					if(assign.get(group.partition.get(i).getNo())!=null) {
						num++;
						if(num>=2){
							tempEntityID.add(assign.get(group.partition.get(i).getNo()));//将一个group中多的智能体放到临时列表中存储
						}
						}
				}
			if(num==0){
				tempNoEGroups.add((Integer)count);//将没有智能体的group所在列表位置的序号放到临时列表中存储
			}
			count++;
		 }
		//if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM)
		//	System.out.println ("first partitionAssignements "+partitionAssignements);
	//	if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM)
	//	System.out.print (" count of groups "+count);
	//	System.out.print (" tempNoEGroups size "+tempNoEGroups.size()+" "+" tempEntityID size "+tempEntityID.size());//此处两值理论上是相等的 ，已论证
		
		if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM){
			System.out.println ("bbbbbbbbbpartitionAssignements "+partitionAssignements);
			System.out.println ("bbbbbbbbbpartitionGroups "+partitionGroups);
		
		//System.out.print (" tempNoEGroups size "+tempNoEGroups); 
		}
		for( i=0;i<tempNoEGroups.size();i++){
		partitionAssignements.put(partitionGroups.get(tempNoEGroups.get(i)), tempEntityID.get(i).getID());//将临时列表中的东西对应地再分配,不管对应关系如何
		//if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM)
			//System.out.print (" par  "+partitionGroups.get(tempNoEGroups.get(i)));
		}
		if(world.getEntity(entities.get(0)).getStandardURN()==StandardEntityURN.AMBULANCE_TEAM)
		System.out.println (" partitionAssignements "+partitionAssignements);
		//System.out.println ("final assign"+assign);
		//System.out.println ("partitionAssignements "+partitionAssignements);
			/*if(group.partition.size()>=2)				
				for(i=1;i<group.partition.size();i++){
					//for (PartitionMap g : partitionGroups) 
					if(partitionAssignements.get(g)==null)
						if(assign.get(group.partition.get(0).getNo())!=null)
						partitionAssignements.put(g,assign.get(g.partition.get(i).getNo()).getID());		
					
					}
					*
		
		
		
	//}catch(Exception e){System.err.print ("group "+e.getMessage());}
	//}
	
		
		 
	}*/

	/*public PartitionMap getAssignedPartitions(EntityID agentId) {
		for (PartitionMap regionGroup : partitionAssignements.keySet()) {
			
			
			EntityID id = partitionAssignements.get(regionGroup);
			
			if (id.equals(agentId)) {
				return regionGroup;
			}
		}
		return null;
	}*/
	public PartitionMap getAssignedPartitions(EntityID agentId) {         //yuan_r
		if(agentId!=null){
			return partitionAssignements.get(agentId);
		}
		return null;
	}

	public List<EntityInPartition> getNeighbors(EntityInPartition region,            //得到指定level范围的分区
			int level) {
		int column, row, minC, minR, maxC, maxR;
		List<EntityInPartition> regions;

		column = region.getColumn();
		row = region.getRow();
		minC = Math.max(0, column - level);
		minR = Math.max(0, row - level);
		maxC = Math.min(entityInPartition[0].length - 1, column + level);
		maxR = Math.min(entityInPartition.length - 1, row + level);
		regions = new ArrayList<EntityInPartition>();
		for (int r = minR; r <= maxR; r++) {
			for (int c = minC; c <= maxC; c++) {
				if (r != row || c != column) {
					regions.add(entityInPartition[r][c]);
				}
			}
		}
		return regions;
	}

	public Set<PartitionMap> getNeighbors(PartitionMap group, int level) {
		Set<PartitionMap> groups;

		groups = new HashSet<PartitionMap>();
		for (EntityInPartition region : group.getPartitions()) {
			List<EntityInPartition> neighborRegions;

			neighborRegions = getNeighbors(region, level);
			for (EntityInPartition neighborRegion : neighborRegions) {
				PartitionMap neighborGroup;

				neighborGroup = neighborRegion.getRegionGroup();
				if (!neighborGroup.equals(group)) {
					groups.add(neighborGroup);
				}
			}
		}
		return groups;
	}

	public List<PartitionMap> getPartitionGroups() {
		return partitionGroups;
	}

	public Map<EntityID, PartitionMap> getPartitionAssignements() {       //yuan_r
		return partitionAssignements;
	}
	public int getNumofRefuges(){                                            //wkk
		List<Refuge> result = world.getEntitiesOfType(Refuge.class,
				StandardEntityURN.REFUGE);
		return result.size();
	}

}
