package seu.utilities;

import java.awt.Point;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Random;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardEntityURN;
import rescuecore2.worldmodel.EntityID;
import seu.agent.SEUAgent;
import seu.object.SEUWorldModel;
import seu.object.Partition.EntityInPartition;
import seu.object.Partition.PartitionModel;
import seu.object.Road.RoadModel;
import seu.utilities.Search.Cost.PathType;

public class ACAMainRoad {
	private final SEUWorldModel world;
	private ArrayList<StandardEntity> roads;
	private HashMap<Integer, Double> pheromone;
	private HashMap<Integer, Boolean> goneRoad;
	private ArrayList<Integer> roadID;
	private final PartitionModel pModel;
	private ArrayList<Path> pathArray;
	private final SEUAgent agent;
	private double[][] dis;
	private int n;
	private ArrayList<ACAEdge> edges;
	private double yita;
	private double Q;
	private final int maxRun = 10;
	private final double toleranceDis = 20000;
	private ArrayList<Integer> criticalRoad;
	private final int numOfCrticalRoad = 11;
	private final ArrayList<Path> criticalPath;

	public ACAMainRoad(SEUAgent<? extends StandardEntity> seuAgent) {
		this.agent = seuAgent;
		this.world = seuAgent.getModel();
		pModel = world.getPartitionModel();
		criticalPath = new ArrayList<Path>();
		createCriticalPath();
	}

	public void createCriticalPath() {
		// 初始化地图的Hash值
		boolean loaded = false;
		long hash = agent.getModel().getHash();
		// System.out.println("node cache" + hash);
		String str = "nodecache/";
		criticalPath.clear();
		String fname = str + hash + ".cr";
		try {
			File f = new File(fname);

			BufferedReader br = new BufferedReader(new FileReader(f));

			String nl;

			while (null != (nl = br.readLine())) {
				int quantity = Integer.parseInt(nl);
				ArrayList<EntityID> entities = new ArrayList<EntityID>();
				for (int i = 0; i < quantity; i++) {
					int id = Integer.parseInt(br.readLine());
					entities.add(new EntityID(id));
				}
				Path path = new Path(entities);
				criticalPath.add(path);
			}

			loaded = true;

		} catch (Exception e) {

			// 文件不一致或者不存在都需要调用原方法进行计算
			// e.printStackTrace();
		}

		try {
			if (!loaded) {
				init();
				for (int i = 0; i < maxRun; i++) {
					// System.err.println(i);
					ACARun();
					updatePheromone();
				}
				selectCriticalPath();

				File f = new File(fname);
				f.createNewFile();
				BufferedWriter bw = new BufferedWriter(new FileWriter(f));
				for (Path p : criticalPath) {
					bw.write(2 + "\n");
					// for (EntityID id : p.getEntities()) {
					// bw.write(id.getValue() + "\n");
					// }
					bw.write(p.getEntities().get(0).getValue() + "\n");
					bw.write(p.getEntities().get(p.getEntities().size() - 1)
							.getValue()
							+ "\n");
				}
				bw.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void init() {
		// TODO Auto-generated method stub
		pheromone = new HashMap<Integer, Double>();
		goneRoad = new HashMap<Integer, Boolean>();
		roads = new ArrayList<StandardEntity>();
		roads.addAll(world.getEntitiesOfType(StandardEntityURN.ROAD));
		roadID = new ArrayList<Integer>();
		pathArray = new ArrayList<Path>();
		edges = new ArrayList<ACAEdge>();
		criticalRoad = new ArrayList<Integer>();
		IDComparator cmp = new IDComparator();
		RoadModel roadModel = world.getRoadModel();
		PartitionModel pModel = world.getPartitionModel();
		Collections.sort(roads, cmp);
		for (int i = 0; i < roads.size(); i++) {
			pheromone.put(roads.get(i).getID().getValue(), 0.0);
			goneRoad.put(roads.get(i).getID().getValue(), false);
			roadID.add(roads.get(i).getID().getValue());
		}
		yita = 0.5;
		Q = 1;
	}

	public void ACARun() {
		// TODO Auto-generated method stub
		ArrayList<Integer> selectedRoad = selectRoad();
		edges.clear();
		// System.out.println(" " + selectedRoad + " size " +
		// selectedRoad.size());
		Path path;
		Area start, end;
		n = selectedRoad.size();
		dis = new double[n][n];

		for (int i = 0; i < selectedRoad.size() - 1; i++) {
			dis[i][i] = 0;
			start = (Area) world.getEntity(new EntityID(selectedRoad.get(i)));
			for (int j = i + 1; j < selectedRoad.size(); j++) {
				end = (Area) world.getEntity(new EntityID(selectedRoad.get(j)));

				path = new Path();
				path = agent.getSearch().getPath(start,
						new Point(start.getX(), start.getY()), end,
						PathType.Shortest);
				pathArray.add(path);
				// System.out.println(start + " " + end + path);
				if (path == null) {
					dis[i][j] = Integer.MAX_VALUE;
					dis[j][i] = dis[i][j];
					ACAEdge edge = new ACAEdge(i, j, path);
					edges.add(edge);
				} else {
					dis[i][j] = path.getCost();
					dis[j][i] = dis[i][j];
					ACAEdge edge = new ACAEdge(i, j, path);
					edges.add(edge);
				}
			}
		}
		// System.out.println("edges" + edges);
		ArrayList<ACAEdge> targetEdge = Krustal(edges);
		// System.out.println("targetEdge" + targetEdge.size() + " \n"
		// + targetEdge);
		for (int id : pheromone.keySet())
			pheromone.put(id, pheromone.get(id) * yita);
		for (int i = 0; i < targetEdge.size(); i++) {
			ACAEdge e = targetEdge.get(i);
			HashMap<Integer, Double> p = new HashMap<Integer, Double>();
			p.putAll(pheromone);
			for (int id : pheromone.keySet()) {
				// pheromone.put(id, pheromone.get(id) * yita);
				p.put(id, 0.0);
			}
			for (EntityID r : e.getPath().getEntities()) {
				if (p.containsKey(r.getValue())) {
					p.put(r.getValue(), p.get(r.getValue()) + 1);
				}
			}
			for (int id : pheromone.keySet()) {
				pheromone.put(id, pheromone.get(id) + p.get(id));
			}
		}
		// System.out.println("pheromone" + pheromone);

	}

	ArrayList<Integer> selectRoad() {
		ArrayList<Integer> targetRoad = new ArrayList<Integer>();
		Random random = new Random();

		for (EntityInPartition[] eInPartition : pModel.getRegions()) {

			for (int i = 0; i < eInPartition.length; i++) {
				// System.out.println("Regions " + eInPartition[i]);
				if (eInPartition[i].getRoads() == null
						|| eInPartition[i].getRoads().size() <= 0) {
					continue;
				}
				int t = random.nextInt(eInPartition[i].getRoads().size());
				// System.out.println(t);
				int id = eInPartition[i].getRoads().get(t).getID().getValue();
				targetRoad.add(id);
			}
		}
		return targetRoad;
	}

	private ArrayList<ACAEdge> Krustal(ArrayList<ACAEdge> arrEdges) {
		int i, j, k;
		ArrayList<ACAEdge> targetEdge = new ArrayList<ACAEdge>();
		// HashSet<HashSet<Integer>> set = new HashSet<HashSet<Integer>>();
		ArrayList<ACAEdge> sortedEdge = new ArrayList<ACAEdge>();
		int[] f = new int[n];
		boolean[] flag = new boolean[arrEdges.size()];
		for (i = 0; i < f.length; i++) {
			f[i] = i;
			flag[i] = false;
		}
		for (i = 0; i < arrEdges.size() - 1; i++) {
			ACAEdge e1 = arrEdges.get(i);
			ACAEdge e2 = null;
			k = i;
			double min = e1.getCost();
			for (j = 0; j < arrEdges.size(); j++) {
				if (flag[j] == false) {
					e2 = arrEdges.get(j);
					if (e2.getCost() < min) {
						min = e2.getCost();
						k = j;
						// System.out.println(e1.toString() + e2.toString());
					}
				}
			}
			sortedEdge.add(arrEdges.get(k));
			flag[k] = true;
			// HashSet<Integer> s=new HashSet<Integer>();
			// s.add(i);
			// set.add(s);
		}
		j = 0;
		// System.out.println("Sorted edges" + sortedEdge);
		for (i = 0; i < sortedEdge.size(); i++) {
			int s1 = sortedEdge.get(i).getStart();
			int s2 = sortedEdge.get(i).getEnd();
			// System.out.println("s1  " + s1 + " " + "s2   " + s2);
			if (f[s1] != f[s2]) {
				j++;
				targetEdge.add(sortedEdge.get(i));
				int x1 = f[s1];
				int x2 = f[s2];
				if (f[s1] > f[s2]) {
					x1 = f[s2];
					x2 = f[s1];
				}
				for (k = 0; k < f.length; k++)
					if (f[k] == x2) {
						f[k] = x1;
					}
				// System.out.println("####");
			}
		}
		// for (i = 0; i < flag.length; i++)
		// System.out.print(flag[i] + " ");
		// System.out.print("\n");
		return targetEdge;
	}

	void selectCriticalPath() {
		selectCriticalRoad(numOfCrticalRoad);
		criticalPath.clear();
		Path path;
		ArrayList<ACAEdge> criticalEdge = new ArrayList<ACAEdge>();
		Area start, end;
		double[][] distance = new double[criticalRoad.size()][criticalRoad
				.size()];
		for (int i = 0; i < criticalRoad.size() - 1; i++) {
			distance[i][i] = 0;
			start = (Area) world.getEntity(new EntityID(criticalRoad.get(i)));
			for (int j = i + 1; j < criticalRoad.size(); j++) {
				end = (Area) world.getEntity(new EntityID(criticalRoad.get(j)));
				path = new Path();
				path = agent.getSearch().getPath(start,
						new Point(start.getX(), start.getY()), end,
						PathType.Shortest);
				pathArray.add(path);
				distance[i][j] = path.getCost();
				distance[j][i] = distance[i][j];
				ACAEdge edge = new ACAEdge(i, j, path);
				criticalEdge.add(edge);
			}
		}
		for (ACAEdge e : Krustal(criticalEdge)) {
			criticalPath.add(e.getPath());
		}
	}

	private void updatePheromone() {
		HashMap<Integer, Boolean> flag = new HashMap<Integer, Boolean>();
		for (int i : pheromone.keySet()) {
			flag.put(i, false);
		}
		HashMap<Integer, Double> partialPress = new HashMap<Integer, Double>();
		// System.out.println("Sorted pheromone");
		int num = 0;
		for (int i : pheromone.keySet()) {
			double maxP;
			maxP = -1;
			int k = -1;
			for (int j : pheromone.keySet())
				if (flag.get(j) == false && maxP < pheromone.get(j)) {
					maxP = pheromone.get(j);
					k = j;
				}
			if (k != -1 && maxP > 0.1) {
				num++;
				StandardEntity e = world.getEntity(new EntityID(k));
				Area r = (Area) e;
				flag.put(k, true);
				if (num <= 50) {
					partialPress.put(k, maxP);
				}
			}
		}
		for (int i : partialPress.keySet()) {
			StandardEntity e1 = world.getEntity(new EntityID(i));
			Area start = (Area) e1;
			for (int j : partialPress.keySet()) {
				if (i < j) {
					StandardEntity e2 = world.getEntity(new EntityID(j));
					Area end = (Area) e2;
					// double dis = world.getDistance(e1, e2);
					// if (dis <= toleranceDis) {
					// pheromone.put(j, 0.0);
					// }
					Path tempPath = agent.getSearch().getPath(start,
							new Point(start.getX(), start.getY()), end,
							PathType.Shortest);
					if (tempPath.size() < 13) {
						// System.err.println(i + " " + j);
						if (pheromone.get(i) < pheromone.get(j))
							pheromone.put(i, pheromone.get(i) * 0.2);
						else
							pheromone.put(j, pheromone.get(j) * 0.2);
					}
				}
			}
		}
		for (int i : pheromone.keySet()) {
			flag.put(i, false);
		}
		num = 0;
		for (int i : pheromone.keySet()) {
			double maxP;
			maxP = -1;
			int k = -1;
			for (int j : pheromone.keySet())
				if (flag.get(j) == false && maxP < pheromone.get(j)) {
					maxP = pheromone.get(j);
					k = j;
				}
			if (k != -1 && num <= numOfCrticalRoad) {
				num++;
				StandardEntity e = world.getEntity(new EntityID(k));
				Area r = (Area) e;
				// System.out.println(k + " " + maxP + " " + r.getX() + " "
				// + r.getY());
				flag.put(k, true);
			}
		}
	}

	void selectCriticalRoad(int numOfRoad) {
		criticalRoad.clear();
		HashMap<Integer, Boolean> flag = new HashMap<Integer, Boolean>();
		for (int i : pheromone.keySet()) {
			flag.put(i, false);
		}
		int num = 0;
		for (int i : pheromone.keySet()) {
			double maxP;
			maxP = -1;
			int k = -1;
			for (int j : pheromone.keySet())
				if (flag.get(j) == false && maxP < pheromone.get(j)) {
					maxP = pheromone.get(j);
					k = j;
				}
			if (k != -1 && num < numOfRoad) {
				num++;
				StandardEntity e = world.getEntity(new EntityID(k));
				Area r = (Area) e;
				flag.put(k, true);
				criticalRoad.add(k);
			} else if (num >= numOfRoad) {
				break;
			}
		}
	}

	ArrayList<Integer> getCriticalRoad() {
		return criticalRoad;
	}

	public ArrayList<Path> getCriticalPath() {
		return criticalPath;
	}

	class ACAEdge {
		private final int start;
		private final int end;
		private final Path path;
		private final double cost;

		public ACAEdge(int start, int end, Path path) {
			this.start = start;
			this.end = end;
			this.path = path;
			cost = path.getCost();
		}

		public double getCost() {
			return cost;
		}

		public int getStart() {
			return start;
		}

		public int getEnd() {
			return end;
		}

		public Path getPath() {
			return path;
		}

		@Override
		public String toString() {
			// TODO Auto-generated method stub
			return start + " " + end + " " + cost + "\n";
		}
	}
}
