package seu.utilities;

import java.util.Comparator;

import rescuecore2.misc.Pair;
import rescuecore2.standard.entities.Civilian;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.standard.entities.StandardWorldModel;
import seu.agent.BasicTask.SaveHuman;
import seu.object.SEUWorldModel;

public class CivDistanceComparator implements Comparator<SaveHuman> {

	private final StandardEntity reference;
	private final SEUWorldModel world;

	/**
	 * Create a DistanceSorter.
	 * 
	 * @param reference
	 *            The reference point to measure distances from.
	 * @param world
	 *            The world model.
	 */
	public CivDistanceComparator(StandardEntity reference,
			SEUWorldModel world) {
		this.reference = reference;
		this.world = world;
	}

	/**
	 * Compares the standard entites according to distance.
	 * 
	 * @param a
	 *            First StandardEntity to compare
	 * @param b
	 *            Second StandardEntity to compare
	 * @return The difference between distances.
	 */

	@Override
	public int compare(SaveHuman human1, SaveHuman human2) {
		if(!(human1.getHuman(world) instanceof Civilian) ){//先比类型 niu14.2.20 原来是standardworldmodel
			return -1;
		}else if(!(human1.getHuman(world) instanceof Civilian)){
			return 1;
		}else{
		int d1 = world.getDistance(reference, human1.getLocationOfHuman());
		int d2 = world.getDistance(reference, human2.getLocationOfHuman());
		if (d1 < 0)
			d1 = Integer.MAX_VALUE;
		if (d2 < 0)
			d2 = Integer.MAX_VALUE;
		return d1 - d2;
		}
	}

}
