/**
 * 将着火区域进行划分，着火房屋放入火区
 */
package seu.utilities;

import java.util.HashSet;

import rescuecore2.standard.entities.Building;
import seu.object.SEUWorldModel;
import seu.object.Building.BuildingInfo;

/**
 * @author knight
 * 
 */
public class FireSite {

	public final SEUWorldModel world;

	// 火区内所有着火房屋
	private final HashSet<Building> fires = new HashSet<Building>();

	// 即将加入火区的房屋
	private final HashSet<Building> dangerousFires = new HashSet<Building>();

	// 火区周围的房屋
	private final HashSet<Building> siteNighbors = new HashSet<Building>();

	// 火区的两个参数，世界模型和当前房屋
	public FireSite(BuildingInfo fire, SEUWorldModel world) {
		super();

		fires.add(fire.getBuilding());

		this.world = world;

		siteNighbors.addAll(fire.getNeighboursFar(world));

	}

	/**
	 * Checks whether fire can be added to the site
	 * 
	 * @return 1, if have, 0 if can, -1 if cannot
	 */
	public int addFire(BuildingInfo fire) {
		if (containsFire(fire.getBuilding())) {
			return 1;
		}
		if (!isSiteNeighbor(fire.getBuilding())) {
			return -1;
		}
		fires.add(fire.getBuilding());

		siteNighbors.remove(fire.getBuilding());

		for (Building b : fire.getNeighboursFar(world)) {
			if (!containsFire(b)) {
				siteNighbors.add(b);
			}
			// }
		}

		updateDangerousFires();

		return 0;
	}

	public void mergeWith(FireSite fs) {
		fires.addAll(fs.fires);

		siteNighbors.addAll(fs.siteNighbors);
		Object[] temp = siteNighbors.toArray();
		for (int i = 0; i < temp.length; ++i) {
			if (fires.contains(temp[i])) {
				siteNighbors.remove(temp[i]);
			}
		}
		dangerousFires.addAll(fs.dangerousFires);
	}

	// 更新危险的房屋
	private void updateDangerousFires() {
		// TODO 更新危险的房屋，这里首先得到所有的房屋

	}

	public boolean containsFire(Building fire) {
		return fires.contains(fire);
	}

	public boolean isSiteNeighbor(Building b) {
		return siteNighbors.contains(b);
	}

	public HashSet<Building> getSiteNighbors() {
		return siteNighbors;
	}

	public HashSet<Building> getFires() {
		return fires;
	}

	public String toString() {
		String s = "";
		for (Building b : fires)
			s = s + b;
		return s;
	}

	public boolean iscontain(Building e) {
		if (fires.contains(e))
			return false;
		for (Building b : fires) {
			if (world.getDistance(b.getID(), e.getID()) < 30000)
				return true;
		}
		return false;

	}

}
