package seu.utilities;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import rescuecore2.standard.entities.Human;
import rescuecore2.standard.entities.StandardEntity;
import rescuecore2.worldmodel.EntityID;
import seu.object.SEUWorldModel;

public class PositionHistory {
	// public static final Logger LOG = Logger.getLogger(PositionHistory.class);

	public static final int DISTANCE_EQUALITY_TRESHOLD = 1000;
	// 保存周期数
	public static final int DELETE_AFTER = 15;

	int latestX;
	int latestY;
	int currentX;
	int currentY;
	int time;

	Human agent;
	Path desiredPath;
	SEUWorldModel world;
	Path lastPath;
	StandardEntity desiredLocation;

	public StandardEntity getDesiredLocation() {
		return desiredLocation;
	}

	StandardEntity currentLocation;

	// Get Current Location
	public StandardEntity getCurrentLocation() {
		return currentLocation;
	}

	StandardEntity latestLocation;

	protected Map<StandardEntity, Integer> unreachableLocationTimes;
	private final Set<Integer> reachedArea;
	private final Map<Integer, Integer> reachedAreaTime;

	public PositionHistory() {
		this(null, null);
	}

	public PositionHistory(Human agent, SEUWorldModel world) {
		this.agent = agent;
		this.world = world;
		unreachableLocationTimes = new HashMap<StandardEntity, Integer>();
		reachedArea = new HashSet<Integer>();
		reachedAreaTime = new HashMap<Integer, Integer>();
	}

	public void process(int timestep) {
		time = timestep;
		if (agent == null || world == null) {
			return;
		}
		updateCurrentValues();
		if (isInDesiredLocation()) {

			desiredLocation = null;
		} else {
			if (isInSameLocation()) {
				// LOG.warn("Found unreachable location: " + desiredLocation_);
				unreachableLocationTimes.put(desiredLocation, timestep);
			}
		}
		updateLatestValues();
		removeOldLocations(timestep);
	}

	public boolean hasMoveFailed() {
		if (agent == null || world == null) {
			return false;
		}
		if (isInDesiredLocation()) {
			return false;
		} else {
			if (isInSameLocation()) {
				return true;
			}
		}
		return false;
	}

	protected void updateCurrentValues() {
		currentLocation = agent.getPosition(world);
		if (agent.isXDefined() && agent.isYDefined()) {
			currentX = agent.getX();
			currentY = agent.getY();
		}
	}

	protected void updateLatestValues() {
		if (latestLocation != null && currentLocation != null
				&& lastPath != null && !lastPath.isEmpty()) {
			if (lastPath.contains(currentLocation.getID())) {
				// System.out.println(lastPath
				// + "index"
				// + lastPath.getEntities().indexOf(
				// currentLocation.getID()));
				for (int i = 0; i < lastPath.getEntities().indexOf(
						currentLocation.getID()); i++) {
					reachedArea.add(lastPath.getEntities().get(i).getValue());
					reachedAreaTime.put(lastPath.getEntities().get(i)
							.getValue(), time);
					// System.out.println(time + "Add"
					// + lastPath.getEntities().get(i));
				}
				if (!unreachableLocationTimes.containsKey(currentLocation)) {
					reachedArea.add(currentLocation.getID().getValue());
					reachedAreaTime.put(currentLocation.getID().getValue(),
							time);
					// System.out.println(time + "Add" +
					// currentLocation.getID());
				}
			}
		}
		latestLocation = currentLocation;
		latestX = currentX;
		latestY = currentY;
	}

	// 判断是否移动
	public boolean isInSameLocation() {
		if (currentLocation.equals(latestLocation)) {
			int xDiff, yDiff, distance;

			xDiff = Math.abs(currentX - latestX);
			yDiff = Math.abs(currentY - latestY);
			distance = xDiff + yDiff;
			// LOG.debug("In same location with distance: " + distance);
			return distance < DISTANCE_EQUALITY_TRESHOLD;
		} else {
			return false;
		}
	}

	public boolean isInDesiredLocation() {
		if (desiredLocation == null) {
			return true;
		}
		return currentLocation.equals(desiredLocation);
	}

	public void setDesiredLocation(StandardEntity location) {
		desiredLocation = location;
	}

	public void setDesiredLocation(EntityID id) {
		StandardEntity entity;

		entity = world.getEntity(id);
		setDesiredLocation(entity);
	}

	public Set<StandardEntity> getUnreachableLocations() {
		Set<StandardEntity> locations;

		locations = unreachableLocationTimes.keySet();
		return locations;
	}

	public void addUnreachableLocations(
			Collection<? extends StandardEntity> locations, int time) {
		// LOG.warn("Found unreachable locations: " + locations);
		for (StandardEntity location : locations) {
			unreachableLocationTimes.put(location, time);
		}
	}

	@Override
	public String toString() {
		return "Position History - Unreachable: "
				+ unreachableLocationTimes.toString();
	}

	private void removeOldLocations(int time) {
		List<StandardEntity> list;

		list = new ArrayList<StandardEntity>();
		list.addAll(unreachableLocationTimes.keySet());
		for (StandardEntity entity : list) {
			int t;

			t = unreachableLocationTimes.get(entity);
			if (t + DELETE_AFTER < time) {
				unreachableLocationTimes.remove(entity);
			}
		}
	}

	public void setDesiredPath(Path path) {
		lastPath = desiredPath;
		desiredPath = path;
	}

	public Set<Integer> getReachedArea() {
		return reachedArea;
	}
}
