package seu.utilities;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;

import rescuecore2.standard.entities.Area;
import rescuecore2.standard.entities.StandardWorldModel;
import rescuecore2.worldmodel.EntityID;
import seu.utils.geo.GeoUtils;
import seu.utils.geo.IPositioning;

public final class RandomSelector {

	public static final double MULTI_SCALE = Math.sqrt(2);

	public static <E> Set<E> randomSelect(final Set<E> set, int selects,
			Random rand) {
		if (selects > set.size())
			throw new IllegalArgumentException();

		Set<E> ret = new HashSet<E>();
		ArrayList<E> list = new ArrayList<E>();
		for (E id : set)
			list.add(id);

		int end = list.size();
		for (int i = 0; i < selects; i++) {
			int pick = rand.nextInt(end);
			E id = list.get(pick);
			ret.add(id);
			list.set(pick, list.get(end - 1));
			end--;
		}
		return ret;
	}

	public static <E extends IPositioning> E multirangeSelect(
			final Set<E> data, int originX, int originY, Random rand,
			int initRange) {
		if (data.size() == 0)
			return null;
		double range = initRange;
		Set<E> candidate = new HashSet<E>();
		while (true) {
			for (E a : data) {
				if (GeoUtils.manhattanDistance(a, originX, originY) < range) {
					candidate.add(a);
				}
			}
			if (!candidate.isEmpty()) {
				return randomSelect(candidate, 1, rand).iterator().next();
			}
			range *= MULTI_SCALE;
		}
	}

	public static <E extends IPositioning> Set<E> multirangeSelect(
			final Set<E> data, int originX, int originY, int maxselects,
			Random rand, int initRange) {
		if (data.size() == 0)
			return null;
		double range = initRange;
		Set<E> candidate = new HashSet<E>();
		Set<E> ret = new HashSet<E>();
		while (true) {
			for (E a : data) {
				if (GeoUtils.manhattanDistance(a, originX, originY) < range) {
					candidate.add(a);
				}
			}
			if (!candidate.isEmpty()) {
				int need = maxselects - ret.size();
				if (candidate.size() <= need) {
					ret.addAll(candidate);
				} else {
					ret.addAll(randomSelect(candidate, need, rand));
				}
			}
			if (ret.size() == maxselects) {
				break;
			}
			range *= MULTI_SCALE;
		}
		return ret;
	}

	public static <E extends Area> E multirangeAreaSelect(final Set<E> data,
			int originX, int originY, Random rand, int initRange) {
		if (data.size() == 0)
			return null;
		double range = initRange;
		Set<E> candidate = new HashSet<E>();
		while (true) {
			for (E a : data) {
				if (GeoUtils.manhattanDistance(a, originX, originY) < range) {
					candidate.add(a);
				}
			}
			if (!candidate.isEmpty()) {
				return randomSelect(candidate, 1, rand).iterator().next();
			}
			range *= MULTI_SCALE;
		}
	}

	/**
	 * @param queries
	 *            areaidset
	 * @return null if queries is empty
	 */
	public static EntityID closestSelect(int x, int y, Set<EntityID> queries,
			StandardWorldModel world) {
		if (queries.isEmpty())
			return null;
		EntityID pick = null;
		double min = Double.MAX_VALUE;
		for (EntityID id : queries) {
			try {
				Area t = (Area) world.getEntity(id);
				double dist = GeoUtils.manhattanDistance(t, x, y);
				if (dist < min) {
					min = dist;
					pick = id;
				}
			} catch (Exception e) {
			}
		}
		return pick;
	}
}
