package seu.utils.geo;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Point2D;
import rescuecore2.standard.entities.Area;

public class GeoUtils {

	public static double manhattanDistance(Area a1, Area a2) {
		return Math.abs(a1.getX() - a2.getX())
				+ Math.abs(a1.getY() - a2.getY());
	}

	public static double manhattanDistance(Area a, int x2, int y2) {
		return Math.abs(a.getX() - x2) + Math.abs(a.getY() - y2);
	}

	public static double manhattanDistance(Area a, double x2, double y2) {
		return Math.abs(a.getX() - x2) + Math.abs(a.getY() - y2);
	}

	public static double manhattanDistance(IPositioning a, int x2, int y2) {
		return Math.abs(a.getX() - x2) + Math.abs(a.getY() - y2);
	}

	public static double manhattanDistance(int x1, int y1, int x2, int y2) {
		return Math.abs(x1 - x2) + Math.abs(y1 - y2);
	}

	// public static double manhattanDistance(Point p1, Point p2) {
	// return Math.abs(p1.x() - p2.x()) + Math.abs(p1.y() - p2.y());
	// }

	public static double manhattanDistance(IPositioning o1, IPositioning o2) {
		return Math.abs(o1.getX() - o2.getX())
				+ Math.abs(o1.getY() - o2.getY());
	}

	public static double manhattanDistance(IPositioning o1, Area o2) {
		return Math.abs(o1.getX() - o2.getX())
				+ Math.abs(o1.getY() - o2.getY());
	}

	// public static double manhattanDistance(IPositioning o1, Point o2) {
	// return Math.abs(o1.getX() - o2.x()) + Math.abs(o1.getY() - o2.y());
	// }

	/**
	 * p1-p2 intersect p3-p4? return true if intersect
	 */
	public static boolean isSegmentIntersect(int p1x, int p1y, int p2x,
			int p2y, int p3x, int p3y, int p4x, int p4y) {
		long d1 = cross(p3x, p3y, p4x, p4y, p1x, p1y);
		long d2 = cross(p3x, p3y, p4x, p4y, p2x, p2y);
		long d3 = cross(p1x, p1y, p2x, p2y, p3x, p3y);
		long d4 = cross(p1x, p1y, p2x, p2y, p4x, p4y);

		if (((d1 > 0 && d2 < 0) || (d1 < 0 && d2 > 0))
				&& ((d3 > 0 && d4 < 0) || (d3 < 0 && d4 > 0)))
			return true;

		if (d1 == 0 && onSegment(p3x, p3y, p4x, p4y, p1x, p1y))
			return true;
		if (d2 == 0 && onSegment(p3x, p3y, p4x, p4y, p2x, p2y))
			return true;
		if (d3 == 0 && onSegment(p1x, p1y, p2x, p2y, p3x, p3y))
			return true;
		if (d4 == 0 && onSegment(p1x, p1y, p2x, p2y, p4x, p4y))
			return true;

		return false;
	}

	public static boolean isSegmentIntersect(Segment segment1, Segment segment2) {
		return isSegmentIntersect(segment1.getX1(), segment1.getY1(),
				segment1.getX2(), segment1.getY2(), segment2.getX1(),
				segment2.getY1(), segment2.getX2(), segment2.getY2());
	}

	/**
	 * pk on pi-pj when cross = 0 return true if onSeg
	 */
	private static boolean onSegment(int pix, int piy, int pjx, int pjy,
			int pkx, int pky) {
		return ((pkx <= pix && pkx >= pjx) || (pkx >= pix && pkx <= pjx))
				&& ((pky <= piy && pky >= pjy) || (pky >= piy && pky <= pjy));
	}

	/**
	 * Vector.pipk cross Vector.pipj <br>
	 * p1 X p2 > 0, p1 在 p2 的顺时针方向
	 * 
	 * @param pi
	 * @param pj
	 * @param pk
	 * @return ik X ij
	 */
	public static long cross(int pix, int piy, int pjx, int pjy, int pkx,
			int pky) {
		// return (pk - pi) * (pj - pi)
		// pk.x - pi.x, pk.y - pi.y
		// pj.x - pi.x, pj.y - pi.y
		long p = (long) pkx - pix;
		long q = (long) pjy - piy;
		long r = (long) pky - piy;
		long s = (long) pjx - pix;

		return p * q - r * s;
	}

	/**
	 * get intersection point of line1 and line2
	 * 
	 * @param l1
	 * @param l2
	 * @return returns intersection point if not point returns null
	 */
	public static Point getIntersectionPoint(Line l1, Line l2) {
		double t1 = intersectionPos(l1, l2);
		double t2 = intersectionPos(l2, l1);
		if (Double.isNaN(t1) || Double.isNaN(t2)) {
			return null;
		}
		return l1.getPoint(t1);
	}

	public static Point getSegmentIntersectionPoint(Segment s1, Segment s2) {
		Line l1 = new Line(s1.getOrigin(), s1.getEnd());
		Line l2 = new Line(s2.getOrigin(), s2.getEnd());
		double t1 = intersectionPos(l1, l2);
		double t2 = intersectionPos(l2, l1);
		if (Double.isNaN(t1) || Double.isNaN(t2) || t1 < 0 || t1 > 1 || t2 < 0
				|| t2 > 1) {
			return null;
		}
		return l1.getPoint(t1);
	}

	private static double intersectionPos(Line base, Line other) {
		double bxax = (base.getEnd().x() - base.getOrigin().x());
		double dycy = (other.getEnd().y() - other.getOrigin().y());
		double byay = (base.getEnd().y() - base.getOrigin().y());
		double dxcx = (other.getEnd().x() - other.getOrigin().x());
		double cxax = other.getOrigin().x() - base.getOrigin().x();
		double cyay = other.getOrigin().y() - base.getOrigin().y();
		double d = (bxax * dycy) - (byay * dxcx);
		double t = (cxax * dycy) - (cyay * dxcx);

		if (d > -0.0000000000001 && d < 0.0000000000001) {
			return Double.NaN;
		}

		return t / d;
	}

	public static double distSquare(int p1x, int p1y, int p2x, int p2y) {
		double a = p1x - p2x;
		double b = p1y - p2y;
		return a * a + b * b;
	}

	public static double distSquare(Point p1, Point p2) {
		return distSquare(p1.x(), p1.y(), p2.x(), p2.y());
	}

	public static long distSquareL(Point p1, Point p2) {
		long a = p1.x() - p2.x();
		long b = p1.y() - p2.y();
		return a * a + b * b;
	}

	public static double dist(int p1x, int p1y, int p2x, int p2y) {
		int a = p1x - p2x;
		int b = p1y - p2y;
		return Math.hypot(a, b);
	}

	public static double dist(Point p1, Point p2) {
		return dist(p1.x(), p1.y(), p2.x(), p2.y());
	}

	public static boolean isOnSegment(int px, int py, int s1x, int s1y,
			int s2x, int s2y) {
		long d1 = cross(s1x, s1y, s2x, s2y, px, py);
		if (d1 == 0 && onSegment(s1x, s1y, s2x, s2y, px, py)) {
			return true;
		}
		return false;
	}

	/**
	 * node n1, n2 on the same side of line p1-p2
	 * 
	 * @return
	 */
	public static boolean onSameSide(int p1x, int p1y, int p2x, int p2y,
			int n1x, int n1y, int n2x, int n2y) {
		long s1 = linefunc(p1x, p1y, p2x, p2y, n1x, n1y);
		long s2 = linefunc(p1x, p1y, p2x, p2y, n2x, n2y);
		if (s1 >= 0 && s2 >= 0) {
			return true;
		}
		if (s1 <= 0 && s2 <= 0) {
			return true;
		}
		return false;
	}

	public static boolean onSameSide(Point origin, Point end, Point p1, Point p2) {
		return onSameSide(origin.x(), origin.y(), end.x(), end.y(), p1.x(),
				p1.y(), p2.x(), p2.y());
	}

	/**
	 * 
	 * @param origin
	 * @param end
	 * @param p
	 * @return Left ret >0, Right ret <0
	 */
	public static long onWhichSide(Point origin, Point end, Point p) {
		long ret = linefunc(origin.x(), origin.y(), end.x(), end.y(), p.x(),
				p.y());
		return ret;
	}

	/**
	 * relationship between point and segment
	 * 
	 * @param origin
	 * @param end
	 * @param p
	 * @return 0 if p is on right, 1 if p is on left, 2 if p is on segment, 3 if
	 *         p is on origin or end point, 4 if p is not on segment
	 */
	public static int dotSegment(Point origin, Point end, Point p) {
		long tmp = linefunc(origin.x(), origin.y(), end.x(), end.y(), p.x(),
				p.y());
		if (tmp < 0) {
			return 0; // right
		} else if (tmp > 0) {
			return 1; // left
		}
		if (isOnSegment(p.x(), p.y(), origin.x(), origin.y(), end.x(), end.y())) {
			if (p.equals(origin) || p.equals(end)) {
				return 3; // on origin or end
			} else {
				return 2; // on segment
			}

		} else {
			return 4; // not no segment
		}
	}

	/**
	 * if p in the space between l1 and l2
	 * 
	 * @param l1
	 * @param l2
	 * @param p
	 * @return
	 */
	public static boolean betweenTwoLines(Line l1, Line l2, Point p) {
		return betweenTwoLines(l1.getOrigin().x(), l1.getOrigin().y(), l1
				.getEnd().x(), l1.getEnd().y(), l2.getOrigin().x(), l2
				.getOrigin().y(), l2.getEnd().x(), l2.getEnd().y(), p.x(),
				p.y());
	}

	public static boolean betweenTwoLines(int p1x, int p1y, int p2x, int p2y,
			int p3x, int p3y, int p4x, int p4y, int nx, int ny) {
		long s1 = linefunc(p1x, p1y, p2x, p2y, nx, ny);
		long s2 = linefunc(p3x, p3y, p4x, p4y, nx, ny);
		if (s1 >= 0 && s2 <= 0) {
			return true;
		}
		if (s1 <= 0 && s2 >= 0) {
			return true;
		}
		return false;
	}

	/**
	 * if point (x,y) in polygon or on polygon's edge, it returns true
	 * 
	 * @param polygonApexes
	 * @param x
	 * @param y
	 * @return
	 */
	public static boolean inPolygon(int[] polygonApexes, int x, int y) {

		Rectangle r = Converter.generateMinBoundingRectangle(polygonApexes);
		Point p = new Point(x, y);
		if (!r.containsPoint(p)) {
			return false;
		} else {
			int xx = r.getX() + r.getWidth() + 1;
			int yy = r.getY() + r.getHeight() / 2;
			Segment s = new Segment(x, y, xx, yy);
			List<Segment> edges = Converter.generateSegments(polygonApexes);
			int intersection = 0;
			Set<Point> pointsOnS = new HashSet<Point>();
			for (Segment e : edges) {
				// Point on edges
				if (e.contains(p)) {
					return true;
				}
				if (GeoUtils.isSegmentIntersect(s.getX1(), s.getY1(),
						s.getX2(), s.getY2(), e.getX1(), e.getY1(), e.getX2(),
						e.getY2())) {
					if (s.contains(e.getOrigin())) {
						pointsOnS.add(e.getOrigin());
					}
					if (s.contains(e.getEnd())) {
						pointsOnS.add(e.getEnd());
					}
					intersection++;
				}
			}

			for (Point next : pointsOnS) {
				Point p1 = null, p2 = null;
				for (Segment e : edges) {
					if (e.getOrigin().equals(next)) {
						p1 = e.getEnd();
					}
					if (e.getEnd().equals(next)) {
						p2 = e.getOrigin();
					}
				}
				if (!GeoUtils.onSameSide(s.getX1(), s.getY1(), s.getX2(),
						s.getY2(), p1.x(), p1.y(), p2.x(), p2.y())) {
					intersection--;
				}

			}

			if (intersection % 2 == 1) {
				return true;
			}
			return false;
		}
	}

	public static boolean inPolygonNotOnEdge(int[] polygonApexes, int x, int y) {

		Rectangle r = Converter.generateMinBoundingRectangle(polygonApexes);
		Point p = new Point(x, y);
		if (!r.containsPoint(p)) {
			return false;
		} else {
			int xx = r.getX() + r.getWidth() + 1;
			int yy = r.getY() + r.getHeight() / 2;
			Segment s = new Segment(x, y, xx, yy);
			List<Segment> edges = Converter.generateSegments(polygonApexes);
			int intersection = 0;
			Set<Point> pointsOnS = new HashSet<Point>();
			for (Segment e : edges) {
				// Point on edges
				if (e.contains(p)) {
					return false;
				}
				if (GeoUtils.isSegmentIntersect(s.getX1(), s.getY1(),
						s.getX2(), s.getY2(), e.getX1(), e.getY1(), e.getX2(),
						e.getY2())) {
					if (s.contains(e.getOrigin())) {
						pointsOnS.add(e.getOrigin());
					}
					if (s.contains(e.getEnd())) {
						pointsOnS.add(e.getEnd());
					}
					intersection++;
				}
			}

			for (Point next : pointsOnS) {
				Point p1 = null, p2 = null;
				for (Segment e : edges) {
					if (e.getOrigin().equals(next)) {
						p1 = e.getEnd();
					}
					if (e.getEnd().equals(next)) {
						p2 = e.getOrigin();
					}
				}
				if (!GeoUtils.onSameSide(s.getX1(), s.getY1(), s.getX2(),
						s.getY2(), p1.x(), p1.y(), p2.x(), p2.y())) {
					intersection--;
				}

			}

			if (intersection % 2 == 1) {
				return true;
			}
			return false;
		}
	}

	@Deprecated
	public static Point getClosestPointOnSegment(Segment seg, Point point) {
		Point p1 = seg.getOrigin();
		Point p2 = seg.getEnd();
		double u = (((point.x() - p1.x()) * (p2.x() - p1.x())) + ((point.y() - p1
				.y()) * (p2.y() - p1.y()))) / distSquare(p1, p2);
		if (u < 0) {
			return null;
		}
		if (u > 1) {
			return null;
		}
		Line line = new Line(p1, p2);
		return line.getPoint(u);
	}

	/**
	 * Find the closest point on a line.
	 * 
	 * @param line
	 *            The line to check.
	 * @param point
	 *            The point to check against.
	 * @return The point on the line that is closest to the reference point.
	 */
	public static Point2D getClosestPoint(Line2D line, Point2D point) {
		Point2D p1 = line.getOrigin();
		Point2D p2 = line.getEndPoint();
		double u = (((point.getX() - p1.getX()) * (p2.getX() - p1.getX())) + ((point
				.getY() - p1.getY()) * (p2.getY() - p1.getY())))
				/ (line.getDirection().getLength() * line.getDirection()
						.getLength());
		return line.getPoint(u);
	}

	/**
	 * Find the closest point on a line.
	 * 
	 * @param line
	 *            The line to check.
	 * @param point
	 *            The point to check against.
	 * @return The point on the line that is closest to the reference point.
	 */
	public static Point2D getClosestPointOnSegment(Line2D line, Point2D point) {
		Point2D p1 = line.getOrigin();
		Point2D p2 = line.getEndPoint();
		double u = (((point.getX() - p1.getX()) * (p2.getX() - p1.getX())) + ((point
				.getY() - p1.getY()) * (p2.getY() - p1.getY())))
				/ (line.getDirection().getLength() * line.getDirection()
						.getLength());
		if (u < 0) {
			return null;
		}
		if (u > 1) {
			return null;
		}
		return line.getPoint(u);
	}

	private static long linefunc(int p1x, int p1y, int p2x, int p2y, int nx,
			int ny) {
		long ret = (p2y - p1y) * (p2x - nx) - (p2y - ny) * (p2x - p1x);
		return ret;
	}

	/***************************************************************************************************************/

	/**
	 * @return neg, no intersect return p1 to Intersect squareDist
	 */
	public static double getP1toIntersectDistSquare(int p1x, int p1y, int p2x,
			int p2y, int p3x, int p3y, int p4x, int p4y) {
		long d1 = cross(p3x, p3y, p4x, p4y, p1x, p1y);
		long d2 = cross(p3x, p3y, p4x, p4y, p2x, p2y);
		long d3 = cross(p1x, p1y, p2x, p2y, p3x, p3y);
		long d4 = cross(p1x, p1y, p2x, p2y, p4x, p4y);

		if (((d1 > 0 && d2 < 0) || (d1 < 0 && d2 > 0))
				&& ((d3 > 0 && d4 < 0) || (d3 < 0 && d4 > 0))) {
			// d1, d2
			double dist = distSquare(p1x, p1y, p2x, p2y);
			// System.out.println(dist);
			double a = 1.0d * Math.abs(d1) / (Math.abs(d1) + Math.abs(d2));
			double b = a * a * dist;
			return b;
		}

		if (d1 == 0 && onSegment(p3x, p3y, p4x, p4y, p1x, p1y)) {
			// return true;
			return 0;
		}
		if (d2 == 0 && onSegment(p3x, p3y, p4x, p4y, p2x, p2y)) {
			// return true;
			return distSquare(p1x, p1y, p2x, p2y);
		}
		if (d3 == 0 && onSegment(p1x, p1y, p2x, p2y, p3x, p3y)) {
			// return true;
			return distSquare(p1x, p1y, p3x, p3y);
		}
		if (d4 == 0 && onSegment(p1x, p1y, p2x, p2y, p4x, p4y)) {
			// return true;
			return distSquare(p1x, p1y, p4x, p4y);
		}

		return -100;
	}

	/**
	 * 求出line中垂线上的点
	 * 
	 * @param line
	 * @param distance
	 *            该点坐标与line中点坐标的哈密尔顿距离
	 * @param ref
	 *            参考是否与ref点在同一侧
	 * @param onSameSide
	 *            设置需求点是否与ref在line的同一侧
	 * @return 该中垂线上的点
	 */
	public static Point getPerpendicular(Line line, int distance, Point ref,
			boolean onSameSide) {
		Point mid = line.getMidPoint();
		Point rv;
		int dx = line.getOrigin().x() - line.getEnd().x();
		int dy = line.getOrigin().y() - line.getEnd().y();
		int absX = Math.abs(dx);
		int absY = Math.abs(dy);
		if (dx != 0) {
			rv = new Point((int) (mid.x() + 1.0 * distance * dy * absX
					/ (1.0 * dx * (absX + absY))),
					(int) (1.0 * mid.y() - distance * absX
							/ (1.0 * (absX + absY))));
			if (GeoUtils.onSameSide(line.getOrigin(), line.getEnd(), rv, ref) == onSameSide) {
				return rv;
			}

			rv = new Point((int) (mid.x() - 1.0 * distance * dy * absX
					/ (1.0 * dx * (absX + absY))),
					(int) (1.0 * mid.y() + distance * absX
							/ (1.0 * (absX + absY))));
			if (GeoUtils.onSameSide(line.getOrigin(), line.getEnd(), rv, ref) == onSameSide) {
				return rv;
			}

			return null;
		} else {
			if (dy != 0) {
				rv = new Point(mid.x() + distance, mid.y());
				if (GeoUtils.onSameSide(line.getOrigin(), line.getEnd(), rv,
						ref) == onSameSide) {
					return rv;
				}

				rv = new Point(mid.x() - distance, mid.y());
				if (GeoUtils.onSameSide(line.getOrigin(), line.getEnd(), rv,
						ref) == onSameSide) {
					return rv;
				}
				return null;
			} else {
				return null;
			}
		}
	}

	public static boolean nearParallel(Segment seg1, Segment seg2) {
		double d = ((seg1.getX2() - seg1.getX1()) * (seg2.getY2() - seg2
				.getY1()))
				- ((seg1.getY2() - seg1.getY1()) * (seg2.getX2() - seg2.getX1()));
		d /= Math.max(seg1.getLength(), seg2.getLength());
		if (d < 1 && d > -1) {
			return true;
		} else {
			return false;
		}
	}

	public static double getDistanceToLine(Segment line, int x, int y) {
		Point p1 = line.getOrigin();
		int p1x = p1.getX();
		int p1y = p1.getY();
		Point p2 = line.getEnd();
		int p2x = p2.getX();
		int p2y = p2.getY();

		int dx = p2x - p1x;
		int dy = p2y - p1y;
		double length = Math.hypot(dx, dy);
		double u = (((x - p1x) * dx) + ((y - p1y) * dy)) / (length * length);

		return Math.hypot(p1x + u * dx - x, p1y + u * dy - y);
	}

	/**
	 * 切除在多边形内的线段
	 * 
	 * @param seg
	 * @param p
	 * @return
	 */
	public static Segment cutSegmentInsidePolygon(Segment seg, Polygon p) {
		if (GeoUtils.inPolygon(p.getApexes(), seg.getX1(), seg.getY1())) { // start
																			// point
																			// in
																			// polygon
			if (GeoUtils.inPolygon(p.getApexes(), seg.getX2(), seg.getY2())) { // end
																				// point
																				// in
																				// polygon
				return null; // contain
			}
			for (Segment next : p.getEdges()) {
				Point intp = GeoUtils.getSegmentIntersectionPoint(seg, next);
				if (intp == null) {
					continue;
				}
				seg = new Segment(intp, seg.getEnd()); //
			}
		} else if (GeoUtils.inPolygon(p.getApexes(), seg.getX2(), seg.getY2())) { // end
																					// point
																					// in
																					// polygon
			for (Segment next : p.getEdges()) {
				Point intp = GeoUtils.getSegmentIntersectionPoint(seg, next);
				if (intp == null) {
					continue;
				}
				seg = new Segment(seg.getOrigin(), intp);
			}
		}

		return seg;
	}

	public static List<Segment> discretizeSegment(Segment seg, int length) {
		List<Segment> ret = new ArrayList<Segment>();

		int segLength = (int) seg.getLength();

		int num = segLength / length;

		if (num <= 0) {
			return ret;
		}

		int incX = length * (seg.getX2() - seg.getX1()) / segLength;
		int incY = length * (seg.getY2() - seg.getY1()) / segLength;

		int startX = seg.getX1();
		int startY = seg.getY1();

		for (int i = 0; i < num; i++) {
			int endX = startX + incX;
			int endY = startY + incY;
			ret.add(new Segment(startX, startY, endX, endY));
			startX = endX;
			startY = endY;
		}

		// !!!!!!!!!!!!
		ret.add(new Segment(seg.getX2() - incX, seg.getY2() - incY,
				seg.getX2(), seg.getY2()));

		return ret;
	}
}
