package seu.utils.geo;

/**
 * 
 * @author Ivan, Fun
 * 
 */
public class Segment {

	private int x1, y1;
	private int x2, y2;
	private Point origin = null;
	private Point end = null;

	public Segment(int x1, int y1, int x2, int y2) {
		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;
	}
	
	public Segment(Point origin,Point end){
		this.origin=origin;
		this.end=end;
		x1 = origin.x();
		y1 = origin.y();
		x2 = end.x();
		y2 = end.y();
	}

	public int getX1() {
		return x1;
	}

	public int getY1() {
		return y1;
	}

	public int getX2() {
		return x2;
	}

	public int getY2() {
		return y2;
	}

	public void setOrigin(Point origin){
		this.origin=origin;
		this.x1=origin.x();
		this.y1=origin.y();
	}
	
	public void setEnd(Point end){
		this.end=end;
		this.x2=end.x();
		this.y2=end.y();
	}
	
	public Point getOrigin() {
		if (origin == null) {
			origin = new Point(x1, y1);
		}
		return origin;
	}

	public Point getEnd() {
		if (end == null) {
			end = new Point(x2, y2);
		}
		return end;
	}
	
	public int getMidX(){
		return (x1+x2)/2;
	}
	
	public int getMidY(){
		return (y1+y2)/2;
	}
	
	public Point getMidPoint(){
		return new Point(getMidX(),getMidY());
	}
	
	public boolean contains(Point p){
		return GeoUtils.isOnSegment(p.x(), p.y(), x1, y1, x2, y2);
	}
	
	public double getLength(){
		return Math.hypot(x2-x1, y2-y1);
	}

	@Override
	public String toString() {
		return "Segment [(" + x1 + "," + y1 + "), (" + x2 + "," + y2
				+ ")]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + x1;
		result = prime * result + x2;
		result = prime * result + y1;
		result = prime * result + y2;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Segment other = (Segment) obj;
		if (x1 != other.x1)
			return false;
		if (x2 != other.x2)
			return false;
		if (y1 != other.y1)
			return false;
		if (y2 != other.y2)
			return false;
		return true;
	}
	
	public static void main(String [] args){
		Segment s = new Segment(0, 0, 5, 0);
		System.out.println(s.contains(new Point(4,4)));
		System.out.println(s.contains(new Point(4,0)));
		System.out.println(s.contains(new Point(4,-4)));
		System.out.println(s.contains(new Point(0, -1)));
		System.out.println(s.contains(new Point(5,0)));
	}
	
}
