package seu.utils.geo.acc;

import java.util.ArrayList;
import java.util.List;

import seu.utils.geo.Converter;
import seu.utils.geo.GeoUtils;
import seu.utils.geo.Line;
import seu.utils.geo.Point;
import seu.utils.geo.Polygon;

/**
 * 
 * @author Fun
 * 
 */
public class AccUtils {

	public static Polygon expandPolygon(Polygon source, int dist) {
		boolean clockwise = true;
		List<Point> vertixes = source.getVertices();
		long res = 0;
		int i = 0;
		do {
			Point p0 = vertixes.get(i);
			Point p1 = vertixes.get(i + 1);
			Point p2 = vertixes.get(i + 2);
			res = GeoUtils.onWhichSide(p0, p1, p2);
		} while (res == 0);

		if (res < 0) {
			clockwise = true;
		} else {
			clockwise = false;
		}
		return new Polygon(expandPolygon(vertixes, dist, clockwise));
	}

	public static int[] expandPolygon(int[] apexes, int dist, boolean clockwise) {
		List<Point> vertixes = Converter.generatePoints(apexes);
		return expandPolygon(vertixes, dist, clockwise);
	}

	public static int[] expandPolygon(List<Point> vertixes, int dist,
			boolean clockwise) {
		int i = 0;

		ArrayList<Point> rmSet = new ArrayList<Point>();

		for (int j = 0; j < vertixes.size() - 2; j++) {
			Point t0 = vertixes.get(j);
			Point t1 = vertixes.get(j + 1);
			Point t2 = vertixes.get(j + 2);
			if (GeoUtils.isOnSegment(t1.x(), t1.y(), t0.x(), t0.y(), t2.x(),
					t2.y())) {
				rmSet.add(t1);
			}
		}
		Point t0 = vertixes.get(vertixes.size() - 2);
		Point t1 = vertixes.get(vertixes.size() - 1);
		Point t2 = vertixes.get(0);
		if (GeoUtils
				.isOnSegment(t1.x(), t1.y(), t0.x(), t0.y(), t2.x(), t2.y())) {
			rmSet.add(t1);
		}
		t0 = vertixes.get(vertixes.size() - 1);
		t1 = vertixes.get(0);
		t2 = vertixes.get(1);
		if (GeoUtils
				.isOnSegment(t1.x(), t1.y(), t0.x(), t0.y(), t2.x(), t2.y())) {
			rmSet.add(t1);
		}

		vertixes.removeAll(rmSet);

		List<Line> elines = new ArrayList<Line>();
		if (clockwise) {
			for (i = 0; i < vertixes.size() - 1; i++) {
				Point p1 = vertixes.get(i);
				Point p2 = vertixes.get(i + 1);

				int x0 = p2.x() - p1.x();
				int y0 = p2.y() - p1.y();

				Point leftPoint = generateLeftPoint(p1, p2, dist);

				Line line = new Line(leftPoint, new Point(leftPoint.x() + x0,
						leftPoint.y() + y0));

				elines.add(line);
			}
			Point p1 = vertixes.get(vertixes.size() - 1);
			Point p2 = vertixes.get(0);

			int x0 = p2.x() - p1.x();
			int y0 = p2.y() - p1.y();

			Point leftPoint = generateLeftPoint(p1, p2, dist);

			Line line = new Line(leftPoint, new Point(leftPoint.x() + x0,
					leftPoint.y() + y0));

			elines.add(line);
		} else {
			for (i = vertixes.size() - 1; i > 0; i--) {
				Point p1 = vertixes.get(i);
				Point p2 = vertixes.get(i - 1);

				int x0 = p2.x() - p1.x();
				int y0 = p2.y() - p1.y();

				Point leftPoint = generateLeftPoint(p1, p2, dist);

				Line line = new Line(leftPoint, new Point(leftPoint.x() + x0,
						leftPoint.y() + y0));

				elines.add(line);
			}
			Point p1 = vertixes.get(0);
			Point p2 = vertixes.get(vertixes.size() - 1);

			int x0 = p2.x() - p1.x();
			int y0 = p2.y() - p1.y();

			Point leftPoint = generateLeftPoint(p1, p2, dist);

			Line line = new Line(leftPoint, new Point(leftPoint.x() + x0,
					leftPoint.y() + y0));

			elines.add(line);
		}

		// return generateApexes(elines, clockwise);
		return generateApexesMod(vertixes, elines, clockwise, dist);
	}

	private static Point generateLeftPoint(Point p1, Point p2, int dist) {
		int x0 = p2.x() - p1.x();
		int y0 = p2.y() - p1.y();

		int x1 = -y0;
		int y1 = x0;

		int x2 = y0;
		int y2 = -x0;

		Point n1 = new Point(p1.x() + x1, p1.y() + y1);

		Point leftPoint = null;

		if (GeoUtils.onWhichSide(p1, p2, n1) > 0) {
			double len = Math.sqrt(x1 * x1 + y1 * y1);
			leftPoint = new Point(p1.x() + (int) (x1 * dist / len), p1.y()
					+ (int) (y1 * dist / len));
		} else {
			double len = Math.sqrt(x2 * x2 + y2 * y2);
			leftPoint = new Point(p1.x() + (int) (x2 * dist / len), p1.y()
					+ (int) (y2 * dist / len));
		}

		return leftPoint;
	}

	@Deprecated
	private static int[] generateApexes(List<Line> elines, boolean clockwise) {
		ArrayList<Integer> la = new ArrayList<Integer>();
		if (clockwise) {
			for (int i = 0; i < elines.size() - 1; i++) {
				Point p = GeoUtils.getIntersectionPoint(elines.get(i),
						elines.get(i + 1));
				if (p == null) {
				}
				la.add(p.x());
				la.add(p.y());
			}

			Point p = GeoUtils.getIntersectionPoint(
					elines.get(elines.size() - 1), elines.get(0));
			la.add(p.x());
			la.add(p.y());
		} else {
			for (int i = elines.size() - 1; i > 0; i--) {
				Point p = GeoUtils.getIntersectionPoint(elines.get(i),
						elines.get(i - 1));
				if (p == null) {
				}
				la.add(p.x());
				la.add(p.y());
			}
			Point p = GeoUtils.getIntersectionPoint(elines.get(0),
					elines.get(elines.size() - 1));
			la.add(p.x());
			la.add(p.y());
		}
		int[] apexes = new int[la.size()];
		for (int i = 0; i < la.size(); i++) {
			apexes[i] = la.get(i);
		}
		return apexes;
	}

	private static int[] generateApexesMod(List<Point> vertixes,
			List<Line> elines, boolean clockwise, int dist) {
		ArrayList<Integer> la = new ArrayList<Integer>();
		long r2 = 4 * ((long) dist * (long) dist);
		if (clockwise) {
			for (int i = 0; i < elines.size() - 1; i++) {
				Line line1 = elines.get(i);
				Line line2 = elines.get(i + 1);
				Point p = GeoUtils.getIntersectionPoint(line1, line2);
				if (p == null) {
					la.add(line1.getEnd().getX());
					la.add(line1.getEnd().getY());
					la.add(line2.getOrigin().getX());
					la.add(line2.getOrigin().getY());
				} else {
					Point center = vertixes.get(i + 1);
					long dist2 = GeoUtils.distSquareL(center, p);
					if (r2 >= dist2) {
						la.add(p.x());
						la.add(p.y());
					} else {
						la.add(line1.getEnd().getX());
						la.add(line1.getEnd().getY());
						la.add(line2.getOrigin().getX());
						la.add(line2.getOrigin().getY());
					}
				}

			}

			Line line1 = elines.get(elines.size() - 1);
			Line line2 = elines.get(0);
			Point p = GeoUtils.getIntersectionPoint(line1, line2);
			if (p == null) {
				la.add(line1.getEnd().getX());
				la.add(line1.getEnd().getY());
				la.add(line2.getOrigin().getX());
				la.add(line2.getOrigin().getY());
			} else {
				Point center = vertixes.get(0);
				long dist2 = GeoUtils.distSquareL(center, p);
				if (r2 >= dist2) {
					la.add(p.x());
					la.add(p.y());
				} else {
					la.add(line1.getEnd().getX());
					la.add(line1.getEnd().getY());
					la.add(line2.getOrigin().getX());
					la.add(line2.getOrigin().getY());
				}
			}

		} else {
			for (int i = elines.size() - 1; i > 0; i--) {
				Line line1 = elines.get(i);
				Line line2 = elines.get(i - 1);
				Point p = GeoUtils.getIntersectionPoint(line1, line2);
				if (p == null) {
					la.add(line1.getEnd().getX());
					la.add(line1.getEnd().getY());
					la.add(line2.getOrigin().getX());
					la.add(line2.getOrigin().getY());
				} else {
					Point center = vertixes.get(i - 1);
					long dist2 = GeoUtils.distSquareL(center, p);
					if (r2 >= dist2) {
						la.add(p.x());
						la.add(p.y());
					} else {
						la.add(line1.getEnd().getX());
						la.add(line1.getEnd().getY());
						la.add(line2.getOrigin().getX());
						la.add(line2.getOrigin().getY());
					}
				}
			}
			Line line1 = elines.get(0);
			Line line2 = elines.get(elines.size() - 1);
			Point p = GeoUtils.getIntersectionPoint(line1, line2);
			if (p == null) {
				la.add(line1.getEnd().getX());
				la.add(line1.getEnd().getY());
				la.add(line2.getOrigin().getX());
				la.add(line2.getOrigin().getY());
			} else {
				Point center = vertixes.get(0);
				long dist2 = GeoUtils.distSquareL(center, p);
				if (r2 >= dist2) {
					la.add(p.x());
					la.add(p.y());
				} else {
					la.add(line1.getEnd().getX());
					la.add(line1.getEnd().getY());
					la.add(line2.getOrigin().getX());
					la.add(line2.getOrigin().getY());
				}
			}
		}

		int[] apexes = new int[la.size()];
		for (int i = 0; i < la.size(); i++) {
			apexes[i] = la.get(i);
		}
		return apexes;
	}

}
