package seu.utils.geo.acc;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import seu.utils.geo.GeoUtils;
import seu.utils.geo.Line;
import seu.utils.geo.Point;
import seu.utils.geo.Polygon;
import seu.utils.geo.Segment;

/**
 * 
 * @author Fun
 * 
 */
public class VGraph {

	public static boolean isAccessible(Segment start, Segment target,
			List<Polygon> blockadeslist, Polygon boundary, int exdist) {
		List<Polygon> blockades = new ArrayList<Polygon>();

		for (Polygon blockade : blockadeslist) {
			blockades.add(AccUtils.expandPolygon(blockade, exdist));
		}

		Set<Point> starts = new HashSet<Point>();
		Set<Point> ends = new HashSet<Point>();
		boolean noIntersectStart = true;
		boolean noIntersectTarget = true;
		for (Polygon blockade : blockades) {
			for (Segment seg : blockade.getEdges()) {
				if (GeoUtils.isSegmentIntersect(seg, start)) {
					noIntersectStart = false;
					Point startPoint = GeoUtils.getIntersectionPoint(new Line(
							seg.getOrigin(), seg.getEnd()),
							new Line(start.getOrigin(), start.getEnd()));
					if (startPoint != null) {
						for (Polygon other : blockades) {
							if (other.equals(blockade)) {
								continue;
							}
							if (GeoUtils.inPolygon(other.getApexes(),
									startPoint.getX(), startPoint.getY())) {
								startPoint = null;
								break;
							}
						}
						if (startPoint != null) {
							starts.add(startPoint);
						}
					}
				}
				if (GeoUtils.isSegmentIntersect(seg, target)) {
					noIntersectTarget = false;
					Point endPoint = GeoUtils.getIntersectionPoint(
							new Line(seg.getOrigin(), seg.getEnd()), new Line(
									target.getOrigin(), target.getEnd()));
					if (endPoint != null) {
						for (Polygon other : blockades) {
							if (other.equals(blockade)) {
								continue;
							}
							if (GeoUtils.inPolygon(other.getApexes(),
									endPoint.getX(), endPoint.getY())) {
								endPoint = null;
								break;
							}
						}
						if (endPoint != null) {
							ends.add(endPoint);
						}
					}
				}

			}
		}

		if (noIntersectStart) {
			boolean startInBlockade = false;
			for (Polygon blockade : blockades) {
				if (GeoUtils.inPolygon(blockade.getApexes(), start.getX1(),
						start.getY1())) {
					startInBlockade = true;
					break;
				}
			}
			if (startInBlockade) {
				return false;
			}
			starts.add(new Point((start.getX1() + start.getX2()) / 2, (start
					.getY1() + start.getY2()) / 2));
		}
		if (noIntersectTarget) {
			boolean targetInBlockade = false;
			for (Polygon blockade : blockades) {
				if (GeoUtils.inPolygon(blockade.getApexes(), target.getX1(),
						target.getY1())) {
					targetInBlockade = true;
				}
			}
			if (targetInBlockade) {
				return false;
			}
			ends.add(new Point((target.getX1() + target.getX2()) / 2, (target
					.getY1() + target.getY2()) / 2));
		}

		if (starts.size() == 0 || ends.size() == 0) {
			return false;
		}
		Iterator<Point> sit = starts.iterator();
		Iterator<Point> eit = ends.iterator();
		while (sit.hasNext()) {
			Point p1 = sit.next();
			while (eit.hasNext()) {
				Point p2 = eit.next();
				if (isAccessible(p1, p2, blockades, boundary, false)) {
					return true;
				}
			}
		}
		return false;
	}

	public static boolean isAccessible(Point start, Point target,
			List<Polygon> blockadeslist, Polygon boundary, int exdist) {
		List<Polygon> blockades = new ArrayList<Polygon>();

		for (Polygon blockade : blockadeslist) {
			blockades.add(AccUtils.expandPolygon(blockade, exdist));
		}
		return isAccessible(start, target, blockades, boundary, true);
	}

	private static boolean isAccessible(Point start, Point target,
			List<Polygon> blockades, Polygon boundary, boolean check) {

		if (check) {
			for (Polygon blockade : blockades) {
				if (GeoUtils.inPolygonNotOnEdge(blockade.getApexes(),
						start.getX(), start.getY())) {
					return false;
				}
				if (GeoUtils.inPolygonNotOnEdge(blockade.getApexes(),
						target.getX(), target.getY())) {
					return false;
				}
			}
		}

		HashSet<VBPair> unaccessedSet = new HashSet<VBPair>();

		for (Polygon blockade : blockades) {
			List<Point> vertixes = blockade.getVertices();
			for (Point point : vertixes) {
				unaccessedSet.add(new VBPair(point, blockade));
			}
		}

		Iterator<VBPair> it = unaccessedSet.iterator();
		while (it.hasNext()) {
			VBPair next = it.next();
			if (!GeoUtils.inPolygon(boundary.getApexes(), next.vertex().x(),
					next.vertex().y())) {
				it.remove();
			}
		}

		// remove the vertex in other blockades
		it = unaccessedSet.iterator();
		while (it.hasNext()) {
			VBPair next = it.next();
			Polygon self = next.blokade();
			for (Polygon blockade : blockades) {
				if (blockade.equals(self)) {
					continue;
				}
				if (GeoUtils.inPolygon(blockade.getApexes(), next.vertex().x(),
						next.vertex().y())) {
					it.remove();
				}
			}
		}

		VBPair targetPair = new VBPair(target, null);

		it = unaccessedSet.iterator();
		while (it.hasNext()) {
			VBPair next = it.next();
		}

		unaccessedSet.add(targetPair);

		ArrayList<VBPair> toExpand = new ArrayList<VBPair>();
		toExpand.add(new VBPair(start, null));

		while ((!toExpand.contains(targetPair)) && (!toExpand.isEmpty())) {
			VBPair next = toExpand.get(0);
			if (next.blokade() == null) { // start point
				for (Polygon blockade : blockades) {
					if (GeoUtils.inPolygon(blockade.getApexes(), next.vertex()
							.getX(), next.vertex().getY())) {
						next.setBlockade(blockade);
						break;
					}
				}
			}
			Iterator<VBPair> iterator = unaccessedSet.iterator();
			while (iterator.hasNext()) {
				VBPair checkPoint = iterator.next();

				Polygon blockade = checkPoint.blokade();
				boolean onBlockade = false;
				boolean onBlockadePass = false;

				if (blockade == null) { // end point
					for (Polygon bn : blockades) {
						if (GeoUtils.inPolygon(bn.getApexes(), checkPoint
								.vertex().getX(), checkPoint.vertex().getY())) {
							checkPoint.setBlockade(bn);
							blockade = checkPoint.blokade();
							break;
						}
					}
				}

				if (blockade != null) {
					// if check point is vertex of this blockade
					if (GeoUtils.inPolygon(blockade.getApexes(), next.vertex()
							.getX(), next.vertex().getY())) {
						onBlockade = true;
					}

					List<Segment> edges = blockade.getEdges();
					for (Segment edge : edges) {

						if (edge.contains(next.vertex())
								|| edge.contains(checkPoint.vertex())) {
							if (GeoUtils.nearParallel(
									edge,
									new Segment(next.vertex(), checkPoint
											.vertex()))) {
								onBlockadePass = true;
								break;
							}

						}
					}
				}

				// other blockades or vertex
				Segment toCheck = new Segment(next.vertex(),
						checkPoint.vertex());

				boolean passable = true;
				for (Polygon other : blockades) {
					if (other.equals(blockade)) {
						continue;
					}
					if (other.equals(next.blokade())) {
						continue;
					}
					List<Segment> otherEdges = other.getEdges();
					for (Segment blockline : otherEdges) {
						if (GeoUtils.isSegmentIntersect(blockline, toCheck)) {

							if (blockline.getOrigin().equals(
									toCheck.getOrigin())
									|| blockline.getEnd().equals(
											toCheck.getOrigin())
									|| blockline.getOrigin().equals(
											toCheck.getEnd())
									|| blockline.getEnd().equals(
											toCheck.getEnd())) {
								continue;
							}
							if (blockline.contains(toCheck.getOrigin())
									&& blockline.contains(toCheck.getEnd())) {
								continue;
							}
							passable = false;
							break;
						}

					}
				}
				// System.out.println("passable:"+passable+" onBlockade:"+onBlockade+" onBlockadePass:"+onBlockadePass);
				if (passable) {
					if (!onBlockade) {
						toExpand.add(checkPoint);
						iterator.remove();
					} else {
						if (onBlockadePass) {
							toExpand.add(checkPoint);
							iterator.remove();
						}
					}
				}
			}

			toExpand.remove(next);
		}

		if (toExpand.contains(targetPair)) {
			return true;
		} else {
			return false;
		}

	}

	// public static void main(String[] args) {
	// int[] boundApexes = { 0, 0, 100, 0, 100, 100, 0, 100 };
	// Polygon boundary = new Polygon(boundApexes);
	// System.out.println(boundary);
	// List<Polygon> blockades = new ArrayList<Polygon>();
	// int[] blockade1 =
	// // { 100, 100, 560, -1, 900, 100, 900, 900, 560, 1001, 100, 900 };
	// {0,0,10,0,10,100,0,100};
	// blockades.add(AccUtils.expandPolygon(new Polygon(blockade1), 5));
	// System.out.println(new Polygon(blockade1));
	// Point start = new Point(50, 50);
	// Point target = new Point(0, 50);
	//
	// GeoFrame frame =new GeoFrame("Main");
	// frame.setSize(600, 600);
	// frame.setStartPoint(-100, -100);
	// frame.fillPolygon(boundary.getApexes());
	//
	// for (Polygon b : blockades) {
	// frame.fillPolygon(b.getApexes(),Color.black);
	// }
	//
	// frame.addPoint(start, Color.green);
	// if(VGraph.isAccessible(start, target, blockades,boundary,0)){
	// frame.addPoint(target,Color.blue);
	// }else{
	// frame.addPoint(target);
	// }
	// }

}
