package seu.utils.geo;

import java.awt.Polygon;

import java.util.ArrayList;
import java.util.List;

import rescuecore2.misc.geometry.GeometryTools2D;
import rescuecore2.misc.geometry.Line2D;
import rescuecore2.misc.geometry.Vector2D;


public class mathTools {
//	返回线段，长度为 =原长度+size
    public static rescuecore2.misc.geometry.Line2D improveLine(rescuecore2.misc.geometry.Line2D line, double size) {
        double x0 = line.getOrigin().getX(), x1 = line.getEndPoint().getX();
        double y0 = line.getOrigin().getY(), y1 = line.getEndPoint().getY();
        double deltaY = y1 - y0;
        double deltaX = x1 - x0;
        double xF, yF;
        double slope;
        if (deltaX != 0)
            slope = deltaY / deltaX;
        else {
            if (deltaY > 0)
                slope = Double.MAX_VALUE;
            else
                slope = -Double.MAX_VALUE;
        }

        double theta = Math.atan(slope);
        if (deltaX > 0) {
            xF = x1 + size * Math.abs(Math.cos(theta));
        } else {
            xF = x1 - size * Math.abs(Math.cos(theta));
        }
        if (deltaY > 0) {
            yF = y1 + size * Math.abs(Math.sin(theta));
        } else {
            yF = y1 - size * Math.abs(Math.sin(theta));
        }
        rescuecore2.misc.geometry.Point2D endPoint = new rescuecore2.misc.geometry.Point2D(xF, yF);
        return new rescuecore2.misc.geometry.Line2D(line.getOrigin(), endPoint);
    }
    
 //不改变line的起始点，修剪线段，返回长度为size的Line2D   
    public static rescuecore2.misc.geometry.Line2D clipLine(rescuecore2.misc.geometry.Line2D line, double size) {
        double length = mathTools.getLineLength(line);
        return improveLine(line, size - length);
    }
    
    public static double getLineLength(rescuecore2.misc.geometry.Line2D line) {
      return GeometryTools2D.getDistance(line.getOrigin(), line.getEndPoint());
    }
    
    public static rescuecore2.misc.geometry.Point2D getMiddle(rescuecore2.misc.geometry.Point2D start, rescuecore2.misc.geometry.Point2D end) {

        double cx = ((start.getX() + end.getX()) / 2);
        double cy = ((start.getY() + end.getY()) / 2);

        return new rescuecore2.misc.geometry.Point2D(cx, cy);

    }

    public static rescuecore2.misc.geometry.Point2D getMiddle(rescuecore2.misc.geometry.Line2D line) {
        return getMiddle(line.getOrigin(), line.getEndPoint());
    }
    public static Polygon getPolygon(int[] apexes) {
        Polygon polygon = new Polygon();
        for (int i = 0; i < apexes.length; i += 2) {
            polygon.addPoint(apexes[i], apexes[i + 1]);
        }
        return polygon;
    }
    
    public static List<rescuecore2.misc.geometry.Line2D> getLines(Polygon polygon) {
        List<rescuecore2.misc.geometry.Line2D> lines = new ArrayList<rescuecore2.misc.geometry.Line2D>();
        int count = polygon.npoints;
        for (int i = 0; i < count; i++) {
            int j = (i + 1) % count;
            rescuecore2.misc.geometry.Point2D p1 = new rescuecore2.misc.geometry.Point2D(polygon.xpoints[i], polygon.ypoints[i]);
            rescuecore2.misc.geometry.Point2D p2 = new rescuecore2.misc.geometry.Point2D(polygon.xpoints[j], polygon.ypoints[j]);
            rescuecore2.misc.geometry.Line2D line = new rescuecore2.misc.geometry.Line2D(p1, p2);
            lines.add(line);
        }
        return lines;
    }
    
    public static List<rescuecore2.misc.geometry.Point2D> intersections(Polygon polygon, rescuecore2.misc.geometry.Line2D line) {
        List<rescuecore2.misc.geometry.Point2D> intersections = new ArrayList<rescuecore2.misc.geometry.Point2D>();
        List<rescuecore2.misc.geometry.Line2D> polyLines = getLines(polygon);

        for (rescuecore2.misc.geometry.Line2D ln : polyLines) {
            rescuecore2.misc.geometry.Point2D intersectPoint = GeometryTools2D.getSegmentIntersectionPoint(line, ln);
            if (intersectPoint!=null) {
                intersections.add(intersectPoint);
            }
        }
        return intersections;
    }
    
    //获得清障矩形，
    public static Polygon clearAreaRectangle(double agentX, double agentY, double destinationX, double destinationY, double clearRad) {
        int clearLength = (int) Math.hypot(agentX - destinationX, agentY - destinationY);
        Vector2D agentToTarget = new Vector2D(destinationX - agentX, destinationY
                - agentY);

        if (agentToTarget.getLength() > clearLength)
            agentToTarget = agentToTarget.normalised().scale(clearLength);

        Vector2D backAgent = (new Vector2D(agentX, agentY))
                .add(agentToTarget.normalised().scale(-clearRad));
        rescuecore2.misc.geometry.Line2D line = new rescuecore2.misc.geometry.Line2D(backAgent.getX(), backAgent.getY(),
                agentToTarget.getX(), agentToTarget.getY());

        Vector2D dir = agentToTarget.normalised().scale(clearRad);
        Vector2D perpend1 = new Vector2D(-dir.getY(), dir.getX());
        Vector2D perpend2 = new Vector2D(dir.getY(), -dir.getX());

        rescuecore2.misc.geometry.Point2D points[] = new rescuecore2.misc.geometry.Point2D[]{
                line.getOrigin().plus(perpend1),
                line.getEndPoint().plus(perpend1),
                line.getEndPoint().plus(perpend2),
                line.getOrigin().plus(perpend2)};
        int[] xPoints = new int[points.length];
        int[] yPoints = new int[points.length];
        for (int i = 0; i < points.length; i++) {
            xPoints[i] = (int) points[i].getX();
            yPoints[i] = (int) points[i].getY();
        }
        return new Polygon(xPoints, yPoints, points.length);
    }
//    获得清障矩形的两条长度边界线
    public static List<Line2D> clearAreaLine(double agentX, double agentY, double destinationX, double destinationY, double clearRad) {
        int clearLength = (int) Math.hypot(agentX - destinationX, agentY - destinationY);
        Vector2D agentToTarget = new Vector2D(destinationX - agentX, destinationY
                - agentY);

//        if (agentToTarget.getLength() > clearLength)
            agentToTarget = agentToTarget.normalised().scale(clearLength*0.8);

        Vector2D backAgent = (new Vector2D(agentX, agentY))
                .add(agentToTarget.normalised().scale(-clearRad));
        rescuecore2.misc.geometry.Line2D line = new rescuecore2.misc.geometry.Line2D(backAgent.getX(), backAgent.getY(),
                agentToTarget.getX(), agentToTarget.getY());

        Vector2D dir = agentToTarget.normalised().scale(clearRad);
        Vector2D perpend1 = new Vector2D(-dir.getY(), dir.getX());
        Vector2D perpend2 = new Vector2D(dir.getY(), -dir.getX());
        rescuecore2.misc.geometry.Point2D points[] = new rescuecore2.misc.geometry.Point2D[]{
                line.getOrigin().plus(perpend1),
                line.getEndPoint().plus(perpend1),
                line.getEndPoint().plus(perpend2),
                line.getOrigin().plus(perpend2)};
        Line2D line1 = new Line2D(points[0],points[1]);
        Line2D line2 = new Line2D(points[2],points[3]);
        List<Line2D> clearLine = new ArrayList();
        clearLine.add(line1);
        clearLine.add(line2);
        return clearLine;
        }

    

}
